#ifndef _DISCAMB_EXAMPLES_REFINE_TARGET_H_
#define _DISCAMB_EXAMPLES_REFINE_TARGET_H_

#include "discamb/Scattering/HansenCoppensStructureFactorCalculator.h"
#include "discamb/HC_Model/XdLocalCoordinateSystem.h"



class Target{
public:
    enum parameter_type { occ_parameter, xyz_parameter, adp_parameter };
    struct Parameter {
        size_t atomIndex;
        parameter_type type;
        size_t componentIndex;
    };
    Target(discamb::HansenCoppensStructureFactorCalculator *engine,
           const discamb::Crystal &crystal,
           const std::vector<discamb::XdLocalCoordinateSystem> &lcs = std::vector<discamb::XdLocalCoordinateSystem>());
    Target(discamb::HansenCoppensStructureFactorCalculator *engine,
           const discamb::Crystal &crystal,
           const std::string &optimizationOptionsFile,
           const std::vector<discamb::XdLocalCoordinateSystem> &lcs = std::vector<discamb::XdLocalCoordinateSystem>());
    ~Target();

    //void setParameters();
    void setObservationsAndInit(const std::vector<discamb::Vector3i> hkl, const std::vector<double> &intensities, const std::vector<double> &sigma);

    void onlyAdps();
    void onlyXyz();
    void onlyXyzAndAdps();
    double calc_wR2_square();

    double targetAndDeriveatives(const std::vector<double> &parameters, std::vector<double> &derivatives);
    void parameterizeCrystal(const std::vector<double> &parameters, discamb::Crystal &crystal);
    void getParameters(std::vector<double> &parameters, const discamb::Crystal &crystal);
    void calculateWeights(std::vector<double> &weights);
    /** Calculate structure factors using the last parameterization of mCrystal*/
    void calculateSF(std::vector<std::complex<double> > &sf);

    /**
    set parameters for the weighting scheme
    w = 1 / [ sigma2(Fo2) + (aP)2 + bP ]
    where P = [ 2Fc2 + Max(Fo2,0) ] / 3
    */
    void setWeightParameters(double a, double b);
private:
    std::vector<Parameter> mParameters;
    size_t mN_Param;
    // mOcc2Param, mXyz2Param, mXyz2Param takes value -1 if fixed
    std::vector<int> mOcc2Param;
    std::vector<std::vector<int> > mXyz2Param;
    std::vector<std::vector<int> > mAdp2Param;
    // 
    std::vector<discamb::Vector3i> mHkl;
    std::vector<double> mObservedIntensities;
    std::vector<double> mIntensitiesSigma;
    /**
    parameters for the weighting scheme
    w = 1 / [ sigma2(Fo2) + (aP)2 + bP ]
    where P = [ 2Fc2 + Max(Fo2,0) ] / 3
    */

    double mA,mB;
    std::vector<std::complex<double> > mF_calc; 
    std::vector<double> mWeights;
    std::vector<discamb::XdLocalCoordinateSystem> mLcs;
    std::vector<discamb::Matrix3d> mLcsMatrices;
    discamb::HansenCoppensStructureFactorCalculator *mEngine;
    discamb::Crystal mCrystal;
    double mScaleFactor;
    void updateLcs(const discamb::Crystal &crystal);
    double calculateScaleFactor();
    void calculate_dTargetdF(std::vector<std::complex<double> > &dTargetdF);
    // sum over hkl weight[i]*I_obs[i]*I_obs[i] - the denominator in wR2
    double calc_wR2_denominator();
    double calc_wR2_numerator();

    void initialize(discamb::HansenCoppensStructureFactorCalculator *engine, const discamb::Crystal &crystal,
                    const std::vector<discamb::XdLocalCoordinateSystem> &lcs, const std::string &optimizationOptionsFile);
    
    void mapParameters(const discamb::Crystal &crystal);
    void mapParameters(const std::string &optimizationOptionsFile);
    
    std::vector<discamb::TargetFunctionAtomicParamDerivatives> mD_target_dParam;
    Target();
};

#endif
