#ifndef _DISCAMB_IO_HKL_IO_H_
#define _DISCAMB_IO_HKL_IO_H_

#include "discamb/MathUtilities/Vector3.h"

#include <vector>
#include <string>
#include <complex>

namespace discamb {

    /**
    * \defgroup IO IO
    \brief Input output handling.
    * @{
    */


/** 
\brief IO for DiSCaMB specific file formats for reflections and reflection indices storage.

Developed mainly for testing purposes.
*/

namespace hkl_io{
/** Read hkl indices from text file  - for each line in the file it takes 3 subsequent numbers as h, k, l 
starting from column \p hIndexColumn if there is enough column in given line. */
void readHkl(const char *fileName, size_t hIndexColumn, std::vector<Vector3i> &hklIndices);

/** Read hkl indices from text file in DiSCaMB 'compact' format.*/

void readCompactHklFile(const std::string &fileName, std::vector<Vector3i> &hkl);


/** Write hkl indices to text file in DiSCaMB 'compact' format.*/

void writeCompactHklFile(const std::string &fileName, const std::vector<Vector3i> &hkl);

/** Read file with hkl and structure fctors of th following format:
- 1-st line header : "number of reflection N" where N is the number of reflections
- subsequent lines "h_index k_index l_index f_r f_i) where f_r and f_i are real
  and inmaginary components of structure factor 
*/
void readReflectionFile(const std::string &reflection_file,
                        std::vector<Vector3i> &hkl,
                        std::vector<std::complex<double> > &sf);

}
/**@}*/
}

#endif /*_DISCAMB_IO_HKL_IO_H_*/
