


#ifndef _DISCAMB_IO_XD_READER_HPP_
#define _DISCAMB_IO_XD_READER_HPP_
#include "discamb/HC_Model/HC_ModelParameters.h"
#include "discamb/HC_Model/XdLocalCoordinateSystem.h"

#include <fstream>

namespace discamb {

    /**
    * \addtogroup IO IO
    * @{
    */


struct XdMasData
{
    UnitCell unitCell;
    SpaceGroup spaceGroup;
    std::vector<std::string> atomLabels;
    std::vector<std::vector<std::string> > lcs;
    std::vector<Vector3d> dummy_atoms;
    bool dummyAtomIndexingFrom0;
    std::vector<std::vector<std::string> > scat_tables;
    std::vector<size_t> atomKappaSetIndex;
    std::vector<size_t> atomScatTableIndex;
};

struct XdParData
{
    std::vector<Vector3d> positionsFractional;
    std::vector<std::string> atomLabels;
    std::vector<std::vector<double> > adps;
    std::vector<std::vector<double> > kappa_sets;
    std::vector<std::vector<std::vector<double> > > plms;
    std::vector<double> p_val;
    std::vector<size_t> atomToKappaSetMap;
    std::vector<size_t> kappaSetToScatTableMap;
    std::vector<Vector3d> dummyAtomsPositionsFractional;
    std::vector<double> occupancy;
};

// error: always crystal.atoms[i].multiplicity = 1;

/**
\brief basic support for reading XD file - reads in and interpretes files generated by LSDB program
*/

namespace xd_io
{
    void read(const std::string & masterFileName,const std::string & parameterFileName,Crystal &crystal);
    
    void readXdMas(const std::string & masterFileName,XdMasData &xdMasData);

    void readXdPar(const std::string & parameterFileName,XdParData &xdParData);

    void readXdMasSections(const std::string & masterFileName,std::vector<std::string> &generalSection,
                          std::vector<std::pair<std::string,std::vector<std::string> > > &sections);

    void getCrystal(const XdMasData &xdMasData,const XdParData &xdParData,Crystal &crystal);

    void read(const std::string & masterFileName,const std::string & parameterFileName,
                     HC_ModelParameters &hc_parameters,Crystal &crystal,
                     std::vector<XdLocalCoordinateSystem> &localCoordinateSystems,bool group_types,bool comparePval=true);
 

    
}

/**@}*/

}





#endif /*_DISCAMB_IO_XD_READER_HPP_*/
