#include "discamb/BasicChemistry/PeriodicTable.h"

#include "discamb/BasicUtilities/StringUtilities.h"

using namespace std;

namespace {

    struct Element {
        std::string symbol;
        char block;
        size_t period;
        size_t group;
        size_t atomic_number;
    };

    struct Data {
        std::map<std::string, size_t> symbolToAtomicNumber;
        std::vector<Element> elements;
        Data();
    };


    Data::Data()
    {
        string pt = "   H   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -  He  " // 1
            "  Li  Be   -   -   -   -   -   -   -   -   -   -   B   C   N   O   F  Ne  " // 2
            "  Na  Mg   -   -   -   -   -   -   -   -   -   -  Al  Si   P   S  Cl  Ar  " // 3
            "   K  Ca  Sc  Ti   V  Cr  Mn  Fe  Co  Ni  Cu  Zn  Ga  Ge  As  Se  Br  Kr  " // 4
            "  Rb  Sr   Y  Zr  Nb  Mo  Tc  Ru  Rh  Pd  Ag  Cd  In  Sn  Sb  Te   I  Xe  " // 5
            "  Cs  Ba   -  Hf  Ta   W  Re  Os  Ir  Pt  Au  Hg  Tl  Pb  Bi  Po  At  Rn  " // 6
            "  Fr  Ra   -  Rf  Db  Sg  Bh  Hs  Mt  Ds  Rg  Cn  Nh  Fl  Mc  Lv  Ts  Og  " // 7
            "   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -  "
            "   -   -   -  La  Ce  Pr  Nd  Pm  Sm  Eu  Gd  Tb  Dy  Ho  Er  Tm  Yb  Lu  "
            "   -   -   -  Ac  Th  Pa   U  Np  Pu  Am  Cm  Bk  Cf  Es  Fm  Md  No  Lr  ";

        vector<string> symbols;
        discamb::string_utilities::split(pt, symbols);
        Element element;
        size_t atomic_number, index;

        atomic_number = 1;
        index = 0;
        for (size_t period = 1; period < 6; period++)
            for (size_t group = 1; group < 19; group++)
            {
                if (symbols[index] != string("-"))
                {
                    element.symbol = symbols[index];
                    element.atomic_number = atomic_number;
                    element.block = (group < 3 ? 'S' : (group > 12 ? 'P' : 'D'));
                    if (element.symbol == string("He"))
                        element.block = 'S';
                    element.group = group;
                    element.period = period;
                    elements.push_back(element);
                    symbolToAtomicNumber[symbols[index]] = atomic_number;
                    atomic_number++;
                }
                index++;
            }

        for (size_t period = 6; period < 8; period++)
            for (size_t group = 1; group < 19; group++)
            {
                if (index == 92)
                    atomic_number = 72;
                if (index == 100)
                    atomic_number = 104;

                if (symbols[index] != string("-"))
                {
                    element.symbol = symbols[index];
                    element.atomic_number = atomic_number++;
                    element.block = (group < 3 ? 'S' : (group > 12 ? 'P' : 'D'));
                    element.group = group;
                    element.period = period;
                    elements.push_back(element);
                }
                index++;
            }

        index += 18;
        atomic_number = 57;


        for (size_t period = 5; period < 7; period++)
            for (size_t group = 1; group < 19; group++)
            {
                if (atomic_number == 72)
                    atomic_number = 89;

                if (symbols[index] != string("-"))
                {
                    element.symbol = symbols[index];
                    element.atomic_number = atomic_number++;
                    element.block = 'F';
                    element.group = 0;
                    element.period = period;
                    elements.push_back(element);
                }
                index++;
            }

    }

    Data periodicTableData;


}


namespace discamb {

    namespace periodic_table
    {

        std::string symbol(
            size_t atomicNumber)
        {
            if (atomicNumber < 119)
                return periodicTableData.elements[atomicNumber - 1].symbol;
            return "X";
        }

        size_t atomicNumber(
            const std::string &symbol)
        {
            if (periodicTableData.symbolToAtomicNumber.find(symbol) != periodicTableData.symbolToAtomicNumber.end())
                return periodicTableData.symbolToAtomicNumber.find(symbol)->second;
            return 0;
        }

        char block(
            size_t atomicNumber)
        {
            if (atomicNumber < 119)
                return periodicTableData.elements[atomicNumber - 1].block;
            return 'X';
        }

        size_t period(
            size_t atomicNumber)
        {
            if (atomicNumber < 119)
                return periodicTableData.elements[atomicNumber - 1].period;
            return 0;
        }

        size_t group(
            size_t atomicNumber)
        {
            if (atomicNumber < 119)
                return periodicTableData.elements[atomicNumber - 1].group;
            return 0;

        }

    }
}