#include "discamb/BasicUtilities/Timer.h"

using namespace std;

namespace discamb {

WallClockTimer::WallClockTimer()
{
#ifndef USE_CPP11
#if !defined(HAS_SYS_TIME) || !defined(HAS_UNISTD)
#if defined(_WIN32)
    QueryPerformanceFrequency(&mFreq);
#endif
#endif
#endif
}

WallClockTimer::~WallClockTimer()
{
}

void WallClockTimer::start()
{
#ifdef USE_CPP11
    mStart = chrono::steady_clock::now();
    //mStart = std::chrono::high_resolution_clock::now();
#elif HAS_SYS_TIME && HAS_UNISTD
    gettimeofday(&mStart, NULL);
#elif defined _WIN32
    QueryPerformanceFrequency(&mStart);
#else
    time(&mStart);
#endif

}

// returns time in ms
double WallClockTimer::stop()
{
#ifdef USE_CPP11
    mStop = chrono::steady_clock::now();
    //mStop = std::chrono::high_resolution_clock::now();
    return double(chrono::duration_cast<chrono::milliseconds>(mStop-mStart).count());
#elif HAS_SYS_TIME && HAS_UNISTD
    gettimeofday(&mStop, NULL);
    return mStop.tv_sec-mStart.tv_sec + (mStop.tv_usec - mStart.tv_usec)*0.001;
#elif defined _WIN32
    QueryPerformanceFrequency(&mStop);
    return 1000.0 * (double)(mStart.QuadPart - mStop.QuadPart) / mFreq.QuadPart;
#else
    time(&mStop);
    return (mStop - mStart)*1000.0;
#endif

}

std::string WallClockTimer::type()
const
{
#ifdef USE_CPP11
    return string("std::chrono::steady_clock");
#elif HAS_SYS_TIME && HAS_UNISTD
    return string("POSIX <sys/time.h> gettimeofday");
#elif defined _WIN32
    return string("Windows QueryPerformanceFrequency");
#else
    return string("C time_t time(time_t *)");
#endif
}




CpuTimer::CpuTimer()
{
}

CpuTimer::~CpuTimer()
{
}

void CpuTimer::start()
{
#if defined(HAS_SYS_TIME) && defined(HAS_UNISTD)
    mStart = clock();
#elif defined(_WIN32)
    GetProcessTimes(GetCurrentProcess(), &mAux1, &mAux2, &mAux3, &mStart);
#else
    mStart = clock();
#endif

}

double CpuTimer::stop()
{
#ifdef _WIN32
    GetProcessTimes(GetCurrentProcess(), &mAux1, &mAux2, &mAux3, &mStop);
    LARGE_INTEGER stop, start;
    stop.HighPart = mStop.dwHighDateTime;
    stop.LowPart = mStop.dwLowDateTime;
    start.HighPart = mStart.dwHighDateTime;
    start.LowPart = mStart.dwLowDateTime;
    return (stop.QuadPart - start.QuadPart)*0.0001;
#else
    return double(clock() - mStart)*1000.0 / CLOCKS_PER_SEC;
#endif
}

std::string CpuTimer::type()
const
{
#ifdef _WIN32
    return string("Windows GetProcessTimes");
#else
    return string("std::clock");
#endif

}

}//namespace discamb
