#include "discamb/IO/hkl_io.h"
#include "discamb/BasicUtilities/OnError.h"
#include "discamb/BasicUtilities/StringUtilities.h"

#include <fstream>
#include <cstdlib>

using namespace std;

namespace {

    bool intepretCompact(const string &s, discamb::Vector3i &v, size_t &count)
    {
        vector<string> words;
        discamb::string_utilities::split(s, words, 'x');
        string vStr;
        if (words.empty())
            return false;
        if (words.size()>1)
        {
            count = atoi(words[0].c_str());
            vStr = words[1];
        }
        else
        {
            count = 1;
            vStr = words[0];
        }

        discamb::string_utilities::split(vStr, words);

        if (words.size() == 3)
            v.set(atoi(words[0].c_str()), atoi(words[1].c_str()), atoi(words[2].c_str()));
        if (words.size() == 2)
            v.set(0, atoi(words[0].c_str()), atoi(words[1].c_str()));
        if (words.size() == 1)
            v.set(0, 0, atoi(words[0].c_str()));
        return true;

    }

    void writeVec(ofstream &out, const discamb::Vector3i &vec)
    {
        if (vec[0] == 0 && vec[1] == 0)
            out<< vec[2] << endl;
        else
        {
            if(vec[0] == 0)
                out<< vec[1] << " " << vec[2] << endl;
            else 
                out<< vec[0] << " " << vec[1] << " " << vec[2] << endl;

        }
    }
}

namespace discamb {

namespace hkl_io{

    void readReflectionFile(
        const std::string &reflection_file,
        std::vector<Vector3i> &hkl,
        std::vector<std::complex<double> > &sf)
    {
        size_t i, n;
        double a, b;
        string s;
        ifstream in(reflection_file.c_str());

        if (!in.good())
            on_error::throwException(string("can not read reflection file ") + reflection_file, __FILE__, __LINE__);

        in >> s >> s >> s >> n;
        hkl.resize(n);
        sf.resize(n);

        for (i = 0; i<n; i++)
        {
            in >> hkl[i][0] >> hkl[i][1] >> hkl[i][2] >> a >> b;
            sf[i] = complex<double>(a, b);
        }

        in.close();
    }


void readHkl(
    const char *fileName,
    size_t hIndexColumn,
    vector<Vector3i> &hklIndices)
{
    hklIndices.clear();

    ifstream in(fileName);
    if (!in.good())
        on_error::throwException(string("can not open hkl file:") + string(fileName), __FILE__, __LINE__);

    vector<string> words;
    string line;

    while (in.good())
    {
        getline(in, line);
        string_utilities::split(line, words);
        if (words.size() >= hIndexColumn + 2)
            hklIndices.push_back(Vector3i(atoi(words[hIndexColumn - 1].c_str()),
                                          atoi(words[hIndexColumn].c_str()),
                                          atoi(words[hIndexColumn + 1].c_str())));
    }
    in.close();
}

void readCompactHklFile(const string &fileName, vector<Vector3i> &hkl)
{
    ifstream in(fileName.c_str());
    hkl.clear();
    string line;
    vector<string> words;
    Vector3i v;
    size_t i, count;
    bool firstLine = true;

    if (!in.good())
        on_error::throwException(string("can not read hkl file '") + fileName + string("'"), __FILE__, __LINE__);

    while (in.good())
    {
        getline(in, line);
        string_utilities::split(line, words);
        if (intepretCompact(line, v, count))
        {
            if (firstLine)
            {
                hkl.push_back(v);
                for (i = 1; i<count; i++)
                    hkl.push_back(hkl.back() + v);
                firstLine = false;
            }
            else
                for (i = 0; i<count; i++)
                    hkl.push_back(hkl.back() + v);
        }
    }
    in.close();
}

void writeCompactHklFile(
    const std::string &fileName,
    const std::vector<Vector3i> &hkl)
{
    size_t counter = 0;
    size_t same_vec_counter = 0;
    Vector3i previous_vec, vec, vec_old;
    ofstream out(fileName.c_str());
    size_t hklIdx, nHkl = hkl.size();

    for (hklIdx = 0; hklIdx<nHkl; hklIdx++)
    {
        previous_vec = vec;

        vec = hkl[hklIdx];
        if (counter == 0)
        {
            vec = hkl[hklIdx];
            counter = 1;
            same_vec_counter = 1;
        }
        else
        {
            vec = vec - vec_old;
            if (vec == previous_vec)
                same_vec_counter += 1;
            else
            {
                if (same_vec_counter != 1)
                    out << same_vec_counter << 'x';
                writeVec(out, previous_vec);
                same_vec_counter = 1;
            }

        }
        vec_old = hkl[hklIdx];

    }

    if (same_vec_counter != 1)
    {
        out << same_vec_counter << 'x';
        writeVec(out, previous_vec);
    }
    else
        writeVec(out, vec);
}

}
} //namespace discamb