#include "discamb/Scattering/scattering_utilities.h"
#include "discamb/MathUtilities/MathUtilities.h"
#include "discamb/BasicUtilities/OnError.h"

#include <iostream>

using namespace std;

namespace discamb {

namespace scattering_utilities
{
    void centeringSfMultipliers(
        char centering, 
        const vector<Vector3i> &hkl, 
        vector<double> &multipliers,
        bool obverse)
    {
        size_t hklIdx, nHkl = hkl.size();
        int hMod2,kMod2,lMod2,sumMod2;
        multipliers.resize(nHkl);

        if(centering == 'P' || centering == 'p')
        {
            multipliers.assign(nHkl,1.0);
        }
        else if (centering == 'R' || centering == 'r')
        {
            if(obverse)
                for (hklIdx = 0; hklIdx<nHkl; hklIdx++)
                {
                    if(abs(-hkl[hklIdx].x + hkl[hklIdx].y + hkl[hklIdx].z) % 3 == 0)
                        multipliers[hklIdx] = 3.0;
                    else
                        multipliers[hklIdx] = 0.0;
                }
            else
                for (hklIdx = 0; hklIdx<nHkl; hklIdx++)
                {
                    if (abs(hkl[hklIdx].x - hkl[hklIdx].y + hkl[hklIdx].z) % 2 == 0)
                        multipliers[hklIdx] = 3.0;
                    else
                        multipliers[hklIdx] = 0.0;
                }
        }
        else if (centering == 'F' || centering == 'f')
        {
            for ( hklIdx = 0; hklIdx<nHkl; hklIdx++ )
            {
                hMod2 = abs(hkl[hklIdx].x % 2);
                kMod2 = abs(hkl[hklIdx].y % 2);
                lMod2 = abs(hkl[hklIdx].z % 2);

                if( hMod2 == kMod2 && hMod2 == lMod2 )
                    multipliers[hklIdx] = 4.0;
                else
                    multipliers[hklIdx] = 0.0;
                  
            }
        }
        else if (centering == 'H' || centering == 'h')
            on_error::throwException("reflection conditions for H centering not implemented",__FILE__,__LINE__);
        else 
        {
            for (hklIdx = 0; hklIdx<nHkl; hklIdx++)
            {
                if ( centering == 'I' || centering == 'i' )
                    sumMod2 = (hkl[hklIdx].x + hkl[hklIdx].y + hkl[hklIdx].z) % 2;
                else if ( centering == 'A' || centering == 'a')
                    sumMod2 = (hkl[hklIdx].y + hkl[hklIdx].z) % 2;
                else if ( centering == 'B' || centering == 'b')
                    sumMod2 = (hkl[hklIdx].x + hkl[hklIdx].z ) % 2;
                else if ( centering == 'C' || centering == 'c' )
                    sumMod2 = (hkl[hklIdx].x + hkl[hklIdx].y) % 2;

                sumMod2 == 0 ? multipliers[hklIdx] = 2.0 : multipliers[hklIdx] = 0.0;
            }
        }
        
    }


    void combineScatteringFactorsSets(
        const vector< vector< complex< REAL > > > &scatteringFactorsSets,
        vector< complex< REAL > > &scatteringFactors)
    {
        size_t nHkl, nHklInSet, hklIdx, nDataSets;

        nDataSets = scatteringFactorsSets.size();

        // merge scattering factors

        nHkl = 0;
        for (size_t i = 0; i<nDataSets; i++)
            nHkl += scatteringFactorsSets[i].size();
        scatteringFactors.resize(nHkl);
        hklIdx=0;
        for (size_t i = 0; i<nDataSets; i++)
        {
            nHklInSet = scatteringFactorsSets[i].size();
            for(size_t j=0 ; j < nHklInSet ; j++ )
                scatteringFactors[hklIdx++] = scatteringFactorsSets[i][j];
        }
    }

     

    void merge_dTarget_dParameterSets(
        const std::vector<std::vector<TargetFunctionAtomicParamDerivatives> > &dTarget_dParamSets,
        std::vector<TargetFunctionAtomicParamDerivatives> &dTarget_dParam)
    {
        size_t nAdpComponents, atomIdx, nAtoms, setIdx, nSets = dTarget_dParamSets.size();
        
        if(nSets == 0)
        {
            on_error::throwException(" BUG(?) - Empty set of derivatives of target function w.r.t. structural parameters passed as parameter",
                                     __FILE__, __LINE__);
            return;
        }

        dTarget_dParam = dTarget_dParamSets[0];
        nAtoms = dTarget_dParamSets[0].size();
        
        
        for (setIdx = 1; setIdx < nSets; setIdx++)
        {
            for (atomIdx = 0; atomIdx<nAtoms; atomIdx++)
            {
                nAdpComponents = dTarget_dParam[atomIdx].adp_derivatives.size();
                for (size_t i = 0; i<nAdpComponents; i++)
                    dTarget_dParam[atomIdx].adp_derivatives[i] += dTarget_dParamSets[setIdx][atomIdx].adp_derivatives[i];

                dTarget_dParam[atomIdx].atomic_position_derivatives += dTarget_dParamSets[setIdx][atomIdx].atomic_position_derivatives;
                dTarget_dParam[atomIdx].occupancy_derivatives += dTarget_dParamSets[setIdx][atomIdx].occupancy_derivatives;
            }

        }

    }
    
}

} // namespace discamb