#ifndef _DISCAMB_TESTS_CORRECTNESS_TEST_1_GRADIENTDATACOLLECTOR_H_
#define _DISCAMB_TESTS_CORRECTNESS_TEST_1_GRADIENTDATACOLLECTOR_H_

#include "discamb/Scattering/SF_CalcDataTypes.h"
#include "discamb/CrystalStructure/UnitCell.h"

#include <vector>
#include <complex>
#include <string>

// gradient data version for small molecules
struct GradientDataSM
{
    std::vector<std::string> atomLabel;
    // [atom index][hkl index]
    std::vector<std::vector<std::complex<double> > > dOccupancy;
    std::vector<std::vector<std::complex<double> > > dU_iso;
    // [atom index][component index][hkl index]
    std::vector<std::vector<std::vector<std::complex<double> > > > dPosition;
    std::vector<std::vector<std::vector<std::complex<double> > > > dU_aniso;

    void allocate(size_t nHkl, const std::vector<size_t> &nAdpsPerAtom);

    void clear();

    GradientDataSM() { clear(); }
};


struct GradientDataSM_AllConventions
{
    bool hasXyzCart, hasXyzFract, hasUcart, hasUstar, hasUcif;
    // [atom index]
    std::vector<std::string> atomLabel;
    // [atom index][hkl index]
    std::vector<std::vector<std::complex<double> > > occupancy;
    std::vector<std::vector<std::complex<double> > > u_iso;
    // [atom index][component index][hkl index]
    std::vector<std::vector<std::vector<std::complex<double> > > > position_fractional;
    std::vector<std::vector<std::vector<std::complex<double> > > > position_cartesian;
    std::vector<std::vector<std::vector<std::complex<double> > > > u_cartesian;
    std::vector<std::vector<std::vector<std::complex<double> > > > u_star;
    std::vector<std::vector<std::vector<std::complex<double> > > > u_cif;

    //void allocate(size_t nHkl, const std::vector<size_t> &nAdpsPerAtom);

    void clear();

    void set(const GradientDataSM &data, const discamb::UnitCell &unitCell);

    GradientDataSM_AllConventions() { clear(); }
};


class GradientDataCollector
{
private:
    GradientDataCollector();
    GradientDataSM mGradientData;
    std::vector<std::complex<double> > mStructureFactors;
public:
    GradientDataCollector(size_t nHkl, const std::vector<size_t> &nAdpsPerAtom);
    ~GradientDataCollector();
    void onPerHklCalculation(size_t hkl_idx, std::complex<double> &structureFactor, discamb::SfDerivativesAtHkl &derivatives);
    void getGradientData(GradientDataSM &gradientData);
    void getStructureFactors(std::vector<std::complex<double> > &structureFactors);
};


#endif /*_DISCAMB_TESTS_CORRECTNESS_TEST_1_GRADIENTDATACOLLECTOR_H_*/
