#include "Test.h"

#include "Test_Conversion.h"
#include "Test_CpuGpu.h"
#include "Test_Multipolar.h"
#include "Test_MultipolarAsIAM.h"
#include "Test_MultipolarConversion.h"
#include "Test_MultipolarGradientLarge.h"
#include "Test_MultipolarAsIamGradientLarge.h"
#include "Test_MultipolarAsIamGradientSmall.h"
#include "Test_Any.h"
#include "TestTypeGrouping.h"

#include "discamb/BasicUtilities/OnError.h"

using namespace std;
using namespace discamb;

Test::Test()
{
    mMessageLevel = 10;
}

Test::~Test()
{
}

bool Test::run_test(
    std::string &message,  
    const vector<string> &path)
{
    return this->run_test(message);
}

Test *Test::create(
const std::string &type)
{
    if(type==string("Any"))
        return new Test_Any;
    if(type==string("Conversion"))
        return new Test_Conversion;
    if(type==string("CpuGpu"))
        return new TestCpuGpu;
    if (type == string("Multipolar"))
        return new TestMultipolar;
    if (type == string("MultipolarAsIAM"))
        return new TestMultipolarAsIAM;
    if (type == string("MultipolarConversion"))
        return new TestMultipolarConversion;
    if (type == string("MultipolarGradientLarge"))
        return new TestMultipolarGradientLarge;
    if (type == string("MultipolarAsIamGradientLarge"))
        return new TestMultipolarAsIamGradientLarge;
    if (type == string("MultipolarAsIamGradientSmall"))
        return new TestMultipolarAsIamGradientSmall;
    if (type == string("TypeGrouping"))
        return new TestTypeGrouping;

    on_error::throwException(string("an attempt to create test of unknown type: ")+type,__FILE__,__LINE__);

    return 0;//C++11 nullptr;
}

void Test::test_names(
std::vector<std::string> &test_names)
{
    test_names.clear();

    
    test_names.push_back("Conversion");
    test_names.push_back("MultipolarConversion");
    test_names.push_back("Multipolar");
    test_names.push_back("MultipolarAsIAM");
    test_names.push_back("MultipolarGradientLarge");
    test_names.push_back("MultipolarAsIamGradientLarge");
    test_names.push_back("MultipolarAsIamGradientSmall");
}


void Test::all_tests(
std::vector<Test*>  &tests)
{
    tests.clear();
    vector<string> names;
    test_names(names);

    for(size_t i=0;i<names.size();i++)
        tests.push_back(create(names[i]));
}

void Test::setMessageLevel(int messageLevel)
{
    mMessageLevel = messageLevel;
}
