#include "TestTypeGrouping.h"

#include "data_directory.h"
#include "test_utilities.h"

#include "discamb/IO/xd_io.h"
#include "discamb/BasicUtilities/Exception.h"
#include "discamb/BasicUtilities/OnError.h"
#include "discamb/Scattering/HansenCoppensStructureFactorCalculator.h"


#include <iostream>

using namespace std;
using namespace discamb;


TestTypeGrouping::TestTypeGrouping()
{
    mRFactorThreshold = 1e-8;
    populateHkl();
}

TestTypeGrouping::~TestTypeGrouping()
{
}


bool TestTypeGrouping::run_test(
    std::string &message)
{
    string directory = correcness_test_1_data_directory;
    string prefix = directory + string("/");
    string filepath = prefix + string("test_grouping.inp");
    string filesDir = directory + string("/type_grouping/");
    ifstream in(filepath.c_str());

    if(!in.good())
        on_error::throwException(string("can not open input file:'") + filepath + string("' for TypeGrouping test"),__FILE__,__LINE__);

    size_t n_entries, i;
    string s, xd_mas_file, xd_inp_file;
    double r_factor;
    bool passed_test;
    in >> n_entries;
    message.clear();

    passed_test = true;

    for (i = 0; i<n_entries; ++i)
    {
        in >> xd_mas_file >> xd_inp_file;
        if (mMessageLevel>1)
            cout << "processing " << xd_mas_file << endl;
        r_factor = compare(filesDir + xd_mas_file, filesDir + xd_inp_file);
        passed_test = passed_test && (r_factor < mRFactorThreshold);

        if (mMessageLevel>1)
            cout<< "R factor for structure from " << xd_mas_file << " : " << r_factor << "\n";
        //message += string("R factor for structure from ") + xd_mas_file + string(" : ") +
          //  string_utilities::convertToString(r_factor) + string("\n");

    }
    return passed_test;

}


void TestTypeGrouping::populateHkl()
{
    int i,j,k,n=5;

    for(i=0;i<n;i++)
        for(j=0;j<=i;j++)
            for(k=0;k<=j;k++)
                if(!(i==0 && j==0 &&k==0))
                    mHkl.push_back(Vector3i(i,j,k));
}

double TestTypeGrouping::compare(
    const std::string &xd_mas_file,
    const std::string &xd_inp_file)
{
    HC_ModelParameters hc_parameters;
    Crystal crystal;
//    vector<Vector3i> hkl;
    vector<complex<double> > ref_f, f, dTarg_df;
    vector<bool> countAtomContribution;
    vector<string> words;
    vector<XdLocalCoordinateSystem> lcs;
    vector<Matrix3d> lcsMatrices;

    // calc. structure factors with no grouping of atom types 
    xd_io::read(xd_mas_file, xd_inp_file, hc_parameters, crystal, lcs, false);
    lcsMatrices.resize(lcs.size());
    for (size_t i = 0; i< lcs.size(); i++)
        lcs[i].calculate(lcsMatrices[i], crystal);


    countAtomContribution.assign(crystal.atoms.size(), true);
    HansenCoppensStructureFactorCalculator calculator(crystal, hc_parameters);
    
    calculator.calculateStructureFactors(crystal.atoms, lcsMatrices, mHkl, ref_f, countAtomContribution);

    // calc. structure factors with grouping of atom types 

    xd_io::read(xd_mas_file, xd_inp_file, hc_parameters, crystal, lcs, true);

    calculator.setModel(crystal, hc_parameters);
    calculator.calculateStructureFactors(crystal.atoms, lcsMatrices, mHkl, f, countAtomContribution);

    // compare

    return test_utilities::calculateR_Factor(f, ref_f, false);

}


