#include "Test_Conversion.h"
#include "discamb/CrystalStructure/StructuralParametersConverter.h"
#include "discamb/BasicUtilities/OnError.h"
#include "data_directory.h"
#include <iostream>
#include <fstream>

using namespace std;
using namespace discamb;
using namespace structural_parameters_convention;

namespace {
    bool equivalent(const Vector3d &v1, const Vector3d &v2)
    {
        for(size_t i=0;i<3;i++)
            if(fabs(v1[i]-v2[i])>1e-5)
                return false;
        return true;
    }

    bool equivalent(const vector<double> &v1, const vector<double> &v2)
    {
        for (size_t i = 0; i<6; i++)
            if (fabs(v1[i] - v2[i])>1e-5)
                return false;
        return true;
    }


    bool test(const string &fileName)
    {
        ifstream in(fileName.c_str());
        UnitCell unitCell;
        Vector3d xyzFracInp, xyzCartInp, xyz;
        vector<double> uCart(6), uCif(6), uStar(6), adps(6), adps2(6);  
        string s, line, label;
        vector<string> words;
        size_t atomIdx, nAtoms;
        StructuralParametersConverter converter;
        bool result = true;

        double a, b, c, alpha, beta, gamma;
        in>> s >> s >> a >> b >> c >> alpha >> beta >> gamma;
        unitCell.set(a, b, c, alpha, beta, gamma);
        converter.set(unitCell);

        in>> s >> s >> nAtoms;
        getline(in,line);

        for(atomIdx = 0; atomIdx<nAtoms; atomIdx++)
        {
            getline(in, line);
            string_utilities::split(line,words);
            label = words[0];
            xyzFracInp.set(atof(words[1].c_str()), atof(words[2].c_str()), atof(words[3].c_str()));
            xyzCartInp.set(atof(words[4].c_str()), atof(words[5].c_str()), atof(words[6].c_str()));

            converter.convertXyz(xyzFracInp, xyz, fractional, cartesian);
            if(!equivalent(xyzCartInp, xyz))
                return false;

            converter.convertXyz(xyzCartInp, xyz, cartesian, fractional);
            if (!equivalent(xyzFracInp, xyz))
                return false;

            converter.xyzFractionalToCartesian(xyzFracInp, xyz);
            if (!equivalent(xyzCartInp, xyz))
                return false;

            converter.xyzCartesianToFractional(xyzCartInp,xyz);
            if(!equivalent(xyzFracInp, xyz))
                return false;


            if(words.size() > 7)// has U_aniso
            {
                // star cif cart
                for(size_t i=0;i<6;i++)
                {
                    uStar[i] = atof(words[i + 7].c_str());
                    uCif[i] = atof(words[i + 13].c_str());
                    uCart[i] = atof(words[i + 19].c_str());
                }

                converter.convertADP(uStar,adps,U_star,U_cif);
                if(!equivalent(adps,uCif))
                    return false;


                converter.convertADP(uStar, adps, U_star, U_cart);
                if (!equivalent(adps, uCart))
                    result = false;


                converter.convertADP(uCart,adps, U_cart, U_cif);
                if(!equivalent(adps,uCif))
                    result = false;

                converter.convertADP(uCart,adps,U_cart,U_star);
                if (!equivalent(adps, uStar))
                    result = false;

                converter.convertADP(uCif, adps, U_cif, U_star);
                if (!equivalent(adps, uStar))
                    result = false;

                converter.convertADP(uCif, adps, U_cif, U_cart);
                if (!equivalent(adps, uCart))
                    result = false;


                converter.uStarToU_cif(uStar, adps);
                if (!equivalent(adps, uCif))
                    return false;

                converter.uStarToU_cart(uStar, adps);
                if (!equivalent(adps, uCart))
                    return false;

                converter.uCartToU_cif(uCart, adps);
                if (!equivalent(adps, uCif))
                    return false;

                converter.uCartToU_star(uCart, adps);
                if (!equivalent(adps, uStar))
                    return false;

                converter.uCifToU_cart(uCif, adps);
                if (!equivalent(adps, uCart))
                    return false;

                converter.uCifToU_star(uCif, adps);
                if (!equivalent(adps, uStar))
                    return false;

            }
        }

        return result;
    }


    

}

Test_Conversion::Test_Conversion()
{
}

Test_Conversion::~Test_Conversion()
{
}



bool Test_Conversion::run_test(
    std::string &message)
{
    message.clear();
    string s, dirName, inputPath;
    ifstream in;
    size_t i, n;

    dirName = correcness_test_1_data_directory + string("/conversion/");
    inputPath = dirName + string("test_conversion.inp");
    in.open(inputPath.c_str());
    if(!in.good())
        on_error::throwException("can not read conversion test input file",__FILE__,__LINE__);

    in>> s >> s >> n;
    for( i=0 ; i<n ; i++ )
    {
        in>> s;
        
        if(!test(dirName+s))
        {
            if(mMessageLevel>1)
                cout<< "failed test for " << s << endl;
            return false;
        }
        if (mMessageLevel>1)
            cout << "passed test for " << s << endl;
    }

    return true;
}


