#ifndef _DISCAMB_TESTS_CORRECTNESS_TEST_1_TEST_GRADIENTLARGE_H_
#define _DISCAMB_TESTS_CORRECTNESS_TEST_1_TEST_GRADIENTLARGE_H_
#include "Test.h"
#include "discamb/CrystalStructure/Crystal.h"
#include "discamb/Scattering/SF_CalcDataTypes.h"

#include <complex>

class TestGradientLarge : public Test
{
public:
    TestGradientLarge();
    virtual ~TestGradientLarge();
    virtual bool run_test(std::string &message);
    virtual bool run_test(std::string &message, const std::vector<std::string> &arguments);
    //virtual bool run_test(std::string &message, const std::string &directory);
    // e.g. gradient/iam_large
    virtual std::string dataDirName() const = 0;
    virtual std::string inputFileName() const = 0;
    static void readGradientLargeInp(const std::string &input_file, std::string &dTargetdParameterFile,
                                     std::vector<std::vector<std::string> > &dataFiles);

    struct GradientData
    {
        double occupancy;
        double position_fractional[3];
        double position_cartesian[3];
        double u_cif[6];
        double u_cartesian[6];
        double u_star[6];
        double u_iso;

        bool hasOccupancy;
        bool hasPosition_fractional;
        bool hasPosition_cartesian;
        bool has_u_cif;
        bool has_u_cartesian;
        bool has_u_star;
        bool has_u_iso;

        void clear()
        {
            hasOccupancy = hasPosition_fractional = hasPosition_cartesian =
                has_u_cif = has_u_cartesian = has_u_star = has_u_iso =
                false;
        }

        GradientData() { clear(); }
    };

    static void readGradientFile(const std::string &fileName, std::vector<GradientData> &gradientData);

protected:
    //bool mOneDirectory;
    //std::string mDataDir;
    bool mWrite;
    static void writeResults(const std::string &xdMasFileName,
                             const std::vector<discamb::AtomInCrystal> &atoms,
                             const std::vector<discamb::Vector3i> &hkl,
                             const std::vector<std::complex<double> > &f,
                             std::vector<discamb::TargetFunctionAtomicParamDerivatives> &dTarg_dParam);

    bool run_test(std::string &message,bool customDir, const std::string &customDirPath = std::string());
    //virtual std::string structureFilesDir() const = 0; 
    virtual std::string gradientFilesDir() const = 0;
    virtual void readStructureAndCalculateDerivatives(
        bool customDir, const std::string &customDirPath,
        const std::vector<std::string> &structureFileNames,
        const std::vector<discamb::Vector3i> &hkl,
        const std::vector<std::complex<double> > &dTarget_dF,
        discamb::Crystal &crystal,
        std::vector<discamb::TargetFunctionAtomicParamDerivatives> &dTarg_dParam) = 0;
    
    virtual void readInputFile(const std::string &inputFileName,
                               std::string &dTarget_dF_file,
                               std::vector<std::vector<std::string> > &structureFiles,
                               std::vector<std::string> &hklFiles,
                               std::vector<std::string> &gradientFiles) = 0;
    static std::string makePrefix(bool customDir, const std::string customDirPath, const std::string defaultPath);

private:


    void calculateR_Factor(bool customDir, 
                           const std::string &customDirPath,
                           const std::vector<std::string> &structureFiles,
                           const std::string &hklFile,
                           const std::string &gradientsFile,
                           GradientData &gradientR_Factor,
                           GradientData &validData,
                           const std::vector<std::complex<double> > &dTarget_dF);

    
    static void checkGradient(const std::string &parameterType,const double *const gradient,const double * const validity,size_t nComponents,double threshold,bool &testState,int messageLevel);
    static void checkGradient(const std::string &parameterType, double gradient, double validity, double threshold, bool &testState, int messageLevel);
    static bool hasNonzeroElement(const std::vector<double> &v);
    /* calculates r-factor for data with respect to referenceData (saves to rFactor) if 
     the corresponding vectors are not empty (indicates by setting hasData). If the reference data are all 0
     then denomiator in R-factor calculation is zero. In this situation rFactor is set to zero and validData to 0,
     otherwise validData = 1 */
    static void calculateR_Factor(std::vector<double> &data,std::vector<double> &refData,
                                 double &rFactor,double &validData,bool &hasData);
    static void calculateTransformedGradients(const discamb::UnitCell &unitCell, std::vector<GradientData> &gradientData);

};

#endif /*_DISCAMB_TESTS_CORRECTNESS_TEST_1_TEST_GRADIENTLARGE_H_*/
