#ifndef _DISCAMB_TESTS_CORRECTNESS_TEST_1_TEST_GRADIENTSMALL_H_
#define _DISCAMB_TESTS_CORRECTNESS_TEST_1_TEST_GRADIENTSMALL_H_
#include "Test.h"
#include "GradientDataCollector.h"
#include "discamb/CrystalStructure/UnitCell.h"
#include "discamb/Scattering/SF_CalcDataTypes.h"
#include "discamb/CrystalStructure/StructuralParametersConverter.h"

#include <complex>



class TestGradientSmall : public Test
{
public:
    TestGradientSmall();
    virtual ~TestGradientSmall();
    virtual bool run_test(std::string &message);
    //virtual bool run_test(std::string &message, const std::string &directory);
    

    static void readGradientSmallInp(const std::string &input_file, 
                                     std::vector<std::vector<std::string> > &structureFiles,
                                     std::vector<std::string> &gradientFiles);
    // e.g. gradient/iam_small
    virtual std::string dataDirName() const=0;
    // e.g. gradient_data/small_molecule/iam
    virtual std::string hklAndGradientDirName() const = 0;
    virtual std::string inputFileName() const = 0;


    static void readGradientAndHkl(const std::string &fileName, std::vector<std::complex<double> > &sf,
                                   GradientDataSM_AllConventions  &gradientData, std::vector<discamb::Vector3i> &hkl);

protected:



   /* static void convertAdps(
                    const std::vector<std::vector<std::complex<double> > > &adpsIn,
                    std::vector<std::vector<std::complex<double> > > &adpsOut,
                    void (StructuralParametersConverter::*convertingFunction)(
                         const std::vector<std::complex<double> > &derivativesIn,
                         std::vector<std::complex<double> > &derivativesOut) const,
                         StructuralParametersConverter &converter);*/

    /*
    struct DerivativesProcessor
    {
    private:
        DerivativesProcessor();
        GradientDataSM mGradientData;
        std::vector<std::complex<double> > mStructureFactors;
    public:
        DerivativesProcessor(size_t nHkl,const std::vector<size_t> &nAdpsPerAtom);
        ~DerivativesProcessor();
        void onPerHklCalculation(size_t hkl_idx, std::complex<double> &structureFactor, SfDerivativesAtHkl &derivatives);
        void clear();
        void getGradientData(GradientDataSM &gradientData);
        void getStructureFactors(std::vector<std::complex<double> > &structureFactors);
    };
    */

    virtual void calculateGradientData(const std::vector<std::string> &structureFiles, const std::vector<discamb::Vector3i> &hkl,
                               std::vector<std::complex<double> > &structureFactors, GradientDataSM_AllConventions &gradientData) const = 0;

    /**
    Convert gradient between coordinate systems to generate the data which are missing.
    E.g. if there are gradients with respect to position in Cartesian coordinate system
    but not for the positions in fractional coordinate system then the data for fractional
    coordinate system are generated by converting the data for Cartesian coordinate system.
    */
    //static void convertGradientToGenerateMissingData(GradientDataSM &gradientData, const UnitCell &unitCell);

private:

    struct GradientRFactorSM
    {
        
        std::string systemLabel;
        // [atom index]
        std::vector<std::string> atomLabel;
        std::vector<double> occupancy;
        std::vector<double> u_iso;
        // [atom index][component index]
        std::vector<std::vector<double> > position_fractional;
        std::vector<std::vector<double> > position_cartesian;
        std::vector<std::vector<double> > u_cif;
        std::vector<std::vector<double> > u_cartesian;
        std::vector<std::vector<double> > u_star;
        
    };

    double calculateR_Factor(const std::vector<std::string> &structureFiles,
                           const std::string &gradientsFile,
                           GradientRFactorSM &gradientR_Factor);


    static bool rFactorTestPassed(const GradientRFactorSM &rFactorData,int messageLevel);
    
    static void checkGradient(const std::string &parameterType,const double *const gradient,const double * const validity,size_t nComponents,double threshold,bool &testState,int messageLevel);
    static void checkGradient(const std::string &parameterType, double gradient, double validity, double threshold, bool &testState, int messageLevel);
    static bool hasNonzeroElement(const std::vector<double> &v);

    //static void calculateTransformedGradients(const UnitCell &unitCell, GradientDataSM &gradientData);

};

#endif /*_DISCAMB_TESTS_CORRECTNESS_TEST_1_TEST_GRADIENTSMALL_H_*/
