#include "Test_MultipolarAsIamGradientLarge.h"
#include "test_utilities.h"
#include "data_directory.h"

#include "discamb/MathUtilities/Vector3.h"

#include "discamb/BasicUtilities/OnError.h"
#include "discamb/BasicUtilities/StringUtilities.h"
#include "discamb/CrystalStructure/ReciprocalLatticeUnitCell.h"

#include "discamb/Scattering/HansenCoppensStructureFactorCalculator.h"

#include "discamb/IO/NativeIAM_Reader.h"
#include "discamb/IO/xd_io.h"


#include <fstream>
#include <iomanip>

#ifdef _OPENMP
#include <omp.h>
#endif

using namespace std;
using namespace discamb;





TestMultipolarAsIamGradientLarge::TestMultipolarAsIamGradientLarge()
{
    mN_Cores=0;
}

TestMultipolarAsIamGradientLarge::~TestMultipolarAsIamGradientLarge()
{
}

string TestMultipolarAsIamGradientLarge::gradientFilesDir()
const
{
    return correcness_test_1_data_directory + string("/gradient_data/large_molecule/iam");
}



void TestMultipolarAsIamGradientLarge::readStructureAndCalculateDerivatives(
    bool customDir,
    const std::string &customDirPath,
    const std::vector<std::string> &structureFileNames,
    const std::vector<Vector3i> &hkl,
    const vector<complex<double> > &dTarget_dF,
    Crystal &crystal,
    std::vector<TargetFunctionAtomicParamDerivatives> &dTarg_dParam) 
{
    vector<complex<double> > f;
    ScatteringParameters scatteringParameters;
    NativeIAM_Reader reader;
    Crystal crystal2;
    


    HC_ModelParameters hc_parameters;
    vector<XdLocalCoordinateSystem> lcs;
    string dirXdFiles = makePrefix(customDir, customDirPath, correcness_test_1_data_directory + string("/structure_data/xd"));
    string dirIamFiles = makePrefix(customDir, customDirPath, correcness_test_1_data_directory + string("/structure_data/iam"));

    xd_io::read(dirXdFiles + structureFileNames[0], dirXdFiles + structureFileNames[1], hc_parameters, crystal, lcs, false);
    
    reader.read(dirIamFiles + structureFileNames[2], crystal, scatteringParameters);


    vector<bool> countAtomContribution(crystal.atoms.size(), true);

    
    if(!test_utilities::setAnomalousDispersionParametersInMultipolarModel(hc_parameters, scatteringParameters))
        on_error::throwException(string("wfnType should correspond to IAM type, invalid test set containing file '") + structureFileNames[0] + string("'"),
        __FILE__, __LINE__);


    vector<Matrix3d> lcsMatrices;


    lcsMatrices.resize(lcs.size());
    for (size_t i = 0; i< lcs.size(); i++)
        lcs[i].calculate(lcsMatrices[i], crystal);

    HansenCoppensStructureFactorCalculator hcCalculator(crystal, hc_parameters);
    hcCalculator.setDerivativesConvention(structural_parameters_convention::cartesian, structural_parameters_convention::U_cart);
    
    hcCalculator.useCPU_IAM();
    hcCalculator.setN_Threads(mN_Cores);
    hcCalculator.calculateStructureFactorsAndDerivatives(crystal.atoms, lcsMatrices, hkl, f, dTarg_dParam, dTarget_dF,countAtomContribution);

    //--------------

    //writeResults(structureFileNames[0], crystal.atoms, hkl, f, dTarg_dParam);
}

void TestMultipolarAsIamGradientLarge::readInputFile(
    const std::string &inputFileName,
    std::string &dTarget_dF_file,
    std::vector<std::vector<string> > &structureFiles,
    std::vector<std::string> &hklFiles,
    std::vector<std::string> &gradientFiles) 
{

    ifstream in(inputFileName.c_str());
    string s, line;
    int nCores,nAvailableThreads;
    vector<string> words;
    if (!in.good())
        on_error::throwException(string("can not open input file: '") + inputFileName +
                                 string("' when testing gradients"), __FILE__, __LINE__);
    in >> s >> s >> s >> nCores;

#ifdef _OPENMP
    if(nCores<0)
    {
        nAvailableThreads = omp_get_max_threads();
        if(nAvailableThreads+nCores>0)
            mN_Cores = nAvailableThreads + nCores;
        else
            mN_Cores = 1;
    }
    else 
        mN_Cores = nCores;
#else
    mN_Cores = 1;
#endif
    
    in >> s >> s >> dTarget_dF_file;
    in >> s >> s >> s;
    getline(in, line);

    size_t nDataSets = atoi(s.c_str());
    
    structureFiles.clear();
    structureFiles.resize(nDataSets);
    hklFiles.resize(nDataSets); 
    gradientFiles.resize(nDataSets); 

    for (size_t i = 0; i<nDataSets; i++)
    {
        if (!in.good())
            on_error::throwException(string("unexpected end of input file: '") + inputFileName +
                                     string("' when testing gradients"), __FILE__, __LINE__);
        getline(in, line);
        string_utilities::split(line, words);
        if (words.size() != 5)
            on_error::throwException(string("invalid format of input file: '") + inputFileName +
                                     string("' when testing gradients"), __FILE__, __LINE__);
        structureFiles[i].push_back(words[0]);
        structureFiles[i].push_back(words[1]);
        structureFiles[i].push_back(words[2]);
        hklFiles[i] = words[3];
        gradientFiles[i] = words[4];
    }

    in.close();

}

