#include "Test_MultipolarAsIamGradientSmall.h"
#include "discamb/IO/NativeIAM_Reader.h"
#include "discamb/Scattering/HansenCoppensStructureFactorCalculator.h"
#include "discamb/IO/xd_io.h"
#include "test_utilities.h"
#include "discamb/BasicUtilities/OnError.h"
#include "data_directory.h"

using namespace std;
using namespace discamb;


TestMultipolarAsIamGradientSmall::TestMultipolarAsIamGradientSmall()
{
}

TestMultipolarAsIamGradientSmall::~TestMultipolarAsIamGradientSmall()
{
}

void TestMultipolarAsIamGradientSmall::calculateGradientData(
    const std::vector<std::string>& structureFiles, 
    const std::vector<Vector3i>& hkl, 
    std::vector<std::complex<double> > &structureFactors,
    GradientDataSM_AllConventions & gradientData)
const
{
    vector<complex<double> > f;
    ScatteringParameters scatteringParameters;
    NativeIAM_Reader reader;
    Crystal crystal;
    string dataDir = correcness_test_1_data_directory;
    string xdFilesDir = dataDir + string("/structure_data/xd/");
    string iamStructureFilesDir = dataDir + string("/structure_data/iam/");
    //string gradFilesDir = dataDir + string("/gradient_data/small_molecule/iam/");

    
    vector<bool> countAtomContribution(crystal.atoms.size(), true);


    HC_ModelParameters hc_parameters;
    vector<XdLocalCoordinateSystem> lcs;
    xd_io::read(xdFilesDir + structureFiles[0], xdFilesDir + structureFiles[1], hc_parameters, crystal, lcs, false);
    reader.read(iamStructureFilesDir + structureFiles[2], crystal, scatteringParameters);
    vector<Matrix3d> lcsMatrices;
    size_t atomIdx, nAtoms = crystal.atoms.size();
    vector<size_t> nAdpsPerAtom(nAtoms);

    if (nAtoms == 0)
        return;

    for (atomIdx = 0; atomIdx<nAtoms; atomIdx++)
        nAdpsPerAtom[atomIdx] = crystal.atoms[atomIdx].adp.size();

    if (!test_utilities::setAnomalousDispersionParametersInMultipolarModel(hc_parameters, scatteringParameters))
        on_error::throwException(string("wfnType should correspond to IAM type, invalid test set containing file '") + structureFiles[0] + string("'"),
                                 __FILE__, __LINE__);


    GradientDataCollector dataCollector(hkl.size(), nAdpsPerAtom);

    lcsMatrices.resize(lcs.size());
    for (size_t i = 0; i< lcs.size(); i++)
        lcs[i].calculate(lcsMatrices[i], crystal);

    HansenCoppensStructureFactorCalculator hcCalculator(crystal, hc_parameters);
    //hcCalculator.setEngineType(CPU_IAM);
    hcCalculator.useCPU_IAM();
    hcCalculator.setDerivativesConvention(structural_parameters_convention::cartesian,structural_parameters_convention::U_cart);
    hcCalculator.calculateStructureFactorsAndDerivatives(crystal.atoms, lcsMatrices, hkl, dataCollector);
//    dataCollector.getGradientData(gradientData);
//    processor.getStructureFactors(structureFactors);
//    convertGradientToGenerateMissingData(gradientData, crystal.unitCell);
    GradientDataSM gradientDataSM;
    dataCollector.getGradientData(gradientDataSM);
    dataCollector.getStructureFactors(structureFactors);
    gradientData.set(gradientDataSM, crystal.unitCell);

    /*
    GradientDataCollector dataCollector(hkl.size(), nAdpsPerAtom);
    sf_calculator.calculateStructureFactorsAndDerivatives(crystal, scatteringParameters, hkl, dataCollector);
    GradientDataSM gradientDataSM;
    dataCollector.getGradientData(gradientDataSM);
    dataCollector.getStructureFactors(structureFactors);
    gradientData.set(gradientDataSM, crystal.unitCell);
    */
}
