#ifndef _DISCAMB_TESTS_CORRECTNESS_TEST_1_TEST_MULTIPOLARCONVERSION_H_
#define _DISCAMB_TESTS_CORRECTNESS_TEST_1_TEST_MULTIPOLARCONVERSION_H_
#include "Test.h"

#include "discamb/MathUtilities/Vector3.h"
#include "discamb/MathUtilities/Matrix3.h"

#include "discamb/CrystalStructure/Crystal.h"
#include "discamb/Scattering/HansenCoppensStructureFactorCalculator.h"
#include "GradientDataCollector.h"

#include <vector>
#include <complex>


class TestMultipolarConversion : public Test
{
public:
    TestMultipolarConversion();
    virtual ~TestMultipolarConversion();
    virtual bool run_test(std::string &message);
    virtual std::string test_name() const {
        return std::string("conversion of coordinates/convention for position and ADP parameters in multipolar SF and derivatives calculation"); 
    };

private:
    std::string mEngineId;
    int mN_Threads;

    bool testStructure(const std::string &mx_mas,
                       const std::string &xd_inp,
                       const std::string &structure_file_R_frac_U_cif,
                       const std::string &structure_file_R_cart_U_cart,
                       const std::string &structure_file_R_cart_U_star,
                       const std::string &gradSmallFile);

    void addGradientInfo(const  std::vector<discamb::TargetFunctionAtomicParamDerivatives> &dTarg_dParam, GradientDataSM &dataSM,size_t hklIdx);

    static bool compare(const GradientDataSM_AllConventions &dataAll, 
                        const GradientDataSM &gradientDataSM_frac_Ustar, 
                        const GradientDataSM &gradientDataSM_cart_Ucart, 
                        const GradientDataSM &gradientDataSM_cart_Ucif,
                        const GradientDataSM &gradientDataLM_frac_Ustar, 
                        const GradientDataSM &gradientDataLM_cart_Ucart, 
                        const GradientDataSM &gradientDataLM_cart_Ucif);

    static bool compare(const std::vector<std::complex<double> > &ref, const std::vector<std::complex<double> > &calc, bool onlyRealPart=false);
    

    double calc_R(const std::string &mx_mas_file, const std::string &xd_inp_file, const std::string &hkl_file, bool print_diff);
    double mRFactorThreshold;
    void printDifference(const std::string &xd_mas_file_name, std::vector<discamb::Vector3i> &hkl,
                         std::vector<std::complex<double> > &ref_f, std::vector<std::complex<double> > &f);
};


#endif /*_DISCAMB_TESTS_CORRECTNESS_TEST_1_TEST_MULTIPOLARCONVERSION_H_*/
