#include "Test.h"
#include "discamb/BasicUtilities/Exception.h"
// C++11 #include <memory>
#include <vector>
#include <iostream>

using namespace std;
using namespace discamb;
/**
usage:

test_1 [-vl=n] TEST_NAME [test_arguments]  
test_1 -all [-vl=n]

-vl=n sets verbosity level to n
-all runs all tests (except of TestAny)
*/

void printUsageInfo()
{
    vector<string> test_names;

    cout << "ERROR: mising or invalid arguments, usage:" << endl
         << "    test_1 TEST_NAME [test_arguments] [-vl=n], or" << endl
         << "    run_tests - all [-vl=n]" << endl << endl
         << "    -vl=n sets verbosity level to n" << endl
         << "    -all runs all tests(except of TestAny)" << endl << endl
         << "available test:" << endl;

        Test::test_names(test_names);
        for (size_t i = 0; i<test_names.size(); i++)
            cout << "  " << test_names[i] << endl;
}



int main(int argc,char *argv[])
{
    vector<string> passed_tests, failed_tests, words, arguments;
    vector<Test*> tests;
    string message, option, testName;
    bool runAll, dataInCurrentDirectory;
    /*
    0 - do not print anything for individual tests,
    1 - print test progress and final result
    2 - print details
    */
    size_t verbosityLevel;

    // default options
    runAll = false;
    verbosityLevel = 1;
    dataInCurrentDirectory = false;
    bool isOption;
    
    try{
    // extract 'options' - starting with '-' like e.g. -all
        for(int i=1;i<argc;i++)
        {
            isOption = false;

            if(argv[i][0]=='-')
                if(argv[i][1]!='-')
                    isOption=true;
            if(isOption)
            {
                string_utilities::split(argv[i], words, '=');
                option = words[0];
                if(option == string("-all"))
                    runAll = true;
                if(option == string("-vl"))
                    verbosityLevel = atoi(words[1].c_str());
                if(option == string("-ld"))
                    dataInCurrentDirectory = true;
            }
            else
            {
                if(i==1)
                    testName = argv[i];
                else
                    arguments.push_back(argv[i]);
            }
        }

        if(dataInCurrentDirectory && arguments.empty())
            arguments.push_back(string());

        if(runAll)
            Test::all_tests(tests);
        else
        {
            if(testName.empty())
            {
                printUsageInfo();
                exit(1);
            }
            else
                tests.push_back(Test::create(testName));
    
        }


         for (size_t i=0,n=tests.size();i<n;i++)
         {
            tests[i]->setMessageLevel(verbosityLevel);

            cout<< "running test: " << tests[i]->test_name() << endl;

            if (!tests[i]->run_test(message, arguments))
                failed_tests.push_back(tests[i]->test_name());
            else
                passed_tests.push_back(tests[i]->test_name());
            if(!message.empty())
                cout << message << endl;        
        }
    
        if(tests.size() == 1)
        {

            if(passed_tests.empty())
                cout<< "Failed test: ";
            else
                cout<< "Passed test: ";
            cout<< tests[0]->test_name() << endl;
        }
        else
        {   
            if(passed_tests.empty())
                cout<< "-------------------------------" << endl
                    << endl
                    << "        ALL TESTS FAILED       " << endl
                    << endl
                    << "-------------------------------" << endl;
            else
            {
                if(failed_tests.empty())
                    cout << "-------------------------------" << endl
                         << endl
                         << "        ALL TESTS PASSED       " << endl
                         << endl
                         << "-------------------------------" << endl;
                else
                {
                    cout << "PASSED TESTS:" << endl;
                    for (size_t i = 0, n = passed_tests.size(); i<n; i++)
                        cout << passed_tests[i] << endl;
                    cout << "FAILED TESTS:" << endl;
                    for (size_t i = 0, n = failed_tests.size(); i<n; i++)
                        cout << failed_tests[i] << endl;

                    cout << endl
                         << "-------------------------------" << endl
                         << endl
                         << "       SOME TESTS FAILED       " << endl
                         << endl
                         << "-------------------------------" << endl;

                }
            }


        }
    }
    catch (Exception &e)
    {
        cout << e.what() << endl;
    }

}
