#include "test_utilities.h"

#include "discamb/BasicUtilities/OnError.h"

#include <fstream>

using namespace std;
using namespace discamb;


namespace test_utilities {

double calculateR_Factor(
    const std::vector<std::complex<double> > &data,
    const std::vector<std::complex<double> > &referenceData,
    bool scaleFactor)
{
    double oo, co, cc;
    size_t i, n = data.size();
    vector<complex<double> > diff;

    if (data.size() != referenceData.size())
        on_error::throwException("an attempt to calculate R-factors between data sets of differtent size", __FILE__, __LINE__);

    if(data.empty())
        return -10;

    oo = 0;
    co = 0;
    cc = 0;

    diff.resize(n);

    for (i = 0; i<n; i++)
    {
        oo += std::norm(referenceData[i]);
        co += (referenceData[i] * conj(data[i])).real();
        cc += std::norm(data[i]);
        diff[i] = referenceData[i] - data[i];
    }

    if (oo == 0)
        return -5;
        //on_error::throwException("denominator 0 in R-factor", __FILE__, __LINE__);

    double numerator, denominator, s;
    
    scaleFactor ? s = co / cc : s = 1.0;

    numerator = oo - 2 * s*co + s*s*cc;
    denominator = oo;

    if(denominator<1e-10)
        return -5;
    if(referenceData.empty())
        return -10;

    return sqrt(abs(numerator / denominator));

}

double calculateR_Factor(
    const std::vector<double> &data,
    const std::vector<double> &referenceData,
    bool scale)
{
    double oo;//, co, cc;
    size_t i, n = data.size();
    double numerator, denominator;
    

    if (data.size() != referenceData.size())
        on_error::throwException("an attempt to calculate R-factors between data sets of differtent size", __FILE__, __LINE__);


    oo = 0;

    if(scale)
    {
        double co,cc,s;

        co = 0;
        cc = 0;

        for (i = 0; i<n; i++)
        {
            oo += referenceData[i] * referenceData[i];
            co += data[i] * referenceData[i];
            cc += data[i] * data[i];
        }
        s = s = co / cc;
        numerator = oo - 2 * s*co + s*s*cc;
    }
    else
    {
        double diff,diff2=0;
        for (i = 0; i<n; i++)
        {
            diff = data[i] - referenceData[i];
            diff2 += diff*diff;
            oo += referenceData[i] * referenceData[i];
        }
        numerator = diff2;
    }
    
    if (oo == 0)
		return -5;
        //on_error::throwException("denominator 0 in R-factor", __FILE__, __LINE__); 

    denominator = oo;

    if (denominator<1e-10)
        return -5;
    if (referenceData.empty())
        return -10;

    return sqrt(abs(numerator / denominator));
}



std::string structureNameFromFileName(const std::string & fileName)
{
    size_t lastDotPosition = fileName.find_last_of('.');
    string result;
    lastDotPosition == string::npos ? 
        result = fileName : 
        result = fileName.substr(0,lastDotPosition);
    return result;
}

void read_dTarget_dSF(const std::string & fileName, std::vector<std::complex<double> >& values)
{
    values.clear();

    ifstream dT_dF(fileName.c_str());
    if (!dT_dF.good())
        on_error::throwException(string("can not open d Target/d F file : '") + fileName +
                                 string("' when testing gradients"), __FILE__, __LINE__);
    string line;
    vector<string> words;

    while (dT_dF.good())
    {
        getline(dT_dF, line);
        string_utilities::split(line, words);
        if (words.size() == 2)
            values.push_back(complex<double>(atof(words[0].c_str()), atof(words[1].c_str())));
    }

    dT_dF.close();

}

/*
void readIamStructuralInformation(
    const std::string &structure_file,
    std::vector<int> &atom_to_wfn_map,
    std::vector<std::string> &scattering_type,
    std::vector<std::vector<double> > &atomic_displacement_parameters,
    Vector3d &a,
    Vector3d &b,
    Vector3d &c,
    std::vector<Vector3d> &atomicPositions,
    std::vector<double> &atomic_occupancy,
    std::vector<double> &atomic_multiplicity_factor,
    std::vector<Matrix3i> &rotations,
    std::vector<Vector3d> &translations)
{

}
*/

bool setAnomalousDispersionParametersInMultipolarModel(
    HC_ModelParameters &hc_parameters,
    const ScatteringParameters &scatteringParameters)
{
    string wfnTypeName;
    vector<string>::const_iterator it;
    for (size_t i = 0; i<hc_parameters.wfn_parameters.size(); i++)
    {
        wfnTypeName = hc_parameters.wfn_parameters[i].wfnAtomTypeName;
        it = find(scatteringParameters.atomTypeSymbols.begin(), scatteringParameters.atomTypeSymbols.end(), wfnTypeName);
        if (it == scatteringParameters.atomTypeSymbols.end())
            return false;
        size_t index = distance(scatteringParameters.atomTypeSymbols.begin(), it);
        hc_parameters.wfn_parameters[i].anomalous_scattering = scatteringParameters.atomTypeAnomalousDispersion[index];
    }
    return true;
}



} //namespace test_utilties
