#ifndef _DISCAMB_TESTS_CORRECTNESS_TEST_1_TEST_UTILITIES_H_
#define _DISCAMB_TESTS_CORRECTNESS_TEST_1_TEST_UTILITIES_H_


#include "discamb/MathUtilities/Vector3.h"
#include "discamb/MathUtilities/Matrix3.h"
#include "discamb/HC_Model/HC_ModelParameters.h"
#include "discamb/Scattering/SF_CalcDataTypes.h"

#include <vector>
#include <complex>
#include <string>

namespace test_utilities{

    /**
    denominator<1e-10 returns -5, referenceData.empty() - returns -10
    */

    double calculateR_Factor(const std::vector<std::complex<double> > &scattering_factors,
                             const std::vector<std::complex<double> > &reference_scattering_factors,
                             bool scale);

    double calculateR_Factor(const std::vector<double> &data,
                             const std::vector<double> &referenceData,
                             bool scale);

    std::string structureNameFromFileName(const std::string &fileName);

    bool setAnomalousDispersionParametersInMultipolarModel(discamb::HC_ModelParameters &hc_parameters,
                                                           const discamb::ScatteringParameters &scatteringParameters);

    /**
    read input file for testing gradient calculations using algorithm for large molecules
    */

    void readGradientLargeInp(const std::string &input_file, std::string &dTargetdParameterFile,
                              std::vector<std::vector<std::string> > &dataFiles);

    /**
    reads input file for testing gradient calculations using algorithm for small molecules
    */

    void readGradientSmallInp(const std::string &input_file, std::vector<std::vector<std::string> > &dataFiles);

    /** 
    reads content of file with derivatives of target function with respect to components (A,B) of scattering factor
    SF = A+iB
    */

    void read_dTarget_dSF(const std::string &fileName,std::vector<std::complex<double> > &values);

} //namespace test_utilties

#endif /*_DISCAMB_TESTS_CORRECTNESS_TEST_1_TEST_UTILITIES_H_*/
