//#include "ClementRoettiData.h"
#include "discamb/HC_Model/ClementiRoettiData.h"
#include "discamb/BasicUtilities/StringUtilities.h"
#include "discamb/BasicUtilities/OnError.h"
#include "discamb/BasicChemistry/PeriodicTable.h"

#include <iostream>

using namespace std;


namespace { // DATA

static const size_t localDef_N_Entries = 68;

string cr_configuration_data[68] = {
    " H   1S(1)",
    "He   1S(2)",
    "Li   1S(2)2S(1)",
    "Be   1S(2)2S(2)",
    " B   1S(2)2S(2)2P(1)",
    " C   1S(2)2S(2)2P(2)",
    " N   1S(2)2S(2)2P(3)",
    " O   1S(2)2S(2)2P(4)",
    " F   1S(2)2S(2)2P(5)",
    "Ne   1S(2)2S(2)2P(6)",
    "Na   1S(2)2S(2)2P(6)3S(1)",
    "Mg   1S(2)2S(2)2P(6)3S(2)",
    "Al   1S(2)2S(2)2P(6)3S(2)3P(1)",
    "Si   1S(2)2S(2)2P(6)3S(2)3P(2)",
    " P   1S(2)2S(2)2P(6)3S(2)3P(3)",
    " S   1S(2)2S(2)2P(6)3S(2)3P(4)",
    "Cl   1S(2)2S(2)2P(6)3S(2)3P(5)",
    "Ar   1S(2)2S(2)2P(6)3S(2)3P(6)",
    " K   K(2)L(8)3S(2)3P(6)4S(1)",
    "Ca   K(2)L(8)3S(2)3P(6)4S(2)",
    "Sc   K(2)L(8)3S(2)3P(6)4S(2)3D(1)",
    "Ti   K(2)L(8)3S(2)3P(6)4S(2)3D(2)",
    " V   K(2)L(8)3S(2)3P(6)4S(2)3D(3)",
    "Cr   K(2)L(8)3S(2)3P(6)4S(1)3D(5)",
    "Mn   K(2)L(8)3S(2)3P(6)4S(2)3D(5)",
    "Fe   K(2)L(8)3S(2)3P(6)4S(2)3D(6)",
    "Co   K(2)L(8)3S(2)3P(6)4S(2)3D(7)",
    "Ni   K(2)L(8)3S(2)3P(6)4S(2)3D(8)",
    "Cu   K(2)L(8)3S(2)3P(6)4S(1)3D(10)",
    "Zn   K(2)L(8)3S(2)3P(6)4S(2)3D(10)",
    "Ga   K(2)L(8)M(18)4S(2)4P(1)",
    "Ge   K(2)L(8)M(18)4S(2)4P(2)",
    "As   K(2)L(8)M(18)4S(2)4P(3)",
    "Se   K(2)L(8)M(18)4S(2)4P(4)",
    "Br   K(2)L(8)M(18)4S(2)4P(5)",
    "Kr   K(2)L(8)M(18)4S(2)4P(6)",
    "Li+  1S(2)",
    "Na+  1S(2)2S(2)2P(6)",
    " K+  K(2)L(8)3S(2)3P(6)",
    "Cu+  K(2)L(8)3S(2)3P(6)4S(0)3D(10)",
    "Be2+ 1S(2)",
    "Mg2+ 1S(2)2S(2)2P(6)",
    "Al3+ 1S(2)2S(2)2P(6)",
    "Ca2+ K(2)L(8)3S(2)3P(6)",
    "Sc3+ K(2)L(8)3S(2)3P(6)",
    "Ti4+ K(2)L(8)3S(2)3P(6)",
    "V5+  K(2)L(8)3S(2)3P(6)",
    "Ti3+ K(2)L(8)3S(2)3P(6)4S(0)3D(1)",
    " V4+ K(2)L(8)3S(2)3P(6)4S(0)3D(1)",
    " V3+ K(2)L(8)3S(2)3P(6)4S(0)3D(2)",
    " V2+ K(2)L(8)3S(2)3P(6)4S(0)3D(3)",
    "Cr3+ K(2)L(8)3S(2)3P(6)4S(0)3D(3)",
    "Cr2+ K(2)L(8)3S(2)3P(6)4S(0)3D(4)",
    "Mn3+ K(2)L(8)3S(2)3P(6)4S(0)3D(4)",
    "Mn2+ K(2)L(8)3S(2)3P(6)4S(0)3D(5)",
    "Fe3+ K(2)L(8)3S(2)3P(6)4S(0)3D(5)",
    "Fe2+ K(2)L(8)3S(2)3P(6)4S(0)3D(6)",
    "Co3+ K(2)L(8)3S(2)3P(6)4S(0)3D(6)",
    "Co2+ K(2)L(8)3S(2)3P(6)4S(0)3D(7)",
    "Ni2+ K(2)L(8)3S(2)3P(6)4S(0)3D(8)",
    "Cu2+ K(2)L(8)3S(2)3P(6)4S(0)3D(9)",
    "Zn2+ K(2)L(8)3S(2)3P(6)4S(0)3D(10)",
    " O-  1S(2)2S(2)2P(5)",
    " F-  1S(2)2S(2)2P(6)",
    " S-  K(2)L(8)3S(2)3P(5)",
    "Cl-  K(2)L(8)3S(2)3P(6)",
    "Se-  K(2)L(8)M(18)4S(2)4P(5)",
    "Br-  K(2)L(8)M(18)4S(2)4P(6)" };


struct LocalDef_SlaterTerm
{
	double coefficient;
	size_t powerR;
	double exponent;
};

struct LocalDef_Orbital {
	size_t principal_number;
	size_t azimutal_number;
	size_t nTerms;
	LocalDef_SlaterTerm term[20];
};

struct LocalDef_Wfn {
	std::string label;
	size_t atomicNumber;
	int charge;
	size_t nOrbs;
	LocalDef_Orbital orbs[20];
	
};

LocalDef_Wfn wfns[] =
{
	{
		"H", 1, 0,
		1,
		{
			{
				1, 0, // 1S
				1,
				{
					{ 1.00000,   0,    1.00000 }
				}
			}
		}
	},
	{
		"He", 2, 0,
		1,
		{
			{
				1, 0, // 1S
				5,
				{
					{ 0.76838,   0,    1.41714 },
					{ 0.22346,   0,    2.37682 },
					{ 0.04082,   0,    4.39628 },
					{ -0.00994,   0,    6.52699 },
					{ 0.00230,   0,    7.94252 }
				}
			}
		}
	},
	{
		"Li", 3, 0,
		2,
		{
			{
				1, 0, // 1S
				6,
				{
					{ 0.89786,   0,    2.47673 },
					{ 0.11131,   0,    4.69873 },
					{ -0.00008,   1,    0.38350 },
					{ 0.00112,   1,    0.66055 },
					{ -0.00216,   1,    1.07000 },
					{ 0.00884,   1,    1.63200 }
				}
			},
			{
				2, 0, // 2S
				6,
				{
					{ -0.14629,   0,    2.47673 },
					{ -0.01516,   0,    4.69873 },
					{ 0.00377,   1,    0.38350 },
					{ 0.98053,   1,    0.66055 },
					{ 0.10971,   1,    1.07000 },
					{ -0.11021,   1,    1.63200 }
				}
			}
		}
	},
	{
		"Be", 4, 0,
		2,
		{
			{
				1, 0, // 1S
				6,
				{
					{ 0.91796,   0,    3.47116 },
					{ 0.08724,   0,    6.36861 },
					{ 0.00108,   1,    0.77820 },
					{ -0.00199,   1,    0.94067 },
					{ 0.00176,   1,    1.48725 },
					{ 0.00628,   1,    2.71830 }
				}
			},
			{
				2, 0, // 2S
				6,
				{
					{ -0.17092,   0,    3.47116 },
					{ -0.01455,   0,    6.36861 },
					{ 0.21186,   1,    0.77820 },
					{ 0.62499,   1,    0.94067 },
					{ 0.26662,   1,    1.48725 },
					{ -0.09919,   1,    2.71830 }
				}
			}
		}
	},
	{
		"B", 5, 0,
		3,
		{
			{
				1, 0, // 1S
				6,
				{
					{ 0.92705,   0,    4.44561 },
					{ 0.07780,   0,    7.91796 },
					{ 0.00088,   1,    0.86709 },
					{ -0.00200,   1,    1.21924 },
					{ 0.00433,   1,    2.07264 },
					{ 0.00270,   1,    3.44332 }
				}
			},
			{
				2, 0, // 2S
				6,
				{
					{ -0.19484,   0,    4.44561 },
					{ -0.01254,   0,    7.91796 },
					{ 0.06941,   1,    0.86709 },
					{ 0.75234,   1,    1.21924 },
					{ 0.31856,   1,    2.07264 },
					{ -0.12642,   1,    3.44332 }
				}
			},
			{
				2, 1, // 2P
				4,
				{
					{ 0.53622,   1,    0.87481 },
					{ 0.40340,   1,    1.36992 },
					{ 0.11653,   1,    2.32262 },
					{ 0.00821,   1,    5.59481 }
				}
			}
		}
	},
	{
		"C", 6, 0,
		3,
		{
			{
				1, 0, // 1S
				6,
				{
					{ 0.93262,   0,    5.43599 },
					{ 0.06931,   0,    9.48256 },
					{ 0.00083,   1,    1.05749 },
					{ -0.00176,   1,    1.52427 },
					{ 0.00559,   1,    2.68435 },
					{ 0.00382,   1,    4.20096 }
				}
			},
			{
				2, 0, // 2S
				6,
				{
					{ -0.20814,   0,    5.43599 },
					{ -0.01071,   0,    9.48256 },
					{ 0.08099,   1,    1.05749 },
					{ 0.75045,   1,    1.52427 },
					{ 0.33549,   1,    2.68435 },
					{ -0.14765,   1,    4.20096 }
				}
			},
			{
				2, 1, // 2P
				4,
				{
					{ 0.28241,   1,    0.98073 },
					{ 0.54697,   1,    1.44361 },
					{ 0.23195,   1,    2.60051 },
					{ 0.01025,   1,    6.51003 }
				}
			}
		}
	},
	{
		"N", 7, 0,
		3,
		{
			{
				1, 0, // 1S
				6,
				{
					{ 0.93780,   0,    6.45739 },
					{ 0.05849,   0,   11.17200 },
					{ 0.00093,   1,    1.36405 },
					{ -0.00170,   1,    1.89734 },
					{ 0.00574,   1,    3.25291 },
					{ 0.00957,   1,    5.08238 }
				}
			},
			{
				2, 0, // 2S
				6,
				{
					{ -0.21677,   0,    6.45739 },
					{ -0.00846,   0,   11.17200 },
					{ 0.17991,   1,    1.36405 },
					{ 0.67416,   1,    1.89734 },
					{ 0.31297,   1,    3.25291 },
					{ -0.14497,   1,    5.08238 }
				}
			},
			{
				2, 1, // 2P
				4,
				{
					{ 0.26639,   1,    1.16068 },
					{ 0.52319,   1,    1.70472 },
					{ 0.27353,   1,    3.03935 },
					{ 0.01292,   1,    7.17482 }
				}
			}
		}
	},
	{
		"O", 8, 0,
		3,
		{
			{
				1, 0, // 1S
				6,
				{
					{ 0.94516,   0,    7.61413 },
					{ 0.03391,   0,   13.75740 },
					{ -0.00034,   1,    1.69824 },
					{ 0.00241,   1,    2.48022 },
					{ -0.00486,   1,    4.31196 },
					{ 0.03681,   1,    5.86596 }
				}
			},
			{
				2, 0, // 2S
				6,
				{
					{ -0.22157,   0,    7.61413 },
					{ -0.00476,   0,   13.75740 },
					{ 0.34844,   1,    1.69824 },
					{ 0.60807,   1,    2.48022 },
					{ 0.25365,   1,    4.31196 },
					{ -0.19183,   1,    5.86596 }
				}
			},
			{
				2, 1, // 2P
				4,
				{
					{ 0.16922,   1,    1.14394 },
					{ 0.57974,   1,    1.81730 },
					{ 0.32352,   1,    3.44988 },
					{ 0.01660,   1,    7.56484 }
				}
			}
		}
	},
	{
		"F", 9, 0,
		3,
		{
			{
				1, 0, // 1S
				6,
				{
					{ 0.94710,   0,    8.55760 },
					{ 0.03718,   0,   14.97660 },
					{ 0.00013,   1,    1.82142 },
					{ 0.00093,   1,    2.67295 },
					{ 0.00068,   1,    4.90066 },
					{ 0.02602,   1,    6.57362 }
				}
			},
			{
				2, 0, // 2S
				6,
				{
					{ -0.22694,   0,    8.55760 },
					{ -0.00530,   0,   14.97660 },
					{ 0.23918,   1,    1.82142 },
					{ 0.68592,   1,    2.67295 },
					{ 0.31489,   1,    4.90066 },
					{ -0.21822,   1,    6.57362 }
				}
			},
			{
				2, 1, // 2P
				4,
				{
					{ 0.17830,   1,    1.26570 },
					{ 0.56185,   1,    2.05803 },
					{ 0.33658,   1,    3.92853 },
					{ 0.01903,   1,    8.20412 }
				}
			}
		}
	},
	{
		"Ne", 10, 0,
		3,
		{
			{
				1, 0, // 1S
				6,
				{
					{ 0.93717,   0,    9.48486 },
					{ 0.04899,   0,   15.56590 },
					{ 0.00058,   1,    1.96184 },
					{ -0.00064,   1,    2.86423 },
					{ 0.00551,   1,    4.82530 },
					{ 0.01999,   1,    7.79242 }
				}
			},
			{
				2, 0, // 2S
				6,
				{
					{ -0.23093,   0,    9.48486 },
					{ -0.00635,   0,   15.56590 },
					{ 0.18620,   1,    1.96184 },
					{ 0.66899,   1,    2.86423 },
					{ 0.30910,   1,    4.82530 },
					{ -0.13871,   1,    7.79242 }
				}
			},
			{
				2, 1, // 2P
				4,
				{
					{ 0.21799,   1,    1.45208 },
					{ 0.53338,   1,    2.38168 },
					{ 0.32933,   1,    4.48489 },
					{ 0.01872,   1,    9.13464 }
				}
			}
		}
	},
	{
		"Na", 11, 0,
		4,
		{
			{
				1, 0, // 1S
				8,
				{
					{ 0.96179,   0,   11.01230 },
					{ 0.04052,   2,   12.66010 },
					{ 0.01919,   2,    8.36156 },
					{ -0.00298,   2,    5.73805 },
					{ 0.00191,   2,    3.61287 },
					{ -0.00049,   2,    2.25096 },
					{ 0.00016,   2,    1.11597 },
					{ -0.00007,   2,    0.71028 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ -0.23474,   0,   11.01230 },
					{ -0.00606,   2,   12.66010 },
					{ 0.11154,   2,    8.36156 },
					{ 0.43179,   2,    5.73805 },
					{ 0.51700,   2,    3.61287 },
					{ 0.04747,   2,    2.25096 },
					{ -0.00324,   2,    1.11597 },
					{ 0.00124,   2,    0.71028 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.03527,   0,   11.01230 },
					{ 0.00121,   2,   12.66010 },
					{ -0.01889,   2,    8.36156 },
					{ -0.06808,   2,    5.73805 },
					{ -0.09232,   2,    3.61287 },
					{ 0.00076,   2,    2.25096 },
					{ 0.40764,   2,    1.11597 },
					{ 0.64467,   2,    0.71028 }
				}
			},
			{
				2, 1, // 2P
				5,
				{
					{ 0.46417,   1,    5.54977 },
					{ 0.03622,   3,    8.66846 },
					{ 0.29282,   3,    5.43460 },
					{ 0.31635,   3,    3.55503 },
					{ 0.07543,   3,    2.31671 }
				}
			}
		}
	},
	{
		"Mg", 12, 0,
		4,
		{
			{
				1, 0, // 1S
				8,
				{
					{ 0.96430,   0,   12.01140 },
					{ 0.03548,   2,   13.91620 },
					{ 0.02033,   2,    9.48612 },
					{ -0.00252,   2,    6.72188 },
					{ 0.00162,   2,    4.24466 },
					{ -0.00038,   2,    2.53466 },
					{ 0.00015,   2,    1.46920 },
					{ -0.00004,   2,    0.89084 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ -0.24357,   0,   12.01140 },
					{ -0.00485,   2,   13.91620 },
					{ 0.08002,   2,    9.48612 },
					{ 0.39902,   2,    6.72188 },
					{ 0.57358,   2,    4.24466 },
					{ 0.05156,   2,    2.53466 },
					{ -0.00703,   2,    1.46920 },
					{ 0.00161,   2,    0.89084 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.04691,   0,   12.01140 },
					{ 0.00144,   2,   13.91620 },
					{ -0.01850,   2,    9.48612 },
					{ -0.07964,   2,    6.72188 },
					{ -0.13478,   2,    4.24466 },
					{ -0.01906,   2,    2.53466 },
					{ 0.48239,   2,    1.46920 },
					{ 0.60221,   2,    0.89084 }
				}
			},
			{
				2, 1, // 2P
				5,
				{
					{ 0.52391,   1,    5.92580 },
					{ 0.07012,   3,    7.98979 },
					{ 0.31965,   3,    5.32964 },
					{ 0.20860,   3,    3.71678 },
					{ 0.03888,   3,    2.59986 }
				}
			}
		}
	},
	{
		"Al", 13, 0,
		5,
		{
			{
				1, 0, // 1S
				8,
				{
					{ 0.96628,   0,   13.01270 },
					{ 0.03266,   2,   15.15090 },
					{ 0.02155,   2,   10.19020 },
					{ -0.00439,   2,    7.25565 },
					{ 0.00247,   2,    4.70291 },
					{ -0.00081,   2,    2.96182 },
					{ 0.00030,   2,    1.77250 },
					{ -0.00008,   2,    1.09108 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ -0.25110,   0,   13.01270 },
					{ -0.00424,   2,   15.15090 },
					{ 0.09248,   2,   10.19020 },
					{ 0.40331,   2,    7.25565 },
					{ 0.56148,   2,    4.70291 },
					{ 0.03902,   2,    2.96182 },
					{ -0.00231,   2,    1.77250 },
					{ 0.00037,   2,    1.09108 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.05795,   0,   13.01270 },
					{ 0.00149,   2,   15.15090 },
					{ -0.02529,   2,   10.19020 },
					{ -0.09664,   2,    7.25565 },
					{ -0.16426,   2,    4.70291 },
					{ -0.03278,   2,    2.96182 },
					{ 0.57610,   2,    1.77250 },
					{ 0.52921,   2,    1.09108 }
				}
			},
			{
				2, 1, // 2P
				8,
				{
					{ 0.53767,   1,    6.51941 },
					{ 0.01909,   3,   10.48190 },
					{ 0.06545,   3,    8.08430 },
					{ 0.32423,   3,    5.85779 },
					{ 0.19557,   3,    3.99863 },
					{ 0.00829,   3,    2.01453 },
					{ -0.00235,   3,    1.25016 },
					{ 0.00101,   3,    0.87219 }
				}
			},
			{
				3, 1, // 3P
				8,
				{
					{ -0.09633,   1,    6.51941 },
					{ -0.00661,   3,   10.48190 },
					{ -0.00128,   3,    8.08430 },
					{ -0.06987,   3,    5.85779 },
					{ 0.01653,   3,    3.99863 },
					{ 0.35563,   3,    2.01453 },
					{ 0.45241,   3,    1.25016 },
					{ 0.29407,   3,    0.87219 }
				}
			}
		}
	},
	{
		"Si", 14, 0,
		5,
		{
			{
				1, 0, // 1S
				8,
				{
					{ 0.96800,   0,   14.01420 },
					{ 0.03033,   2,   16.39320 },
					{ 0.02248,   2,   10.87950 },
					{ -0.00617,   2,    7.72709 },
					{ 0.00326,   2,    5.16500 },
					{ -0.00143,   2,    2.97451 },
					{ 0.00081,   2,    2.14316 },
					{ -0.00016,   2,    1.31306 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ -0.25755,   0,   14.01420 },
					{ -0.00446,   2,   16.39320 },
					{ 0.11153,   2,   10.87950 },
					{ 0.40339,   2,    7.72709 },
					{ 0.55032,   2,    5.16500 },
					{ 0.03381,   2,    2.97451 },
					{ -0.00815,   2,    2.14316 },
					{ 0.00126,   2,    1.31306 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.06595,   0,   14.01420 },
					{ 0.00185,   2,   16.39320 },
					{ -0.03461,   2,   10.87950 },
					{ -0.10378,   2,    7.72709 },
					{ -0.19229,   2,    5.16500 },
					{ -0.06561,   2,    2.97451 },
					{ 0.59732,   2,    2.14316 },
					{ 0.55390,   2,    1.31306 }
				}
			},
			{
				2, 1, // 2P
				8,
				{
					{ 0.54290,   1,    7.14360 },
					{ 0.00234,   3,   16.25720 },
					{ 0.04228,   3,   10.79720 },
					{ 0.32155,   3,    6.89724 },
					{ 0.22474,   3,    4.66598 },
					{ 0.00732,   3,    2.32046 },
					{ -0.00105,   3,    1.33470 },
					{ 0.00041,   3,    0.79318 }
				}
			},
			{
				3, 1, // 3P
				8,
				{
					{ -0.11535,   1,    7.14360 },
					{ -0.00189,   3,   16.25720 },
					{ -0.00473,   3,   10.79720 },
					{ -0.07552,   3,    6.89724 },
					{ 0.01041,   3,    4.66598 },
					{ 0.46075,   3,    2.32046 },
					{ 0.57665,   3,    1.33470 },
					{ 0.06274,   3,    0.79318 }
				}
			}
		}
	},
	{
		"P", 15, 0,
		5,
		{
			{
				1, 0, // 1S
				8,
				{
					{ 0.96992,   0,   15.01120 },
					{ 0.02944,   2,   17.31520 },
					{ 0.01933,   2,   11.77300 },
					{ -0.00403,   2,    8.66300 },
					{ 0.00196,   2,    5.90778 },
					{ -0.00051,   2,    3.69253 },
					{ 0.00016,   2,    2.47379 },
					{ -0.00004,   2,    1.51103 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ -0.26326,   0,   15.01120 },
					{ -0.00434,   2,   17.31520 },
					{ 0.10333,   2,   11.77300 },
					{ 0.34612,   2,    8.66300 },
					{ 0.58778,   2,    5.90778 },
					{ 0.06043,   2,    3.69253 },
					{ -0.00901,   2,    2.47379 },
					{ 0.00193,   2,    1.51103 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.07230,   0,   15.01120 },
					{ 0.00186,   2,   17.31520 },
					{ -0.03447,   2,   11.77300 },
					{ -0.09503,   2,    8.66300 },
					{ -0.21241,   2,    5.90778 },
					{ -0.09001,   2,    3.69253 },
					{ 0.60361,   2,    2.47379 },
					{ 0.56185,   2,    1.51103 }
				}
			},
			{
				2, 1, // 2P
				8,
				{
					{ 0.57352,   1,    7.60940 },
					{ 0.00664,   3,   13.97590 },
					{ 0.02478,   3,   11.89390 },
					{ 0.30460,   3,    7.55531 },
					{ 0.21442,   3,    5.17707 },
					{ 0.00552,   3,    2.62934 },
					{ -0.00045,   3,    1.50494 },
					{ 0.00011,   3,    0.77783 }
				}
			},
			{
				3, 1, // 3P
				8,
				{
					{ -0.13569,   1,    7.60940 },
					{ -0.00813,   3,   13.97590 },
					{ 0.00586,   3,   11.89390 },
					{ -0.08424,   3,    7.55531 },
					{ 0.02002,   3,    5.17707 },
					{ 0.51314,   3,    2.62934 },
					{ 0.55176,   3,    1.50494 },
					{ 0.02781,   3,    0.77783 }
				}
			}
		}
	},
	{
		"S", 16, 0,
		5,
		{
			{
				1, 0, // 1S
				8,
				{
					{ 0.97077,   0,   16.01910 },
					{ 0.02122,   2,   19.45290 },
					{ 0.02511,   2,   13.77880 },
					{ -0.00287,   2,    9.85150 },
					{ 0.00194,   2,    6.48374 },
					{ -0.00068,   2,    4.05540 },
					{ 0.00025,   2,    2.67399 },
					{ -0.00007,   2,    1.66032 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ -0.26808,   0,   16.01910 },
					{ -0.00390,   2,   19.45290 },
					{ 0.04869,   2,   13.77880 },
					{ 0.36323,   2,    9.85150 },
					{ 0.62663,   2,    6.48374 },
					{ 0.05414,   2,    4.05540 },
					{ -0.00517,   2,    2.67399 },
					{ 0.00135,   2,    1.66032 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.07778,   0,   16.01910 },
					{ 0.00244,   2,   19.45290 },
					{ -0.02131,   2,   13.77880 },
					{ -0.10107,   2,    9.85150 },
					{ -0.25158,   2,    6.48374 },
					{ -0.06277,   2,    4.05540 },
					{ 0.65107,   2,    2.67399 },
					{ 0.50175,   2,    1.66032 }
				}
			},
			{
				2, 1, // 2P
				8,
				{
					{ 0.61294,   1,    8.00375 },
					{ -0.75360,   3,    7.67346 },
					{ 0.03668,   3,   12.09980 },
					{ 1.07415,   3,    7.67727 },
					{ 0.14538,   3,    5.32630 },
					{ 0.00199,   3,    2.87350 },
					{ 0.00094,   3,    1.67242 },
					{ -0.00014,   3,    1.03092 }
				}
			},
			{
				3, 1, // 3P
				8,
				{
					{ -0.15586,   1,    8.00375 },
					{ -18.50193,   3,    7.67346 },
					{ -0.01199,   3,   12.09980 },
					{ 18.39285,   3,    7.67727 },
					{ 0.09041,   3,    5.32630 },
					{ 0.52697,   3,    2.87350 },
					{ 0.49465,   3,    1.67242 },
					{ 0.05112,   3,    1.03092 }
				}
			}
		}
	},
	{
		"Cl", 17, 0,
		5,
		{
			{
				1, 0, // 1S
				8,
				{
					{ 0.97335,   0,   17.00140 },
					{ 0.02682,   2,   19.26490 },
					{ 0.01612,   2,   13.45290 },
					{ -0.00266,   2,   10.04290 },
					{ 0.00129,   2,    6.93920 },
					{ -0.00029,   2,    4.43640 },
					{ 0.00005,   2,    2.90570 },
					{ -0.00002,   2,    1.81900 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ -0.27278,   0,   17.00140 },
					{ -0.00266,   2,   19.26490 },
					{ 0.09766,   2,   13.45290 },
					{ 0.34603,   2,   10.04290 },
					{ 0.59594,   2,    6.93920 },
					{ 0.04978,   2,    4.43640 },
					{ -0.00324,   2,    2.90570 },
					{ 0.00121,   2,    1.81900 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.08249,   0,   17.00140 },
					{ 0.00237,   2,   19.26490 },
					{ -0.04193,   2,   13.45290 },
					{ -0.08968,   2,   10.04290 },
					{ -0.27243,   2,    6.93920 },
					{ -0.03736,   2,    4.43640 },
					{ 0.67062,   2,    2.90570 },
					{ 0.47342,   2,    1.81900 }
				}
			},
			{
				2, 1, // 2P
				8,
				{
					{ 0.63254,   1,    8.50000 },
					{ 0.00287,   3,   15.01240 },
					{ 0.03393,   3,   12.32570 },
					{ 0.27156,   3,    8.37240 },
					{ 0.16389,   3,    6.10920 },
					{ 0.00707,   3,    3.19310 },
					{ -0.00034,   3,    1.78630 },
					{ 0.00036,   3,    0.92930 }
				}
			},
			{
				3, 1, // 3P
				8,
				{
					{ -0.16954,   1,    8.50000 },
					{ -0.00982,   3,   15.01240 },
					{ 0.01280,   3,   12.32570 },
					{ -0.10925,   3,    8.37240 },
					{ 0.07066,   3,    6.10920 },
					{ 0.56909,   3,    3.19310 },
					{ 0.49144,   3,    1.78630 },
					{ 0.02336,   3,    0.92930 }
				}
			}
		}
	},
	{
		"Ar", 18, 0,
		5,
		{
			{
				1, 0, // 1S
				8,
				{
					{ 0.97349,   0,   18.01640 },
					{ 0.01684,   2,   22.04650 },
					{ 0.02422,   2,   16.08250 },
					{ -0.00114,   2,   11.63570 },
					{ 0.00123,   2,    7.70365 },
					{ -0.00039,   2,    4.87338 },
					{ 0.00010,   2,    3.32987 },
					{ -0.00003,   2,    2.02791 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ 0.27635,   0,   18.01640 },
					{ 0.00289,   2,   22.04650 },
					{ -0.03241,   2,   16.08250 },
					{ -0.33229,   2,   11.63570 },
					{ -0.65828,   2,    7.70365 },
					{ -0.06834,   2,    4.87338 },
					{ 0.00623,   2,    3.32987 },
					{ -0.00174,   2,    2.02791 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.08634,   0,   18.01640 },
					{ 0.00186,   2,   22.04650 },
					{ -0.01540,   2,   16.08250 },
					{ -0.10236,   2,   11.63570 },
					{ -0.27614,   2,    7.70365 },
					{ -0.11879,   2,    4.87338 },
					{ 0.68436,   2,    3.32987 },
					{ 0.52050,   2,    2.02791 }
				}
			},
			{
				2, 1, // 2P
				8,
				{
					{ 0.64116,   1,    9.05477 },
					{ 0.00865,   3,   15.54410 },
					{ 0.04186,   3,   12.39970 },
					{ 0.31735,   3,    8.56120 },
					{ 0.09642,   3,    5.94658 },
					{ 0.00003,   3,    3.42459 },
					{ 0.00055,   3,    1.96709 },
					{ -0.00013,   3,    1.06717 }
				}
			},
			{
				3, 1, // 3P
				8,
				{
					{ -0.17850,   1,    9.05477 },
					{ -0.00812,   3,   15.54410 },
					{ 0.00520,   3,   12.39970 },
					{ -0.10986,   3,    8.56120 },
					{ 0.10994,   3,    5.94658 },
					{ 0.56149,   3,    3.42459 },
					{ 0.46314,   3,    1.96709 },
					{ 0.02951,   3,    1.06717 }
				}
			}
		}
	},
	{
		"K", 19, 0,
		6,
		{
			{
				1, 0, // 1S
				11,
				{
					{ -0.93619,   0,   19.13500 },
					{ -0.01385,   0,   31.52500 },
					{ -0.06342,   1,   16.49860 },
					{ -0.00014,   1,    7.67410 },
					{ -0.00139,   2,    6.68508 },
					{ 0.00189,   2,    4.04102 },
					{ -0.00212,   2,    2.66919 },
					{ 0.00118,   3,    2.59794 },
					{ -0.00005,   3,    0.56203 },
					{ -0.00015,   3,    1.29017 },
					{ 0.00011,   3,    0.76641 }
				}
			},
			{
				2, 0, // 2S
				11,
				{
					{ 0.27612,   0,   19.13500 },
					{ 0.00055,   0,   31.52500 },
					{ 0.14725,   1,   16.49860 },
					{ -0.95199,   1,    7.67410 },
					{ -0.19289,   2,    6.68508 },
					{ -0.00059,   2,    4.04102 },
					{ -0.00704,   2,    2.66919 },
					{ 0.00327,   3,    2.59794 },
					{ -0.00016,   3,    0.56203 },
					{ -0.00045,   3,    1.29017 },
					{ 0.00033,   3,    0.76641 }
				}
			},
			{
				3, 0, // 3S
				11,
				{
					{ 0.09267,   0,   19.13500 },
					{ -0.00066,   0,   31.52500 },
					{ 0.04980,   1,   16.49860 },
					{ -0.33547,   1,    7.67410 },
					{ -0.21345,   2,    6.68508 },
					{ 0.43855,   2,    4.04102 },
					{ 0.65200,   2,    2.66919 },
					{ 0.09749,   3,    2.59794 },
					{ 0.00560,   3,    0.56203 },
					{ 0.01932,   3,    1.29017 },
					{ -0.01161,   3,    0.76641 }
				}
			},
			{
				4, 0, // 4S
				11,
				{
					{ -0.01825,   0,   19.13500 },
					{ 0.00031,   0,   31.52500 },
					{ -0.00899,   1,   16.49860 },
					{ 0.06350,   1,    7.67410 },
					{ 0.05015,   2,    6.68508 },
					{ -0.11346,   2,    4.04102 },
					{ -0.11474,   2,    2.66919 },
					{ -0.03065,   3,    2.59794 },
					{ 0.05190,   3,    0.56203 },
					{ 0.33431,   3,    1.29017 },
					{ 0.70417,   3,    0.76641 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.66743,   1,    8.64187 },
					{ 0.04207,   1,   15.19360 },
					{ 0.34752,   2,    6.91359 },
					{ 0.01398,   2,    3.26163 },
					{ -0.00944,   2,    2.00984 },
					{ 0.00526,   2,    1.68876 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.20797,   1,    8.64187 },
					{ -0.01176,   1,   15.19360 },
					{ -0.12744,   2,    6.91359 },
					{ 0.56718,   2,    3.26163 },
					{ 0.45273,   2,    2.00984 },
					{ 0.09340,   2,    1.68876 }
				}
			}
		}
	},
	{
		"Ca", 20, 0,
		6,
		{
			{
				1, 0, // 1S
				11,
				{
					{ -0.94194,   0,   19.97310 },
					{ -0.01803,   0,   32.15000 },
					{ -0.05072,   1,   17.23940 },
					{ -0.00216,   1,    8.40361 },
					{ 0.00051,   2,    7.46907 },
					{ -0.00010,   2,    3.95936 },
					{ 0.00017,   2,    3.01657 },
					{ -0.00006,   3,    3.09105 },
					{ -0.00000,   3,    1.62884 },
					{ 0.00000,   3,    1.01203 },
					{ -0.00000,   3,    0.66733 }
				}
			},
			{
				2, 0, // 2S
				11,
				{
					{ -0.28216,   0,   19.97310 },
					{ -0.00093,   0,   32.15000 },
					{ -0.15034,   1,   17.23940 },
					{ 0.89604,   1,    8.40361 },
					{ 0.25659,   2,    7.46907 },
					{ 0.00978,   2,    3.95936 },
					{ 0.00087,   2,    3.01657 },
					{ -0.00173,   3,    3.09105 },
					{ 0.00051,   3,    1.62884 },
					{ -0.00029,   3,    1.01203 },
					{ 0.00010,   3,    0.66733 }
				}
			},
			{
				3, 0, // 3S
				11,
				{
					{ -0.09896,   0,   19.97310 },
					{ 0.00053,   0,   32.15000 },
					{ -0.05334,   1,   17.23940 },
					{ 0.32943,   1,    8.40361 },
					{ 0.22440,   2,    7.46907 },
					{ -0.70524,   2,    3.95936 },
					{ -0.12144,   2,    3.01657 },
					{ -0.35525,   3,    3.09105 },
					{ -0.01549,   3,    1.62884 },
					{ 0.00566,   3,    1.01203 },
					{ -0.00179,   3,    0.66733 }
				}
			},
			{
				4, 0, // 4S
				11,
				{
					{ -0.02394,   0,   19.97310 },
					{ 0.00028,   0,   32.15000 },
					{ -0.01223,   1,   17.23940 },
					{ 0.07777,   1,    8.40361 },
					{ 0.06258,   2,    7.46907 },
					{ -0.24085,   2,    3.95936 },
					{ 0.05280,   2,    3.01657 },
					{ -0.15243,   3,    3.09105 },
					{ 0.32699,   3,    1.62884 },
					{ 0.66264,   3,    1.01203 },
					{ 0.13150,   3,    0.66733 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.67837,   1,    9.12915 },
					{ 0.04233,   1,   15.99470 },
					{ 0.33353,   2,    7.37779 },
					{ 0.01241,   2,    3.62974 },
					{ -0.00454,   2,    2.34862 },
					{ 0.00132,   2,    1.64038 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.22553,   1,    9.12915 },
					{ -0.01276,   1,   15.99470 },
					{ -0.13716,   2,    7.37779 },
					{ 0.54465,   2,    3.62974 },
					{ 0.52593,   2,    2.34862 },
					{ 0.04492,   2,    1.64038 }
				}
			}
		}
	},
	{
		"Sc", 21, 0,
		7,
		{
			{
				1, 0, // 1S
				11,
				{
					{ -0.94540,   0,   20.88130 },
					{ -0.02010,   0,   33.43000 },
					{ -0.04294,   1,   18.18780 },
					{ -0.00497,   1,    8.42800 },
					{ 0.00385,   2,    7.45830 },
					{ -0.00392,   2,    4.78240 },
					{ 0.00457,   2,    3.24065 },
					{ -0.00244,   3,    3.25855 },
					{ 0.00021,   3,    1.54276 },
					{ -0.00020,   3,    0.96985 },
					{ 0.00010,   3,    0.76062 }
				}
			},
			{
				2, 0, // 2S
				11,
				{
					{ 0.28654,   0,   20.88130 },
					{ 0.00116,   0,   33.43000 },
					{ 0.14707,   1,   18.18780 },
					{ -1.01154,   1,    8.42800 },
					{ -0.12762,   2,    7.45830 },
					{ -0.00992,   2,    4.78240 },
					{ 0.00040,   2,    3.24065 },
					{ -0.00051,   3,    3.25855 },
					{ -0.00008,   3,    1.54276 },
					{ 0.00008,   3,    0.96985 },
					{ -0.00004,   3,    0.76062 }
				}
			},
			{
				3, 0, // 3S
				11,
				{
					{ -0.10222,   0,   20.88130 },
					{ 0.00026,   0,   33.43000 },
					{ -0.05463,   1,   18.18780 },
					{ 0.38509,   1,    8.42800 },
					{ 0.22889,   2,    7.45830 },
					{ -0.45514,   2,    4.78240 },
					{ -0.63181,   2,    3.24065 },
					{ -0.13662,   3,    3.25855 },
					{ -0.01874,   3,    1.54276 },
					{ 0.01435,   3,    0.96985 },
					{ -0.00712,   3,    0.76062 }
				}
			},
			{
				4, 0, // 4S
				11,
				{
					{ -0.02400,   0,   20.88130 },
					{ -0.00007,   0,   33.43000 },
					{ -0.01351,   1,   18.18780 },
					{ 0.09325,   1,    8.42800 },
					{ 0.05788,   2,    7.45830 },
					{ -0.10274,   2,    4.78240 },
					{ -0.23446,   2,    3.24065 },
					{ 0.02272,   3,    3.25855 },
					{ 0.48631,   3,    1.54276 },
					{ 0.50614,   3,    0.96985 },
					{ 0.10593,   3,    0.76062 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.68935,   1,    9.58828 },
					{ 0.04459,   1,   16.63600 },
					{ 0.31660,   2,    7.83942 },
					{ 0.01293,   2,    4.10068 },
					{ -0.00397,   2,    2.86710 },
					{ 0.00096,   2,    2.08891 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.23574,   1,    9.58828 },
					{ -0.01406,   1,   16.63600 },
					{ -0.14208,   2,    7.83942 },
					{ 0.43727,   2,    4.10068 },
					{ 0.49413,   2,    2.86710 },
					{ 0.19247,   2,    2.08891 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.31357,   2,    3.60600 },
					{ 0.02663,   2,    8.63190 },
					{ 0.06141,   2,    4.92190 },
					{ 0.56780,   2,    1.88940 },
					{ 0.20271,   2,    1.01000 }
				}
			}
		}
	},
	{
		"Ti", 22, 0,
		7,
		{
			{
				1, 0, // 1S
				11,
				{
					{ -0.95015,   0,   22.02960 },
					{ -0.01209,   0,   37.11330 },
					{ -0.04928,   1,   18.60820 },
					{ 0.00058,   1,    9.39745 },
					{ -0.00140,   2,    8.17280 },
					{ 0.00187,   2,    4.98509 },
					{ -0.00187,   2,    3.62802 },
					{ 0.00080,   3,    3.45561 },
					{ -0.00009,   3,    1.58708 },
					{ 0.00008,   3,    1.00652 },
					{ -0.00004,   3,    0.75064 }
				}
			},
			{
				2, 0, // 2S
				11,
				{
					{ -0.28972,   0,   22.02960 },
					{ 0.00055,   0,   37.11330 },
					{ -0.16752,   1,   18.60820 },
					{ 0.93056,   1,    9.39745 },
					{ 0.24986,   2,    8.17280 },
					{ -0.02361,   2,    4.98509 },
					{ 0.03254,   2,    3.62802 },
					{ -0.01329,   3,    3.45561 },
					{ 0.00159,   3,    1.58708 },
					{ -0.00137,   3,    1.00652 },
					{ 0.00064,   3,    0.75064 }
				}
			},
			{
				3, 0, // 3S
				11,
				{
					{ 0.10510,   0,   22.02960 },
					{ -0.00094,   0,   37.11330 },
					{ 0.06192,   1,   18.60820 },
					{ -0.35502,   1,    9.39745 },
					{ -0.26666,   2,    8.17280 },
					{ 0.47346,   2,    4.98509 },
					{ 0.50703,   2,    3.62802 },
					{ 0.24381,   3,    3.45561 },
					{ 0.01578,   3,    1.58708 },
					{ -0.01098,   3,    1.00652 },
					{ 0.00487,   3,    0.75064 }
				}
			},
			{
				4, 0, // 4S
				11,
				{
					{ -0.02387,   0,   22.02960 },
					{ -0.00004,   0,   37.11330 },
					{ -0.01572,   1,   18.60820 },
					{ 0.08607,   1,    9.39745 },
					{ 0.05985,   2,    8.17280 },
					{ -0.07272,   2,    4.98509 },
					{ -0.24523,   2,    3.62802 },
					{ 0.02662,   3,    3.45561 },
					{ 0.50408,   3,    1.58708 },
					{ 0.48670,   3,    1.00652 },
					{ 0.10196,   3,    0.75064 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.71614,   1,    9.74943 },
					{ 0.06307,   1,   16.39080 },
					{ 0.26624,   2,    8.15953 },
					{ 0.01033,   2,    4.50482 },
					{ -0.00126,   2,    2.84111 },
					{ 0.00022,   2,    1.51430 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.24806,   1,    9.74943 },
					{ -0.02143,   1,   16.39080 },
					{ -0.14149,   2,    8.15953 },
					{ 0.46677,   2,    4.50482 },
					{ 0.64339,   2,    2.84111 },
					{ 0.03031,   2,    1.51430 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.52678,   2,    2.17900 },
					{ 0.02539,   2,    9.54575 },
					{ 0.17639,   2,    4.81759 },
					{ 0.21326,   2,    3.67072 },
					{ 0.21819,   2,    1.23154 }
				}
			}
		}
	},
	{
		"V", 23, 0,
		7,
		{
			{
				1, 0, // 1S
				11,
				{
					{ -0.94977,   0,   23.05740 },
					{ -0.01180,   0,   39.01750 },
					{ -0.04848,   1,   19.85370 },
					{ -0.00233,   1,    9.24671 },
					{ 0.00137,   2,    8.54632 },
					{ -0.00169,   2,    5.11726 },
					{ 0.00165,   2,    4.22281 },
					{ -0.00038,   3,    3.67054 },
					{ 0.00005,   3,    1.76174 },
					{ -0.00004,   3,    1.07700 },
					{ 0.00001,   3,    0.73215 }
				}
			},
			{
				2, 0, // 2S
				11,
				{
					{ -0.29138,   0,   23.05740 },
					{ 0.00046,   0,   39.01750 },
					{ -0.15630,   1,   19.85370 },
					{ 1.05201,   1,    9.24671 },
					{ 0.07670,   2,    8.54632 },
					{ 0.05858,   2,    5.11726 },
					{ -0.04248,   2,    4.22281 },
					{ 0.00932,   3,    3.67054 },
					{ -0.00105,   3,    1.76174 },
					{ 0.00069,   3,    1.07700 },
					{ -0.00028,   3,    0.73215 }
				}
			},
			{
				3, 0, // 3S
				11,
				{
					{ 0.10566,   0,   23.05740 },
					{ -0.00031,   0,   39.01750 },
					{ 0.06121,   1,   19.85370 },
					{ -0.41455,   1,    9.24671 },
					{ -0.19175,   2,    8.54632 },
					{ 0.45614,   2,    5.11726 },
					{ 0.42379,   2,    4.22281 },
					{ 0.34028,   3,    3.67054 },
					{ 0.01151,   3,    1.76174 },
					{ -0.00529,   3,    1.07700 },
					{ 0.00195,   3,    0.73215 }
				}
			},
			{
				4, 0, // 4S
				11,
				{
					{ -0.02384,   0,   23.05740 },
					{ -0.00001,   0,   39.01750 },
					{ -0.01436,   1,   19.85370 },
					{ 0.09588,   1,    9.24671 },
					{ 0.04743,   2,    8.54632 },
					{ -0.08873,   2,    5.11726 },
					{ -0.16412,   2,    4.22281 },
					{ -0.05043,   3,    3.67054 },
					{ 0.44160,   3,    1.76174 },
					{ 0.57980,   3,    1.07700 },
					{ 0.08713,   3,    0.73215 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.72031,   1,   10.33610 },
					{ 0.05657,   1,   17.45840 },
					{ 0.26705,   2,    8.65722 },
					{ 0.01107,   2,    4.79067 },
					{ -0.00158,   2,    3.05940 },
					{ 0.00038,   2,    1.85188 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.25528,   1,   10.33610 },
					{ -0.01900,   1,   17.45840 },
					{ -0.14260,   2,    8.65722 },
					{ 0.47076,   2,    4.79067 },
					{ 0.62323,   2,    3.05940 },
					{ 0.04815,   2,    1.85188 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.37700,   2,    3.14686 },
					{ 0.02512,   2,   10.05350 },
					{ 0.23791,   2,    5.15213 },
					{ 0.37270,   2,    2.01131 },
					{ 0.14492,   2,    1.26881 }
				}
			}
		}
	},
	{
		"Cr", 24, 0,
		7,
		{
			{
				1, 0, // 1S
				11,
				{
					{ -0.02722,   0,   35.59110 },
					{ -0.93357,   0,   23.95440 },
					{ -0.04720,   1,   21.65020 },
					{ -0.00604,   1,   10.09060 },
					{ 0.00363,   2,    9.65415 },
					{ -0.00151,   2,    5.90457 },
					{ 0.00100,   2,    4.09494 },
					{ -0.00034,   3,    3.12628 },
					{ 0.00014,   3,    1.76632 },
					{ -0.00009,   3,    1.07837 },
					{ 0.00004,   3,    0.75455 }
				}
			},
			{
				2, 0, // 2S
				11,
				{
					{ -0.00977,   0,   35.59110 },
					{ -0.27707,   0,   23.95440 },
					{ -0.15417,   1,   21.65020 },
					{ 0.93227,   1,   10.09060 },
					{ 0.19827,   2,    9.65415 },
					{ 0.02629,   2,    5.90457 },
					{ -0.00339,   2,    4.09494 },
					{ 0.00078,   3,    3.12628 },
					{ 0.00009,   3,    1.76632 },
					{ 0.00005,   3,    1.07837 },
					{ 0.00006,   3,    0.75455 }
				}
			},
			{
				3, 0, // 3S
				11,
				{
					{ -0.00127,   0,   35.59110 },
					{ -0.10429,   0,   23.95440 },
					{ -0.05407,   1,   21.65020 },
					{ 0.34969,   1,   10.09060 },
					{ 0.23940,   2,    9.65415 },
					{ -0.25716,   2,    5.90457 },
					{ -0.82284,   2,    4.09494 },
					{ -0.12469,   3,    3.12628 },
					{ 0.00348,   3,    1.76632 },
					{ -0.00196,   3,    1.07837 },
					{ 0.00086,   3,    0.75455 }
				}
			},
			{
				4, 0, // 4S
				11,
				{
					{ -0.00018,   0,   35.59110 },
					{ -0.02218,   0,   23.95440 },
					{ -0.01125,   1,   21.65020 },
					{ 0.07371,   1,   10.09060 },
					{ 0.05676,   2,    9.65415 },
					{ -0.05977,   2,    5.90457 },
					{ -0.21516,   2,    4.09494 },
					{ 0.01657,   3,    3.12628 },
					{ 0.40517,   3,    1.76632 },
					{ 0.57019,   3,    1.07837 },
					{ 0.12089,   3,    0.75455 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.11416,   1,   16.08310 },
					{ 0.67530,   1,   10.47530 },
					{ 0.24355,   2,    9.35810 },
					{ 0.01945,   2,    5.64044 },
					{ -0.00112,   2,    3.41092 },
					{ 0.00034,   2,    1.98103 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.04558,   1,   16.08310 },
					{ -0.22347,   1,   10.47530 },
					{ -0.16991,   2,    9.35810 },
					{ 0.36285,   2,    5.64044 },
					{ 0.72269,   2,    3.41092 },
					{ 0.08413,   2,    1.98103 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.03088,   2,    9.76336 },
					{ 0.20871,   2,    5.47299 },
					{ 0.33792,   2,    3.45917 },
					{ 0.35411,   2,    2.18964 },
					{ 0.25996,   2,    1.26443 }
				}
			}
		}
	},
	{
		"Mn", 25, 0,
		7,
		{
			{
				1, 0, // 1S
				11,
				{
					{ -0.93746,   0,   25.35890 },
					{ -0.01123,   0,   40.22850 },
					{ -0.06458,   1,   22.18590 },
					{ 0.00156,   1,   10.18450 },
					{ -0.00288,   2,    9.63560 },
					{ 0.00310,   2,    6.13576 },
					{ -0.00251,   2,    4.43473 },
					{ 0.00092,   3,    3.90249 },
					{ -0.00017,   3,    1.86815 },
					{ 0.00013,   3,    1.15642 },
					{ -0.00005,   3,    0.81912 }
				}
			},
			{
				2, 0, // 2S
				11,
				{
					{ -0.28436,   0,   25.35890 },
					{ -0.00262,   0,   40.22850 },
					{ -0.16507,   1,   22.18590 },
					{ 1.04380,   1,   10.18450 },
					{ 0.08786,   2,    9.63560 },
					{ 0.02502,   2,    6.13576 },
					{ -0.00473,   2,    4.43473 },
					{ 0.00119,   3,    3.90249 },
					{ 0.00012,   3,    1.86815 },
					{ -0.00012,   3,    1.15642 },
					{ 0.00006,   3,    0.81912 }
				}
			},
			{
				3, 0, // 3S
				11,
				{
					{ -0.10595,   0,   25.35890 },
					{ -0.00022,   0,   40.22850 },
					{ -0.06286,   1,   22.18590 },
					{ 0.40954,   1,   10.18450 },
					{ 0.22020,   2,    9.63560 },
					{ -0.35686,   2,    6.13576 },
					{ -0.65559,   2,    4.43473 },
					{ -0.21920,   3,    3.90249 },
					{ -0.01240,   3,    1.86815 },
					{ 0.00736,   3,    1.15642 },
					{ -0.00303,   3,    0.81912 }
				}
			},
			{
				4, 0, // 4S
				11,
				{
					{ -0.02239,   0,   25.35890 },
					{ -0.00035,   0,   40.22850 },
					{ -0.01473,   1,   22.18590 },
					{ 0.09217,   1,   10.18450 },
					{ 0.04683,   2,    9.63560 },
					{ -0.05743,   2,    6.13576 },
					{ -0.22417,   2,    4.43473 },
					{ 0.00628,   3,    3.90249 },
					{ 0.45255,   3,    1.86815 },
					{ 0.52136,   3,    1.15642 },
					{ 0.12840,   3,    0.81912 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.86804,   1,    9.71579 },
					{ 0.12920,   1,   16.72170 },
					{ 0.02225,   2,    6.51366 },
					{ 0.01728,   2,    5.77096 },
					{ -0.00920,   2,    3.73980 },
					{ 0.00343,   2,    2.67402 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.34099,   1,    9.71579 },
					{ -0.03805,   1,   16.72170 },
					{ -0.29351,   2,    6.51366 },
					{ 0.67446,   2,    5.77096 },
					{ 0.52325,   2,    3.73980 },
					{ 0.20373,   2,    2.67402 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.40990,   2,    3.63884 },
					{ 0.02214,   2,   11.55660 },
					{ 0.23736,   2,    6.02189 },
					{ 0.35035,   2,    2.28340 },
					{ 0.13622,   2,    1.46766 }
				}
			}
		}
	},
	{
		"Fe", 26, 0,
		7,
		{
			{
				1, 0, // 1S
				11,
				{
					{ 0.95996,   0,   25.73090 },
					{ 0.01813,   0,   41.38410 },
					{ 0.02737,   1,   21.89900 },
					{ 0.00605,   1,   10.61880 },
					{ -0.00773,   2,    9.55626 },
					{ 0.00621,   2,    7.45884 },
					{ -0.00249,   2,    5.04693 },
					{ 0.00072,   3,    4.02856 },
					{ -0.00019,   3,    2.00760 },
					{ 0.00014,   3,    1.23271 },
					{ -0.00006,   3,    0.85375 }
				}
			},
			{
				2, 0, // 2S
				11,
				{
					{ -0.30606,   0,   25.73090 },
					{ 0.00190,   0,   41.38410 },
					{ -0.16565,   1,   21.89900 },
					{ 1.09354,   1,   10.61880 },
					{ 0.03690,   2,    9.55626 },
					{ 0.04145,   2,    7.45884 },
					{ -0.00540,   2,    5.04693 },
					{ 0.00242,   3,    4.02856 },
					{ -0.00053,   3,    2.00760 },
					{ 0.00039,   3,    1.23271 },
					{ -0.00016,   3,    0.85375 }
				}
			},
			{
				3, 0, // 3S
				11,
				{
					{ -0.11318,   0,   25.73090 },
					{ 0.00061,   0,   41.38410 },
					{ -0.06828,   1,   21.89900 },
					{ 0.44592,   1,   10.61880 },
					{ 0.25239,   2,    9.55626 },
					{ -0.21356,   2,    7.45884 },
					{ -0.81903,   2,    5.04693 },
					{ -0.26346,   3,    4.02856 },
					{ -0.00422,   3,    2.00760 },
					{ 0.00122,   3,    1.23271 },
					{ -0.00037,   3,    0.85375 }
				}
			},
			{
				4, 0, // 4S
				11,
				{
					{ -0.02381,   0,   25.73090 },
					{ -0.00007,   0,   41.38410 },
					{ -0.01561,   1,   21.89900 },
					{ 0.09861,   1,   10.61880 },
					{ 0.05201,   2,    9.55626 },
					{ -0.03147,   2,    7.45884 },
					{ -0.23277,   2,    5.04693 },
					{ -0.02645,   3,    4.02856 },
					{ 0.42197,   3,    2.00760 },
					{ 0.53816,   3,    1.23271 },
					{ 0.15172,   3,    0.85375 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.87094,   1,   10.26560 },
					{ 0.12008,   1,   17.60900 },
					{ 0.04500,   2,    6.91165 },
					{ -0.00315,   2,    6.05627 },
					{ -0.00400,   2,    3.76984 },
					{ 0.00190,   2,    2.62185 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.34583,   1,   10.26560 },
					{ -0.03518,   1,   17.60900 },
					{ -0.29719,   2,    6.91165 },
					{ 0.70788,   2,    6.05627 },
					{ 0.57147,   2,    3.76984 },
					{ 0.12516,   2,    2.62185 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.39452,   2,    3.90292 },
					{ 0.02192,   2,   12.23240 },
					{ 0.24207,   2,    6.37125 },
					{ 0.37020,   2,    2.40608 },
					{ 0.13365,   2,    1.48653 }
				}
			}
		}
	},
	{
		"Co", 27, 0,
		7,
		{
			{
				1, 0, // 1S
				11,
				{
					{ -0.96126,   0,   26.83210 },
					{ -0.01419,   0,   44.27070 },
					{ -0.03023,   1,   22.93140 },
					{ -0.00546,   1,   12.11360 },
					{ 0.00399,   2,   11.31180 },
					{ -0.00444,   2,    6.72828 },
					{ 0.00374,   2,    5.58191 },
					{ -0.00068,   3,    4.36115 },
					{ 0.00015,   3,    2.15692 },
					{ -0.00010,   3,    1.30768 },
					{ 0.00004,   3,    0.86641 }
				}
			},
			{
				2, 0, // 2S
				11,
				{
					{ -0.30396,   0,   26.83210 },
					{ 0.00089,   0,   44.27070 },
					{ -0.17098,   1,   22.93140 },
					{ 0.88074,   1,   12.11360 },
					{ 0.27856,   2,   11.31180 },
					{ 0.05672,   2,    6.72828 },
					{ -0.02838,   2,    5.58191 },
					{ 0.00418,   3,    4.36115 },
					{ -0.00064,   3,    2.15692 },
					{ 0.00041,   3,    1.30768 },
					{ -0.00016,   3,    0.86641 }
				}
			},
			{
				3, 0, // 3S
				11,
				{
					{ -0.11474,   0,   26.83210 },
					{ 0.00105,   0,   44.27070 },
					{ -0.06499,   1,   22.93140 },
					{ 0.34417,   1,   12.11360 },
					{ 0.25970,   2,   11.31180 },
					{ -0.02690,   2,    6.72828 },
					{ -0.84188,   2,    5.58191 },
					{ -0.32635,   3,    4.36115 },
					{ -0.00831,   3,    2.15692 },
					{ 0.00309,   3,    1.30768 },
					{ -0.00106,   3,    0.86641 }
				}
			},
			{
				4, 0, // 4S
				11,
				{
					{ -0.02395,   0,   26.83210 },
					{ 0.00012,   0,   44.27070 },
					{ -0.01425,   1,   22.93140 },
					{ 0.07402,   1,   12.11360 },
					{ 0.05705,   2,   11.31180 },
					{ 0.01936,   2,    6.72828 },
					{ -0.24460,   2,    5.58191 },
					{ -0.04774,   3,    4.36115 },
					{ 0.38442,   3,    2.15692 },
					{ 0.57290,   3,    1.30768 },
					{ 0.16252,   3,    0.86641 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.86458,   1,   11.15900 },
					{ 0.08618,   1,   19.34770 },
					{ 0.08273,   2,    8.62558 },
					{ -0.00014,   2,    6.06638 },
					{ -0.00016,   2,    3.93006 },
					{ 0.00071,   2,    2.78815 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.33949,   1,   11.15900 },
					{ -0.02527,   1,   19.34770 },
					{ -0.10400,   2,    8.62558 },
					{ 0.52001,   2,    6.06638 },
					{ 0.54134,   2,    3.93006 },
					{ 0.12972,   2,    2.78815 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.38834,   2,    4.18799 },
					{ 0.01895,   2,   13.17460 },
					{ 0.23761,   2,    6.85725 },
					{ 0.36586,   2,    2.61143 },
					{ 0.15535,   2,    1.57829 }
				}
			}
		}
	},
	{
		"Ni", 28, 0,
		7,
		{
			{
				1, 0, // 1S
				11,
				{
					{ -0.96109,   0,   27.70810 },
					{ -0.01879,   0,   43.60820 },
					{ -0.02605,   1,   23.22390 },
					{ -0.00362,   1,   11.93940 },
					{ 0.00324,   2,   10.96830 },
					{ -0.00380,   2,    7.20834 },
					{ 0.00293,   2,    5.86511 },
					{ -0.00053,   3,    4.72090 },
					{ 0.00009,   3,    2.27022 },
					{ -0.00006,   3,    1.34716 },
					{ 0.00002,   3,    0.86480 }
				}
			},
			{
				2, 0, // 2S
				11,
				{
					{ -0.31249,   0,   27.70810 },
					{ 0.00353,   0,   43.60820 },
					{ -0.17853,   1,   23.22390 },
					{ 1.03779,   1,   11.93940 },
					{ 0.12533,   2,   10.96830 },
					{ 0.04318,   2,    7.20834 },
					{ -0.01953,   2,    5.86511 },
					{ 0.00383,   3,    4.72090 },
					{ -0.00050,   3,    2.27022 },
					{ 0.00031,   3,    1.34716 },
					{ -0.00012,   3,    0.86480 }
				}
			},
			{
				3, 0, // 3S
				11,
				{
					{ -0.11762,   0,   27.70810 },
					{ 0.00162,   0,   43.60820 },
					{ -0.07249,   1,   23.22390 },
					{ 0.42204,   1,   11.93940 },
					{ 0.23460,   2,   10.96830 },
					{ -0.19094,   2,    7.20834 },
					{ -0.67309,   2,    5.86511 },
					{ -0.37926,   3,    4.72090 },
					{ -0.01470,   3,    2.27022 },
					{ 0.00597,   3,    1.34716 },
					{ -0.00210,   3,    0.86480 }
				}
			},
			{
				4, 0, // 4S
				11,
				{
					{ -0.02413,   0,   27.70810 },
					{ 0.00021,   0,   43.60820 },
					{ -0.01568,   1,   23.22390 },
					{ 0.08946,   1,   11.93940 },
					{ 0.05112,   2,   10.96830 },
					{ -0.02007,   2,    7.20834 },
					{ -0.20008,   2,    5.86511 },
					{ -0.06158,   3,    4.72090 },
					{ 0.37388,   3,    2.27022 },
					{ 0.60023,   3,    1.34716 },
					{ 0.15044,   3,    0.86480 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.83639,   1,   11.57800 },
					{ 0.10552,   1,   19.26870 },
					{ 0.06952,   2,   10.31320 },
					{ 0.02932,   2,    6.67054 },
					{ -0.00937,   2,    4.58463 },
					{ 0.00281,   2,    3.09195 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.32814,   1,   11.57800 },
					{ -0.03327,   1,   19.26870 },
					{ -0.07993,   2,   10.31320 },
					{ 0.34694,   2,    6.67054 },
					{ 0.58559,   2,    4.58463 },
					{ 0.22929,   2,    3.09195 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.35554,   2,    4.39854 },
					{ 0.02557,   2,   12.88330 },
					{ 0.25412,   2,    6.97998 },
					{ 0.36406,   2,    2.79684 },
					{ 0.16885,   2,    1.67922 }
				}
			}
		}
	},
	{
		"Cu", 29, 0,
		7,
		{
			{
				1, 0, // 1S
				11,
				{
					{ -0.95789,   0,   28.48390 },
					{ -0.02856,   0,   42.50560 },
					{ -0.01759,   1,   23.54780 },
					{ -0.00457,   1,   13.26670 },
					{ 0.00440,   2,   11.52060 },
					{ -0.00584,   2,    8.09772 },
					{ 0.00407,   2,    6.70827 },
					{ -0.00063,   3,    5.07948 },
					{ 0.00015,   3,    3.19095 },
					{ -0.00003,   3,    1.53564 },
					{ 0.00001,   3,    0.87051 }
				}
			},
			{
				2, 0, // 2S
				11,
				{
					{ -0.31805,   0,   28.48390 },
					{ 0.00412,   0,   42.50560 },
					{ -0.19769,   1,   23.54780 },
					{ 0.93663,   1,   13.26670 },
					{ 0.29775,   2,   11.52060 },
					{ -0.04582,   2,    8.09772 },
					{ 0.04007,   2,    6.70827 },
					{ -0.00460,   3,    5.07948 },
					{ 0.00127,   3,    3.19095 },
					{ -0.00028,   3,    1.53564 },
					{ 0.00010,   3,    0.87051 }
				}
			},
			{
				3, 0, // 3S
				11,
				{
					{ -0.11906,   0,   28.48390 },
					{ 0.00160,   0,   42.50560 },
					{ -0.08169,   1,   23.54780 },
					{ 0.38342,   1,   13.26670 },
					{ 0.28466,   2,   11.52060 },
					{ 0.04973,   2,    8.09772 },
					{ -0.83111,   2,    6.70827 },
					{ -0.44639,   3,    5.07948 },
					{ -0.03257,   3,    3.19095 },
					{ 0.00086,   3,    1.53564 },
					{ -0.00037,   3,    0.87051 }
				}
			},
			{
				4, 0, // 4S
				11,
				{
					{ -0.02267,   0,   28.48390 },
					{ 0.00143,   0,   42.50560 },
					{ -0.00941,   1,   23.54780 },
					{ 0.05604,   1,   13.26670 },
					{ 0.08953,   2,   11.52060 },
					{ -0.11580,   2,    8.09772 },
					{ -0.03951,   2,    6.70827 },
					{ -0.15751,   3,    5.07948 },
					{ 0.17463,   3,    3.19095 },
					{ 0.66426,   3,    1.53564 },
					{ 0.34469,   3,    0.87051 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.84302,   1,   11.88610 },
					{ 0.11714,   1,   19.58060 },
					{ 0.04499,   2,   10.83980 },
					{ 0.03012,   2,    7.30670 },
					{ -0.00511,   2,    4.57017 },
					{ 0.00182,   2,    2.89365 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.32074,   1,   11.88610 },
					{ -0.04070,   1,   19.58060 },
					{ -0.10529,   2,   10.83980 },
					{ 0.37164,   2,    7.30670 },
					{ 0.67096,   2,    4.57017 },
					{ 0.14959,   2,    2.89365 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.29853,   2,    5.21851 },
					{ 0.02649,   2,   12.96880 },
					{ 0.18625,   2,    7.61139 },
					{ 0.42214,   2,    3.18734 },
					{ 0.26291,   2,    1.66248 }
				}
			}
		}
	},
	{
		"Zn", 30, 0,
		7,
		{
			{
				1, 0, // 1S
				11,
				{
					{ -0.95504,   0,   29.48600 },
					{ -0.03057,   0,   43.28590 },
					{ -0.01886,   1,   24.73990 },
					{ -0.00384,   1,   13.15510 },
					{ 0.00264,   2,   12.00720 },
					{ -0.00287,   2,    7.19899 },
					{ 0.00246,   2,    5.90512 },
					{ -0.00049,   3,    4.83215 },
					{ 0.00009,   3,    2.30295 },
					{ -0.00006,   3,    1.39478 },
					{ 0.00002,   3,    0.94264 }
				}
			},
			{
				2, 0, // 2S
				11,
				{
					{ -0.31973,   0,   29.48600 },
					{ 0.00458,   0,   43.28590 },
					{ -0.18330,   1,   24.73990 },
					{ 1.00556,   1,   13.15510 },
					{ 0.17770,   2,   12.00720 },
					{ 0.03350,   2,    7.19899 },
					{ -0.01651,   2,    5.90512 },
					{ 0.00305,   3,    4.83215 },
					{ -0.00035,   3,    2.30295 },
					{ 0.00024,   3,    1.39478 },
					{ -0.00010,   3,    0.94264 }
				}
			},
			{
				3, 0, // 3S
				11,
				{
					{ -0.11970,   0,   29.48600 },
					{ 0.00113,   0,   43.28590 },
					{ -0.07840,   1,   24.73990 },
					{ 0.41934,   1,   13.15510 },
					{ 0.23499,   2,   12.00720 },
					{ -0.35324,   2,    7.19899 },
					{ -0.58692,   2,    5.90512 },
					{ -0.29127,   3,    4.83215 },
					{ -0.00539,   3,    2.30295 },
					{ 0.00164,   3,    1.39478 },
					{ -0.00050,   3,    0.94264 }
				}
			},
			{
				4, 0, // 4S
				11,
				{
					{ -0.02302,   0,   29.48600 },
					{ -0.00032,   0,   43.28590 },
					{ -0.01758,   1,   24.73990 },
					{ 0.08850,   1,   13.15510 },
					{ 0.04466,   2,   12.00720 },
					{ -0.02731,   2,    7.19899 },
					{ -0.21465,   2,    5.90512 },
					{ -0.00882,   3,    4.83215 },
					{ 0.41428,   3,    2.30295 },
					{ 0.52676,   3,    1.39478 },
					{ 0.17422,   3,    0.94264 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.87225,   1,   12.37110 },
					{ 0.10237,   1,   20.71820 },
					{ 0.06399,   2,    8.82555 },
					{ -0.01065,   2,    6.89964 },
					{ 0.00203,   2,    4.13660 },
					{ -0.00007,   2,    2.20119 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.35894,   1,   12.37110 },
					{ -0.02867,   1,   20.71820 },
					{ -0.14269,   2,    8.82555 },
					{ 0.64545,   2,    6.89964 },
					{ 0.58274,   2,    4.13660 },
					{ 0.02297,   2,    2.20119 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.38099,   2,    5.02809 },
					{ 0.02036,   2,   14.55740 },
					{ 0.22908,   2,    8.00450 },
					{ 0.38095,   2,    3.05216 },
					{ 0.16037,   2,    1.78869 }
				}
			}
		}
	},
	{
		"Ga", 31, 0,
		8,
		{
			{
				1, 0, // 1S
				10,
				{
					{ 0.74710,   0,   28.81010 },
					{ 0.25535,   0,   35.61210 },
					{ 0.00008,   1,   26.15790 },
					{ 0.00169,   1,   13.27810 },
					{ -0.00014,   2,   12.54240 },
					{ -0.00033,   2,    7.45372 },
					{ 0.00015,   2,    4.91188 },
					{ -0.00005,   3,    2.62926 },
					{ 0.00004,   3,    1.58911 },
					{ -0.00002,   3,    1.03307 }
				}
			},
			{
				2, 0, // 2S
				10,
				{
					{ 0.30472,   0,   28.81010 },
					{ 0.02593,   0,   35.61210 },
					{ 0.15106,   1,   26.15790 },
					{ -1.03361,   1,   13.27810 },
					{ -0.12909,   2,   12.54240 },
					{ -0.02392,   2,    7.45372 },
					{ 0.00424,   2,    4.91188 },
					{ -0.00119,   3,    2.62926 },
					{ 0.00084,   3,    1.58911 },
					{ -0.00034,   3,    1.03307 }
				}
			},
			{
				3, 0, // 3S
				10,
				{
					{ -0.11978,   0,   28.81010 },
					{ -0.00728,   0,   35.61210 },
					{ -0.06164,   1,   26.15790 },
					{ 0.42524,   1,   13.27810 },
					{ 0.23678,   2,   12.54240 },
					{ -0.54469,   2,    7.45372 },
					{ -0.67749,   2,    4.91188 },
					{ -0.00788,   3,    2.62926 },
					{ 0.00186,   3,    1.58911 },
					{ -0.00049,   3,    1.03307 }
				}
			},
			{
				4, 0, // 4S
				10,
				{
					{ -0.03229,   0,   28.81010 },
					{ 0.00156,   0,   35.61210 },
					{ -0.01150,   1,   26.15790 },
					{ 0.09475,   1,   13.27810 },
					{ 0.07128,   2,   12.54240 },
					{ -0.16710,   2,    7.45372 },
					{ -0.16344,   2,    4.91188 },
					{ 0.44141,   3,    2.62926 },
					{ 0.57985,   3,    1.58911 },
					{ 0.10455,   3,    1.03307 }
				}
			},
			{
				2, 1, // 2P
				9,
				{
					{ 0.89110,   1,   12.51630 },
					{ 0.11879,   1,   20.94950 },
					{ -0.01678,   2,   12.15290 },
					{ 0.04239,   2,    7.64617 },
					{ -0.01281,   2,    4.93738 },
					{ 0.00488,   3,    3.55119 },
					{ -0.00227,   3,    1.98803 },
					{ 0.00163,   3,    1.18954 },
					{ -0.00076,   3,    0.86212 }
				}
			},
			{
				3, 1, // 3P
				9,
				{
					{ 0.35639,   1,   12.51630 },
					{ 0.04000,   1,   20.94950 },
					{ 0.05337,   2,   12.15290 },
					{ -0.37284,   2,    7.64617 },
					{ -0.71286,   2,    4.93738 },
					{ -0.08520,   3,    3.55119 },
					{ 0.00550,   3,    1.98803 },
					{ -0.00441,   3,    1.18954 },
					{ 0.00191,   3,    0.86212 }
				}
			},
			{
				4, 1, // 4P
				9,
				{
					{ 0.05893,   1,   12.51630 },
					{ 0.00634,   1,   20.94950 },
					{ 0.00969,   2,   12.15290 },
					{ -0.06858,   2,    7.64617 },
					{ -0.12982,   2,    4.93738 },
					{ 0.02852,   3,    3.55119 },
					{ 0.39484,   3,    1.98803 },
					{ 0.46618,   3,    1.18954 },
					{ 0.24077,   3,    0.86212 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.42542,   2,    3.49966 },
					{ 0.14613,   2,    2.08524 },
					{ 0.31824,   2,    5.57215 },
					{ 0.23367,   2,    8.17656 },
					{ 0.02762,   2,   14.38050 }
				}
			}
		}
	},
	{
		"Ge", 32, 0,
		8,
		{
			{
				1, 0, // 1S
				10,
				{
					{ 0.75815,   0,   30.02100 },
					{ 0.23936,   0,   36.80580 },
					{ 0.00627,   1,   27.00910 },
					{ 0.00058,   1,   13.79970 },
					{ 0.00059,   2,   12.88370 },
					{ -0.00062,   2,    7.89859 },
					{ 0.00028,   2,    5.27181 },
					{ -0.00010,   3,    2.87538 },
					{ 0.00009,   3,    1.79597 },
					{ -0.00004,   3,    1.29724 }
				}
			},
			{
				2, 0, // 2S
				10,
				{
					{ 0.30153,   0,   30.02100 },
					{ 0.02606,   0,   36.80580 },
					{ 0.16087,   1,   27.00910 },
					{ -1.03666,   1,   13.79970 },
					{ -0.13209,   2,   12.88370 },
					{ -0.02214,   2,    7.89859 },
					{ 0.00323,   2,    5.27181 },
					{ -0.00097,   3,    2.87538 },
					{ 0.00081,   3,    1.79597 },
					{ -0.00036,   3,    1.29724 }
				}
			},
			{
				3, 0, // 3S
				10,
				{
					{ 0.11337,   0,   30.02100 },
					{ 0.01215,   0,   36.80580 },
					{ 0.07053,   1,   27.00910 },
					{ -0.43821,   1,   13.79970 },
					{ -0.24038,   2,   12.88370 },
					{ 0.49626,   2,    7.89859 },
					{ 0.72788,   2,    5.27181 },
					{ 0.01467,   3,    2.87538 },
					{ -0.00665,   3,    1.79597 },
					{ 0.00270,   3,    1.29724 }
				}
			},
			{
				4, 0, // 4S
				10,
				{
					{ -0.03022,   0,   30.02100 },
					{ -0.00226,   0,   36.80580 },
					{ -0.01741,   1,   27.00910 },
					{ 0.11165,   1,   13.79970 },
					{ 0.07481,   2,   12.88370 },
					{ -0.16000,   2,    7.89859 },
					{ -0.21325,   2,    5.27181 },
					{ 0.46131,   3,    2.87538 },
					{ 0.53115,   3,    1.79597 },
					{ 0.13401,   3,    1.29724 }
				}
			},
			{
				2, 1, // 2P
				9,
				{
					{ 0.86134,   1,   13.35370 },
					{ 0.10385,   1,   22.09880 },
					{ 0.02927,   2,   12.89370 },
					{ 0.04517,   2,    8.12507 },
					{ -0.01327,   2,    5.26670 },
					{ 0.00507,   3,    3.74374 },
					{ -0.00240,   3,    2.16483 },
					{ 0.00147,   3,    1.27537 },
					{ -0.00063,   3,    0.88029 }
				}
			},
			{
				3, 1, // 3P
				9,
				{
					{ 0.34590,   1,   13.35370 },
					{ 0.03597,   1,   22.09880 },
					{ 0.07589,   2,   12.89370 },
					{ -0.33604,   2,    8.12507 },
					{ -0.74958,   2,    5.26670 },
					{ -0.08490,   3,    3.74374 },
					{ 0.00762,   3,    2.16483 },
					{ -0.00478,   3,    1.27537 },
					{ 0.00190,   3,    0.88029 }
				}
			},
			{
				4, 1, // 4P
				9,
				{
					{ 0.06904,   1,   13.35370 },
					{ 0.00684,   1,   22.09880 },
					{ 0.01618,   2,   12.89370 },
					{ -0.07467,   2,    8.12507 },
					{ -0.16721,   2,    5.26670 },
					{ 0.03395,   3,    3.74374 },
					{ 0.48432,   3,    2.16483 },
					{ 0.53214,   3,    1.27537 },
					{ 0.07195,   3,    0.88029 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.44017,   2,    3.82993 },
					{ 0.12938,   2,    2.34719 },
					{ 0.33840,   2,    6.11651 },
					{ 0.20172,   2,    8.86174 },
					{ 0.02653,   2,   14.92730 }
				}
			}
		}
	},
	{
		"As", 33, 0,
		8,
		{
			{
				1, 0, // 1S
				10,
				{
					{ 0.80944,   0,   31.34600 },
					{ 0.18590,   0,   38.60770 },
					{ 0.00949,   1,   26.26730 },
					{ -0.00069,   1,   14.94890 },
					{ 0.00121,   2,   13.45360 },
					{ -0.00083,   2,    8.13994 },
					{ 0.00039,   2,    5.53193 },
					{ -0.00014,   3,    3.14867 },
					{ 0.00012,   3,    2.01557 },
					{ -0.00005,   3,    1.42236 }
				}
			},
			{
				2, 0, // 2S
				10,
				{
					{ 0.35297,   0,   31.34600 },
					{ -0.01548,   0,   38.60770 },
					{ 0.18189,   1,   26.26730 },
					{ -0.98989,   1,   14.94890 },
					{ -0.22138,   2,   13.45360 },
					{ -0.01725,   2,    8.13994 },
					{ 0.00209,   2,    5.53193 },
					{ -0.00065,   3,    3.14867 },
					{ 0.00050,   3,    2.01557 },
					{ -0.00020,   3,    1.42236 }
				}
			},
			{
				3, 0, // 3S
				10,
				{
					{ -0.13676,   0,   31.34600 },
					{ 0.00550,   0,   38.60770 },
					{ -0.08033,   1,   26.26730 },
					{ 0.42323,   1,   14.94890 },
					{ 0.27579,   2,   13.45360 },
					{ -0.49356,   2,    8.13994 },
					{ -0.72763,   2,    5.53193 },
					{ -0.01353,   3,    3.14867 },
					{ 0.00483,   3,    2.01557 },
					{ -0.00178,   3,    1.42236 }
				}
			},
			{
				4, 0, // 4S
				10,
				{
					{ -0.03678,   0,   31.34600 },
					{ 0.00086,   0,   38.60770 },
					{ -0.02303,   1,   26.26730 },
					{ 0.11843,   1,   14.94890 },
					{ 0.08597,   2,   13.45360 },
					{ -0.16171,   2,    8.13994 },
					{ -0.24575,   2,    5.53193 },
					{ 0.45102,   3,    3.14867 },
					{ 0.54525,   3,    2.01557 },
					{ 0.13765,   3,    1.42236 }
				}
			},
			{
				2, 1, // 2P
				9,
				{
					{ 0.83284,   1,   14.05460 },
					{ 0.10192,   1,   22.78430 },
					{ 0.06278,   2,   13.52830 },
					{ 0.04654,   2,    8.37724 },
					{ -0.01655,   2,    5.57821 },
					{ 0.00612,   3,    4.34244 },
					{ -0.00205,   3,    2.42567 },
					{ 0.00109,   3,    1.45140 },
					{ -0.00039,   3,    0.91898 }
				}
			},
			{
				3, 1, // 3P
				9,
				{
					{ 0.33887,   1,   14.05460 },
					{ 0.03589,   1,   22.78430 },
					{ 0.08838,   2,   13.52830 },
					{ -0.33676,   2,    8.37724 },
					{ -0.73040,   2,    5.57821 },
					{ -0.09452,   3,    4.34244 },
					{ -0.00136,   3,    2.42567 },
					{ -0.00010,   3,    1.45140 },
					{ 0.00004,   3,    0.91898 }
				}
			},
			{
				4, 1, // 4P
				9,
				{
					{ 0.07735,   1,   14.05460 },
					{ 0.00736,   1,   22.78430 },
					{ 0.01932,   2,   13.52830 },
					{ -0.08004,   2,    8.37724 },
					{ -0.19430,   2,    5.57821 },
					{ 0.02331,   3,    4.34244 },
					{ 0.50866,   3,    2.42567 },
					{ 0.53655,   3,    1.45140 },
					{ 0.05044,   3,    0.91898 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.44874,   2,    4.15670 },
					{ 0.11466,   2,    2.56420 },
					{ 0.30600,   2,    6.36170 },
					{ 0.22855,   2,    9.09175 },
					{ 0.02710,   2,   15.54610 }
				}
			}
		}
	},
	{
		"Se", 34, 0,
		8,
		{
			{
				1, 0, // 1S
				10,
				{
					{ 0.84307,   0,   32.00980 },
					{ 0.16359,   0,   40.79540 },
					{ -0.00790,   1,   27.17130 },
					{ 0.00666,   1,   15.13520 },
					{ -0.00349,   2,   13.62030 },
					{ 0.00127,   2,    8.41955 },
					{ -0.00060,   2,    5.75207 },
					{ 0.00018,   3,    3.33167 },
					{ -0.00013,   3,    2.19374 },
					{ 0.00005,   3,    1.45697 }
				}
			},
			{
				2, 0, // 2S
				10,
				{
					{ 0.34872,   0,   32.00980 },
					{ -0.00786,   0,   40.79540 },
					{ 0.17658,   1,   27.17130 },
					{ -1.03246,   1,   15.13520 },
					{ -0.17253,   2,   13.62030 },
					{ -0.01678,   2,    8.41955 },
					{ 0.00228,   2,    5.75207 },
					{ -0.00069,   3,    3.33167 },
					{ 0.00050,   3,    2.19374 },
					{ -0.00016,   3,    1.45697 }
				}
			},
			{
				3, 0, // 3S
				10,
				{
					{ -0.13350,   0,   32.00980 },
					{ 0.00048,   0,   40.79540 },
					{ -0.08308,   1,   27.17130 },
					{ 0.45513,   1,   15.13520 },
					{ 0.26080,   2,   13.62030 },
					{ -0.51814,   2,    8.41955 },
					{ -0.71665,   2,    5.75207 },
					{ -0.00820,   3,    3.33167 },
					{ 0.00114,   3,    2.19374 },
					{ -0.00034,   3,    1.45697 }
				}
			},
			{
				4, 0, // 4S
				10,
				{
					{ -0.03934,   0,   32.00980 },
					{ 0.00037,   0,   40.79540 },
					{ -0.02419,   1,   27.17130 },
					{ 0.13351,   1,   15.13520 },
					{ 0.09050,   2,   13.62030 },
					{ -0.18446,   2,    8.41955 },
					{ -0.25805,   2,    5.75207 },
					{ 0.47154,   3,    3.33167 },
					{ 0.54954,   3,    2.19374 },
					{ 0.11764,   3,    1.45697 }
				}
			},
			{
				2, 1, // 2P
				9,
				{
					{ 0.80693,   1,   14.75700 },
					{ 0.10012,   1,   23.48300 },
					{ 0.09287,   2,   14.16660 },
					{ 0.04670,   2,    8.69278 },
					{ -0.01717,   2,    5.83188 },
					{ 0.00658,   3,    4.54856 },
					{ -0.00276,   3,    2.59061 },
					{ 0.00213,   3,    1.63458 },
					{ -0.00101,   3,    1.25528 }
				}
			},
			{
				3, 1, // 3P
				9,
				{
					{ 0.33247,   1,   14.75700 },
					{ 0.03580,   1,   23.48300 },
					{ 0.10108,   2,   14.16660 },
					{ -0.33254,   2,    8.69278 },
					{ -0.74258,   2,    5.83188 },
					{ -0.08364,   3,    4.54856 },
					{ -0.00162,   3,    2.59061 },
					{ -0.00034,   3,    1.63458 },
					{ -0.00007,   3,    1.25528 }
				}
			},
			{
				4, 1, // 4P
				9,
				{
					{ 0.08386,   1,   14.75700 },
					{ 0.00743,   1,   23.48300 },
					{ 0.02171,   2,   14.16660 },
					{ -0.07972,   2,    8.69278 },
					{ -0.22666,   2,    5.83188 },
					{ 0.04165,   3,    4.54856 },
					{ 0.54173,   3,    2.59061 },
					{ 0.36695,   3,    1.63458 },
					{ 0.18340,   3,    1.25528 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.15501,   2,    4.59417 },
					{ 0.14364,   2,    2.84235 },
					{ 0.48035,   2,    5.27029 },
					{ 0.32562,   2,    9.42659 },
					{ 0.01261,   2,   19.54570 }
				}
			}
		}
	},
	{
		"Br", 35, 0,
		8,
		{
			{
				1, 0, // 1S
				10,
				{
					{ 0.81095,   0,   32.58640 },
					{ 0.20011,   0,   41.35570 },
					{ -0.01554,   1,   26.53420 },
					{ 0.01020,   1,   15.87920 },
					{ -0.00540,   2,   13.79510 },
					{ 0.00246,   2,    8.78775 },
					{ -0.00115,   2,    6.01958 },
					{ 0.00037,   3,    3.65032 },
					{ -0.00025,   3,    2.39212 },
					{ 0.00009,   3,    1.63837 }
				}
			},
			{
				2, 0, // 2S
				10,
				{
					{ 0.37448,   0,   32.58640 },
					{ -0.02121,   0,   41.35570 },
					{ 0.19595,   1,   26.53420 },
					{ -1.05505,   1,   15.87920 },
					{ -0.18909,   2,   13.79510 },
					{ -0.00887,   2,    8.78775 },
					{ -0.00028,   2,    6.01958 },
					{ -0.00002,   3,    3.65032 },
					{ 0.00005,   3,    2.39212 },
					{ -0.00001,   3,    1.63837 }
				}
			},
			{
				3, 0, // 3S
				10,
				{
					{ -0.14513,   0,   32.58640 },
					{ 0.00582,   0,   41.35570 },
					{ -0.09653,   1,   26.53420 },
					{ 0.47805,   1,   15.87920 },
					{ 0.27859,   2,   13.79510 },
					{ -0.52656,   2,    8.78775 },
					{ -0.72544,   2,    6.01958 },
					{ -0.00594,   3,    3.65032 },
					{ -0.00099,   3,    2.39212 },
					{ 0.00027,   3,    1.63837 }
				}
			},
			{
				4, 0, // 4S
				10,
				{
					{ -0.04178,   0,   32.58640 },
					{ -0.00013,   0,   41.35570 },
					{ -0.03333,   1,   26.53420 },
					{ 0.15368,   1,   15.87920 },
					{ 0.09132,   2,   13.79510 },
					{ -0.17533,   2,    8.78775 },
					{ -0.30447,   2,    6.01958 },
					{ 0.46075,   3,    3.65032 },
					{ 0.56685,   3,    2.39212 },
					{ 0.12548,   3,    1.63837 }
				}
			},
			{
				2, 1, // 2P
				9,
				{
					{ 0.74854,   1,   15.81260 },
					{ 0.09604,   1,   24.12670 },
					{ 0.16709,   2,   14.74150 },
					{ 0.04123,   2,    8.64224 },
					{ -0.01974,   2,    5.90145 },
					{ 0.00780,   3,    5.18209 },
					{ -0.00154,   3,    2.82688 },
					{ 0.00090,   3,    1.70190 },
					{ -0.00039,   3,    1.21513 }
				}
			},
			{
				3, 1, // 3P
				9,
				{
					{ 0.31389,   1,   15.81260 },
					{ 0.03407,   1,   24.12670 },
					{ 0.12804,   2,   14.74150 },
					{ -0.41337,   2,    8.64224 },
					{ -0.66876,   2,    5.90145 },
					{ -0.06708,   3,    5.18209 },
					{ -0.01083,   3,    2.82688 },
					{ 0.00362,   3,    1.70190 },
					{ -0.00184,   3,    1.21513 }
				}
			},
			{
				4, 1, // 4P
				9,
				{
					{ 0.08539,   1,   15.81260 },
					{ 0.00721,   1,   24.12670 },
					{ 0.02963,   2,   14.74150 },
					{ -0.10120,   2,    8.64224 },
					{ -0.24356,   2,    5.90145 },
					{ 0.05048,   3,    5.18209 },
					{ 0.56500,   3,    2.82688 },
					{ 0.45389,   3,    1.70190 },
					{ 0.08536,   3,    1.21513 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.47817,   2,    4.81505 },
					{ 0.09572,   2,    3.01772 },
					{ 0.31609,   2,    7.36217 },
					{ 0.19151,   2,   10.19710 },
					{ 0.02657,   2,   16.62740 }
				}
			}
		}
	},
	{
		"Kr", 36, 0,
		8,
		{
			{
				1, 0, // 1S
				10,
				{
					{ 0.71521,   0,   32.83510 },
					{ 0.29911,   0,   40.94470 },
					{ -0.01854,   1,   27.45800 },
					{ 0.00897,   1,   16.06660 },
					{ -0.00464,   2,   14.29620 },
					{ 0.00190,   2,    9.10937 },
					{ -0.00088,   2,    6.37181 },
					{ 0.00026,   3,    3.84546 },
					{ -0.00018,   3,    2.57902 },
					{ 0.00006,   3,    1.77192 }
				}
			},
			{
				2, 0, // 2S
				10,
				{
					{ 0.38139,   0,   32.83510 },
					{ -0.01823,   0,   40.94470 },
					{ 0.17175,   1,   27.45800 },
					{ -1.07160,   1,   16.06660 },
					{ -0.14913,   2,   14.29620 },
					{ -0.01920,   2,    9.10937 },
					{ 0.00401,   2,    6.37181 },
					{ -0.00122,   3,    3.84546 },
					{ 0.00092,   3,    2.57902 },
					{ -0.00031,   3,    1.77192 }
				}
			},
			{
				3, 0, // 3S
				10,
				{
					{ -0.14543,   0,   32.83510 },
					{ 0.00181,   0,   40.94470 },
					{ -0.09037,   1,   27.45800 },
					{ 0.49528,   1,   16.06660 },
					{ 0.25451,   2,   14.29620 },
					{ -0.48504,   2,    9.10937 },
					{ -0.75593,   2,    6.37181 },
					{ -0.01203,   3,    3.84546 },
					{ 0.00218,   3,    2.57902 },
					{ -0.00085,   3,    1.77192 }
				}
			},
			{
				4, 0, // 4S
				10,
				{
					{ -0.04349,   0,   32.83510 },
					{ -0.00148,   0,   40.94470 },
					{ -0.03219,   1,   27.45800 },
					{ 0.16451,   1,   16.06660 },
					{ 0.08852,   2,   14.29620 },
					{ -0.16671,   2,    9.10937 },
					{ -0.33291,   2,    6.37181 },
					{ 0.46913,   3,    3.84546 },
					{ 0.55106,   3,    2.57902 },
					{ 0.13572,   3,    1.77192 }
				}
			},
			{
				2, 1, // 2P
				9,
				{
					{ 0.72322,   1,   17.03660 },
					{ 0.06774,   1,   26.04380 },
					{ 0.22056,   2,   15.51000 },
					{ 0.04478,   2,    9.49403 },
					{ -0.01672,   2,    6.57275 },
					{ 0.00609,   3,    5.38507 },
					{ -0.00195,   3,    3.15603 },
					{ 0.00111,   3,    2.02966 },
					{ -0.00040,   3,    1.42733 }
				}
			},
			{
				3, 1, // 3P
				9,
				{
					{ 0.30185,   1,   17.03660 },
					{ 0.02508,   1,   26.04380 },
					{ 0.15903,   2,   15.51000 },
					{ -0.28475,   2,    9.49403 },
					{ -0.76440,   2,    6.57275 },
					{ -0.10670,   3,    5.38507 },
					{ -0.00562,   3,    3.15603 },
					{ 0.00137,   3,    2.02966 },
					{ -0.00053,   3,    1.42733 }
				}
			},
			{
				4, 1, // 4P
				9,
				{
					{ 0.08488,   1,   17.03660 },
					{ 0.00571,   1,   26.04380 },
					{ 0.04169,   2,   15.51000 },
					{ -0.07425,   2,    9.49403 },
					{ -0.26866,   2,    6.57275 },
					{ 0.01341,   3,    5.38507 },
					{ 0.51241,   3,    3.15603 },
					{ 0.42557,   3,    2.02966 },
					{ 0.18141,   3,    1.42733 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.50854,   2,    5.30650 },
					{ 0.11070,   2,    3.36240 },
					{ 0.24778,   2,    7.94963 },
					{ 0.20584,   2,   10.35430 },
					{ 0.02863,   2,   17.11420 }
				}
			}
		}
	},
	{
		"Li+", 3, 1,
		1,
		{
			{
				1, 0, // 1S
				4,
				{
					{ 0.89459,   0,    2.46376 },
					{ 0.11839,   0,    4.70359 },
					{ -0.00232,   0,    6.46694 },
					{ 0.00445,   0,    1.35790 }
				}
			}
		}
	},
	{
		"Na+", 11, 1,
		3,
		{
			{
				1, 0, // 1S
				5,
				{
					{ 1.04471,   0,    9.60854 },
					{ 0.06184,   0,   16.19560 },
					{ 0.00584,   1,    4.38717 },
					{ -0.11882,   1,   10.20650 },
					{ -0.00106,   1,    2.81650 }
				}
			},
			{
				2, 0, // 2S
				5,
				{
					{ -0.23717,   0,    9.60854 },
					{ -0.02342,   0,   16.19560 },
					{ 0.48735,   1,    4.38717 },
					{ -0.07038,   1,   10.20650 },
					{ 0.61604,   1,    2.81650 }
				}
			},
			{
				2, 1, // 2P
				4,
				{
					{ 0.54279,   1,    2.89481 },
					{ 0.30481,   1,    5.26509 },
					{ 0.21629,   1,    1.95889 },
					{ 0.01413,   1,   10.69400 }
				}
			}
		}
	},
	{
		"K+", 19, 1,
		5,
		{
			{
				1, 0, // 1S
				7,
				{
					{ 0.93931,   0,   18.98260 },
					{ 0.01789,   0,   31.39050 },
					{ 0.00420,   1,    7.30079 },
					{ 0.05305,   1,   16.70010 },
					{ 0.00135,   2,    3.88331 },
					{ -0.00037,   2,    2.40881 },
					{ -0.00255,   2,    5.70995 }
				}
			},
			{
				2, 0, // 2S
				7,
				{
					{ -0.27701,   0,   18.98260 },
					{ -0.00171,   0,   31.39050 },
					{ 1.04965,   1,    7.30079 },
					{ -0.13733,   1,   16.70010 },
					{ -0.00664,   2,    3.88331 },
					{ 0.00268,   2,    2.40881 },
					{ 0.08655,   2,    5.70995 }
				}
			},
			{
				3, 0, // 3S
				7,
				{
					{ 0.08889,   0,   18.98260 },
					{ 0.00173,   0,   31.39050 },
					{ -0.39368,   1,    7.30079 },
					{ 0.05415,   1,   16.70010 },
					{ 0.64114,   2,    3.88331 },
					{ 0.61485,   2,    2.40881 },
					{ -0.21575,   2,    5.70995 }
				}
			},
			{
				2, 1, // 2P
				5,
				{
					{ 0.67152,   1,    8.61735 },
					{ 0.04224,   1,   15.19580 },
					{ 0.01009,   2,    3.24817 },
					{ -0.00196,   2,    1.94563 },
					{ 0.34429,   2,    6.88378 }
				}
			},
			{
				3, 1, // 3P
				5,
				{
					{ -0.20999,   1,    8.61735 },
					{ -0.01178,   1,   15.19580 },
					{ 0.58221,   2,    3.24817 },
					{ 0.52976,   2,    1.94563 },
					{ -0.12638,   2,    6.88378 }
				}
			}
		}
	},
	{
		"Cu+", 29, 1,
		6,
		{
			{
				1, 0, // 1S
				8,
				{
					{ -0.95851,   0,   28.45300 },
					{ -0.02906,   0,   42.53270 },
					{ -0.01709,   1,   23.10340 },
					{ -0.00297,   1,   13.01550 },
					{ 0.00168,   2,   11.52770 },
					{ -0.00089,   2,    6.72990 },
					{ 0.00078,   2,    4.61704 },
					{ -0.00031,   2,    3.53495 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ -0.32587,   0,   28.45300 },
					{ 0.00819,   0,   42.53270 },
					{ -0.19715,   1,   23.10340 },
					{ 0.98177,   1,   13.01550 },
					{ 0.23062,   2,   11.52770 },
					{ 0.01499,   2,    6.72990 },
					{ -0.00405,   2,    4.61704 },
					{ 0.00191,   2,    3.53495 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.12160,   0,   28.45300 },
					{ -0.00297,   0,   42.53270 },
					{ 0.08245,   1,   23.10340 },
					{ -0.40335,   1,   13.01550 },
					{ -0.25689,   2,   11.52770 },
					{ 0.55723,   2,    6.72990 },
					{ 0.54604,   2,    4.61704 },
					{ 0.11723,   2,    3.53495 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.76853,   1,   12.36430 },
					{ 0.11626,   1,   19.52640 },
					{ 0.12971,   2,   11.52780 },
					{ 0.02772,   2,    7.15679 },
					{ -0.00468,   2,    4.50992 },
					{ 0.00165,   2,    2.86886 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.28521,   1,   12.36430 },
					{ -0.04270,   1,   19.52640 },
					{ -0.12476,   2,   11.52780 },
					{ 0.37278,   2,    7.15679 },
					{ 0.66459,   2,    4.50992 },
					{ 0.13742,   2,    2.86886 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.33575,   2,    4.70557 },
					{ 0.02401,   2,   13.47820 },
					{ 0.23802,   2,    7.38627 },
					{ 0.36175,   2,    2.93665 },
					{ 0.22921,   2,    1.69283 }
				}
			}
		}
	},
	{
		"Be2+", 4, 2,
		1,
		{
			{
				1, 0, // 1S
				3,
				{
					{ 0.89855,   0,    3.43071 },
					{ 0.09068,   0,    5.63150 },
					{ 0.02158,   0,    7.35143 }
				}
			}
		}
	},
	{
		"Mg2+", 12, 2,
		3,
		{
			{
				1, 0, // 1S
				5,
				{
					{ 0.89822,   0,   11.44730 },
					{ 0.07634,   0,   17.00270 },
					{ 0.00675,   1,    4.97455 },
					{ 0.03263,   1,   10.70430 },
					{ -0.00143,   1,    3.30779 }
				}
			},
			{
				2, 0, // 2S
				5,
				{
					{ -0.22646,   0,   11.44730 },
					{ -0.01956,   0,   17.00270 },
					{ 0.43898,   1,    4.97455 },
					{ -0.10768,   1,   10.70430 },
					{ 0.66951,   1,    3.30779 }
				}
			},
			{
				2, 1, // 2P
				4,
				{
					{ 0.55496,   1,    3.40554 },
					{ 0.28364,   1,    6.05091 },
					{ 0.21606,   1,    2.44651 },
					{ 0.01021,   1,   12.58860 }
				}
			}
		}
	},
	{
		"Al3+", 13, 3,
		3,
		{
			{
				1, 0, // 1S
				5,
				{
					{ 0.90480,   0,   12.81930 },
					{ 0.04457,   0,   19.46880 },
					{ 0.00547,   1,    5.50851 },
					{ 0.06224,   1,   11.65060 },
					{ -0.00102,   1,    3.79920 }
				}
			},
			{
				2, 0, // 2S
				5,
				{
					{ -0.23417,   0,   12.81930 },
					{ -0.01226,   0,   19.46880 },
					{ 0.39868,   1,    5.50851 },
					{ -0.12065,   1,   11.65060 },
					{ 0.71041,   1,    3.79920 }
				}
			},
			{
				2, 1, // 2P
				4,
				{
					{ 0.56284,   1,    3.92330 },
					{ 0.26314,   1,    6.83553 },
					{ 0.22166,   1,    2.93879 },
					{ 0.00803,   1,   14.24410 }
				}
			}
		}
	},
	{
		"Ca2+", 20, 2,
		5,
		{
			{
				1, 0, // 1S
				7,
				{
					{ 0.94553,   0,   19.85610 },
					{ 0.02016,   0,   32.43970 },
					{ 0.00475,   1,    7.79921 },
					{ 0.04280,   1,   17.36070 },
					{ 0.00150,   2,    4.18323 },
					{ -0.00048,   2,    2.73133 },
					{ -0.00297,   2,    6.35894 }
				}
			},
			{
				2, 0, // 2S
				7,
				{
					{ -0.28435,   0,   19.85610 },
					{ -0.00105,   0,   32.43970 },
					{ 1.04968,   1,    7.79921 },
					{ -0.14030,   1,   17.36070 },
					{ -0.00078,   2,    4.18323 },
					{ 0.00177,   2,    2.73133 },
					{ 0.08860,   2,    6.35894 }
				}
			},
			{
				3, 0, // 3S
				7,
				{
					{ 0.09576,   0,   19.85610 },
					{ 0.00148,   0,   32.43970 },
					{ -0.41086,   1,    7.79921 },
					{ 0.05767,   1,   17.36070 },
					{ 0.60666,   2,    4.18323 },
					{ 0.63178,   2,    2.73133 },
					{ -0.20777,   2,    6.35894 }
				}
			},
			{
				2, 1, // 2P
				5,
				{
					{ 0.67714,   1,    9.11433 },
					{ 0.04420,   1,   15.86340 },
					{ 0.01098,   2,    3.56861 },
					{ -0.00245,   2,    2.27163 },
					{ 0.33327,   2,    7.37993 }
				}
			},
			{
				3, 1, // 3P
				5,
				{
					{ -0.22768,   1,    9.11433 },
					{ -0.01304,   1,   15.86340 },
					{ 0.58722,   2,    3.56861 },
					{ 0.52192,   2,    2.27163 },
					{ -0.13455,   2,    7.37993 }
				}
			}
		}
	},
	{
		"Sc3+", 21, 3,
		5,
		{
			{
				1, 0, // 1S
				7,
				{
					{ 0.95026,   0,   20.71740 },
					{ 0.02310,   0,   33.63060 },
					{ 0.00707,   1,    8.32218 },
					{ 0.03256,   1,   18.28940 },
					{ 0.00250,   2,    4.60777 },
					{ -0.00082,   2,    3.09084 },
					{ -0.00492,   2,    7.00851 }
				}
			},
			{
				2, 0, // 2S
				7,
				{
					{ -0.28616,   0,   20.71740 },
					{ -0.00303,   0,   33.63060 },
					{ 1.03977,   1,    8.32218 },
					{ -0.14371,   1,   18.28940 },
					{ -0.00086,   2,    4.60777 },
					{ 0.00245,   2,    3.09084 },
					{ 0.10322,   2,    7.00851 }
				}
			},
			{
				3, 0, // 3S
				7,
				{
					{ 0.10127,   0,   20.71740 },
					{ 0.00170,   0,   33.63060 },
					{ -0.41703,   1,    8.32218 },
					{ 0.05920,   1,   18.28940 },
					{ 0.54911,   2,    4.60777 },
					{ 0.69451,   2,    3.09084 },
					{ -0.22866,   2,    7.00851 }
				}
			},
			{
				2, 1, // 2P
				5,
				{
					{ 0.68169,   1,    9.61209 },
					{ 0.04630,   1,   16.50940 },
					{ 0.01208,   2,    3.91166 },
					{ -0.00301,   2,    2.61991 },
					{ 0.32338,   2,    7.88243 }
				}
			},
			{
				3, 1, // 3P
				5,
				{
					{ -0.24276,   1,    9.61209 },
					{ -0.01437,   1,   16.50940 },
					{ 0.57218,   2,    3.91166 },
					{ 0.53707,   2,    2.61991 },
					{ -0.14245,   2,    7.88243 }
				}
			}
		}
	},
	{
		"Ti4+", 22, 4,
		5,
		{
			{
				1, 0, // 1S
				7,
				{
					{ 0.95310,   0,   21.64990 },
					{ 0.02433,   0,   34.75470 },
					{ 0.00738,   1,    8.82635 },
					{ 0.02761,   1,   18.96180 },
					{ 0.00238,   2,    4.90094 },
					{ -0.00089,   2,    3.43212 },
					{ -0.00502,   2,    7.70277 }
				}
			},
			{
				2, 0, // 2S
				7,
				{
					{ -0.29255,   0,   21.64990 },
					{ -0.00181,   0,   34.75470 },
					{ 1.03769,   1,    8.82635 },
					{ -0.14660,   1,   18.96180 },
					{ 0.00594,   2,    4.90094 },
					{ 0.00071,   2,    3.43212 },
					{ 0.10619,   2,    7.70277 }
				}
			},
			{
				3, 0, // 3S
				7,
				{
					{ 0.10731,   0,   21.64990 },
					{ 0.00118,   0,   34.75470 },
					{ -0.42797,   1,    8.82635 },
					{ 0.06193,   1,   18.96180 },
					{ 0.50138,   2,    4.90094 },
					{ 0.72611,   2,    3.43212 },
					{ -0.22191,   2,    7.70277 }
				}
			},
			{
				2, 1, // 2P
				5,
				{
					{ 0.67900,   1,   10.12070 },
					{ 0.05120,   1,   16.90970 },
					{ 0.01343,   2,    4.23842 },
					{ -0.00368,   2,    2.95011 },
					{ 0.31841,   2,    8.41308 }
				}
			},
			{
				3, 1, // 3P
				5,
				{
					{ -0.25319,   1,   10.12070 },
					{ -0.01679,   1,   16.90970 },
					{ 0.56819,   2,    4.23842 },
					{ 0.54337,   2,    2.95011 },
					{ -0.15126,   2,    8.41308 }
				}
			}
		}
	},
	{
		"V5+", 23, 5,
		5,
		{
			{
				1, 0, // 1S
				7,
				{
					{ 0.95599,   0,   22.56550 },
					{ 0.02586,   0,   35.92840 },
					{ 0.00805,   1,    9.33647 },
					{ 0.02204,   1,   19.64550 },
					{ 0.00261,   2,    5.20116 },
					{ -0.00105,   2,    3.76303 },
					{ -0.00552,   2,    8.27824 }
				}
			},
			{
				2, 0, // 2S
				7,
				{
					{ -0.29798,   0,   22.56550 },
					{ -0.00107,   0,   35.92840 },
					{ 1.03711,   1,    9.33647 },
					{ -0.15012,   1,   19.64550 },
					{ 0.00808,   2,    5.20116 },
					{ 0.00001,   2,    3.76303 },
					{ 0.11161,   2,    8.27824 }
				}
			},
			{
				3, 0, // 3S
				7,
				{
					{ 0.11263,   0,   22.56550 },
					{ 0.00086,   0,   35.92840 },
					{ -0.43894,   1,    9.33647 },
					{ 0.06503,   1,   19.64550 },
					{ 0.47716,   2,    5.20116 },
					{ 0.74939,   2,    3.76303 },
					{ -0.22807,   2,    8.27824 }
				}
			},
			{
				2, 1, // 2P
				5,
				{
					{ 0.68798,   1,   10.63190 },
					{ 0.04962,   1,   17.77070 },
					{ 0.01264,   2,    4.57247 },
					{ -0.00350,   2,    3.28636 },
					{ 0.30928,   2,    8.88729 }
				}
			},
			{
				3, 1, // 3P
				5,
				{
					{ -0.26644,   1,   10.63190 },
					{ -0.01692,   1,   17.77070 },
					{ 0.55900,   2,    4.57247 },
					{ 0.55688,   2,    3.28636 },
					{ -0.15820,   2,    8.88729 }
				}
			}
		}
	},
	{
		"Ti3+", 22, 3,
		6,
		{
			{
				1, 0, // 1S
				8,
				{
					{ -0.94366,   0,   22.02320 },
					{ -0.01663,   0,   34.95600 },
					{ -0.05095,   1,   18.91320 },
					{ -0.00041,   1,    9.10692 },
					{ -0.00063,   2,    8.32663 },
					{ 0.00075,   2,    4.88396 },
					{ -0.00075,   2,    3.41639 },
					{ 0.00038,   2,    2.83054 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ 0.28964,   0,   22.02320 },
					{ -0.00053,   0,   34.95600 },
					{ 0.15511,   1,   18.91320 },
					{ -0.96647,   1,    9.10692 },
					{ -0.17823,   2,    8.32663 },
					{ -0.02114,   2,    4.88396 },
					{ 0.01026,   2,    3.41639 },
					{ -0.00490,   2,    2.83054 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.10674,   0,   22.02320 },
					{ -0.00108,   0,   34.95600 },
					{ 0.05810,   1,   18.91320 },
					{ -0.37400,   1,    9.10692 },
					{ -0.22716,   2,    8.32663 },
					{ 0.43393,   2,    4.88396 },
					{ 0.68717,   2,    3.41639 },
					{ 0.07039,   2,    2.83054 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.70437,   1,    9.97373 },
					{ 0.05060,   1,   16.97770 },
					{ 0.29369,   2,    8.23656 },
					{ 0.00749,   2,    4.73340 },
					{ 0.00100,   2,    3.05281 },
					{ -0.00050,   2,    2.02182 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.24843,   1,    9.97373 },
					{ -0.01832,   1,   16.97770 },
					{ -0.16204,   2,    8.23656 },
					{ 0.38494,   2,    4.73340 },
					{ 0.71878,   2,    3.05281 },
					{ 0.03219,   2,    2.02182 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.61402,   2,    2.22666 },
					{ 0.03463,   2,    9.19926 },
					{ 0.31729,   2,    4.45190 },
					{ 0.11104,   2,    3.45709 },
					{ 0.03178,   2,    1.37743 }
				}
			}
		}
	},
	{
		"V4+", 23, 4,
		6,
		{
			{
				1, 0, // 1S
				8,
				{
					{ -0.94834,   0,   22.98030 },
					{ -0.01612,   0,   36.37280 },
					{ -0.04698,   1,   19.36360 },
					{ 0.00141,   1,    9.29061 },
					{ -0.00218,   2,    8.18536 },
					{ 0.00208,   2,    5.31087 },
					{ -0.00166,   2,    3.79841 },
					{ 0.00067,   2,    3.02661 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ 0.29873,   0,   22.98030 },
					{ -0.00351,   0,   36.37280 },
					{ 0.15933,   1,   19.36360 },
					{ -1.05908,   1,    9.29061 },
					{ -0.08622,   2,    8.18536 },
					{ -0.02150,   2,    5.31087 },
					{ 0.01078,   2,    3.79841 },
					{ -0.00457,   2,    3.02661 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.11000,   0,   22.98030 },
					{ -0.00057,   0,   36.37280 },
					{ 0.06868,   1,   19.36360 },
					{ -0.44277,   1,    9.29061 },
					{ -0.21151,   2,    8.18536 },
					{ 0.41803,   2,    5.31087 },
					{ 0.71787,   2,    3.79841 },
					{ 0.08970,   2,    3.02661 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.67382,   1,   10.45830 },
					{ 0.07011,   1,   16.75840 },
					{ 0.29443,   2,    8.98880 },
					{ 0.03170,   2,    4.82249 },
					{ -0.02674,   2,    3.56805 },
					{ 0.01290,   2,    3.09750 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.25019,   1,   10.45830 },
					{ -0.02514,   1,   16.75840 },
					{ -0.15557,   2,    8.98880 },
					{ 0.41730,   2,    4.82249 },
					{ 0.29021,   2,    3.56805 },
					{ 0.41276,   2,    3.09750 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.52539,   2,    2.83724 },
					{ 0.03110,   2,    9.99502 },
					{ 0.33798,   2,    4.81778 },
					{ 0.17308,   2,    2.18827 },
					{ 0.02473,   2,    6.30589 }
				}
			}
		}
	},
	{
		"V3+", 23, 3,
		6,
		{
			{
				1, 0, // 1S
				8,
				{
					{ -0.94620,   0,   22.99560 },
					{ -0.01675,   0,   36.39660 },
					{ -0.04758,   1,   19.73170 },
					{ -0.00080,   1,    9.19475 },
					{ -0.00020,   2,    7.98035 },
					{ 0.00045,   2,    5.17853 },
					{ -0.00064,   2,    3.33366 },
					{ 0.00046,   2,    3.02860 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ 0.29397,   0,   22.99560 },
					{ -0.00124,   0,   36.39660 },
					{ 0.15570,   1,   19.73170 },
					{ -1.07054,   1,    9.19475 },
					{ -0.06893,   2,    7.98035 },
					{ -0.01692,   2,    5.17853 },
					{ 0.01330,   2,    3.33366 },
					{ -0.00963,   2,    3.02860 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.10639,   0,   22.99560 },
					{ 0.00017,   0,   36.39660 },
					{ 0.06541,   1,   19.73170 },
					{ -0.43891,   1,    9.19475 },
					{ -0.22105,   2,    7.98035 },
					{ 0.58537,   2,    5.17853 },
					{ 0.84313,   2,    3.33366 },
					{ -0.18165,   2,    3.02860 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.66668,   1,   10.54650 },
					{ 0.06767,   1,   16.76970 },
					{ 0.30623,   2,    8.99295 },
					{ 0.01953,   2,    5.12169 },
					{ -0.00297,   2,    3.21967 },
					{ 0.00046,   2,    1.48920 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.23376,   1,   10.54650 },
					{ -0.02631,   1,   16.76970 },
					{ -0.17402,   2,    8.99295 },
					{ 0.37704,   2,    5.12169 },
					{ 0.74974,   2,    3.21967 },
					{ 0.01256,   2,    1.48920 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.36443,   2,    2.81227 },
					{ 0.03298,   2,    9.81281 },
					{ 0.34944,   2,    4.82096 },
					{ 0.34740,   2,    2.18658 },
					{ 0.01698,   2,    1.18344 }
				}
			}
		}
	},
	{
		"V2+", 23, 2,
		6,
		{
			{
				1, 0, // 1S
				8,
				{
					{ -0.95270,   0,   22.68620 },
					{ -0.02422,   0,   35.32790 },
					{ -0.03037,   1,   19.06680 },
					{ -0.00313,   1,    9.16308 },
					{ 0.00232,   2,    8.08172 },
					{ -0.00212,   2,    5.23907 },
					{ 0.00201,   2,    3.97752 },
					{ -0.00076,   2,    3.25320 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ 0.30793,   0,   22.68620 },
					{ -0.00549,   0,   35.32790 },
					{ 0.15110,   1,   19.06680 },
					{ -1.09209,   1,    9.16308 },
					{ -0.03887,   2,    8.08172 },
					{ -0.05049,   2,    5.23907 },
					{ 0.03883,   2,    3.97752 },
					{ -0.01558,   2,    3.25320 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.11022,   0,   22.68620 },
					{ -0.00152,   0,   35.32790 },
					{ 0.06302,   1,   19.06680 },
					{ -0.44102,   1,    9.16308 },
					{ -0.19546,   2,    8.08172 },
					{ 0.48594,   2,    5.23907 },
					{ 0.22940,   2,    3.97752 },
					{ 0.51653,   2,    3.25320 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.68354,   1,   10.61930 },
					{ 0.05466,   1,   17.22680 },
					{ 0.30923,   2,    8.83331 },
					{ 0.00920,   2,    5.06617 },
					{ 0.00115,   2,    3.10519 },
					{ -0.00036,   2,    1.70725 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.23471,   1,   10.61930 },
					{ -0.02114,   1,   17.22680 },
					{ -0.17516,   2,    8.83331 },
					{ 0.41367,   2,    5.06617 },
					{ 0.71034,   2,    3.10519 },
					{ 0.02683,   2,    1.70725 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.42372,   2,    2.92393 },
					{ 0.03246,   2,    9.46905 },
					{ 0.27666,   2,    4.93457 },
					{ 0.38574,   2,    1.82643 },
					{ 0.01811,   2,    1.19855 }
				}
			}
		}
	},
	{
		"Cr3+", 24, 3,
		6,
		{
			{
				1, 0, // 1S
				8,
				{
					{ -0.92735,   0,   24.20250 },
					{ -0.02271,   0,   35.46650 },
					{ -0.06167,   1,   21.57540 },
					{ -0.00153,   1,   10.05530 },
					{ -0.00010,   2,    9.62032 },
					{ 0.00071,   2,    5.65407 },
					{ -0.00073,   2,    4.08070 },
					{ 0.00034,   2,    3.32977 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ -0.27648,   0,   24.20250 },
					{ -0.00772,   0,   35.46650 },
					{ -0.15856,   1,   21.57540 },
					{ 0.94313,   1,   10.05530 },
					{ 0.18866,   2,    9.62032 },
					{ 0.02969,   2,    5.65407 },
					{ -0.01142,   2,    4.08070 },
					{ 0.00440,   2,    3.32977 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.10643,   0,   24.20250 },
					{ 0.00028,   0,   35.46650 },
					{ 0.05643,   1,   21.57540 },
					{ -0.36017,   1,   10.05530 },
					{ -0.24176,   2,    9.62032 },
					{ 0.34457,   2,    5.65407 },
					{ 0.59179,   2,    4.08070 },
					{ 0.25479,   2,    3.32977 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.67446,   1,   10.78810 },
					{ 0.08743,   1,   16.80730 },
					{ 0.27072,   2,    9.45924 },
					{ 0.02244,   2,    5.87748 },
					{ -0.00137,   2,    3.48939 },
					{ 0.00029,   2,    1.76997 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.22708,   1,   10.78810 },
					{ -0.03837,   1,   16.80730 },
					{ -0.19541,   2,    9.45924 },
					{ 0.33518,   2,    5.87748 },
					{ 0.80808,   2,    3.48939 },
					{ 0.02186,   2,    1.76997 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.33577,   2,    3.31765 },
					{ 0.03130,   2,   10.49770 },
					{ 0.31164,   2,    5.29867 },
					{ 0.42213,   2,    2.31223 },
					{ 0.01304,   2,    1.28257 }
				}
			}
		}
	},
	{
		"Cr2+", 24, 2,
		6,
		{
			{
				1, 0, // 1S
				8,
				{
					{ -0.93454,   0,   23.89530 },
					{ -0.02900,   0,   35.24250 },
					{ -0.04445,   1,   21.44100 },
					{ -0.00520,   1,    9.99196 },
					{ 0.00298,   2,    9.55952 },
					{ -0.00148,   2,    5.61823 },
					{ 0.00127,   2,    4.35735 },
					{ -0.00035,   2,    3.30865 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ -0.28126,   0,   23.89530 },
					{ -0.00810,   0,   35.24250 },
					{ -0.15362,   1,   21.44100 },
					{ 0.96098,   1,    9.99196 },
					{ 0.17024,   2,    9.55952 },
					{ 0.03224,   2,    5.61823 },
					{ -0.01261,   2,    4.35735 },
					{ 0.00320,   2,    3.30865 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.10551,   0,   23.89530 },
					{ 0.00117,   0,   35.24250 },
					{ 0.05581,   1,   21.44100 },
					{ -0.36746,   1,    9.99196 },
					{ -0.22357,   2,    9.55952 },
					{ 0.30261,   2,    5.61823 },
					{ 0.47754,   2,    4.35735 },
					{ 0.40922,   2,    3.30865 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.67579,   1,   10.56370 },
					{ 0.10345,   1,   16.48250 },
					{ 0.24160,   2,    9.56221 },
					{ 0.03302,   2,    6.23083 },
					{ -0.00177,   2,    3.50921 },
					{ 0.00053,   2,    1.77028 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.21199,   1,   10.56370 },
					{ -0.04730,   1,   16.48250 },
					{ -0.20450,   2,    9.56221 },
					{ 0.30001,   2,    6.23083 },
					{ 0.83439,   2,    3.50921 },
					{ 0.04843,   2,    1.77028 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.25314,   2,    3.69953 },
					{ 0.03242,   2,   10.20870 },
					{ 0.25485,   2,    5.30439 },
					{ 0.49184,   2,    2.32695 },
					{ 0.10781,   2,    1.47575 }
				}
			}
		}
	},
	{
		"Mn3+", 25, 3,
		6,
		{
			{
				1, 0, // 1S
				8,
				{
					{ -0.94814,   0,   24.87930 },
					{ -0.02101,   0,   38.01670 },
					{ -0.03987,   1,   21.34430 },
					{ -0.00105,   1,   10.78720 },
					{ -0.00000,   2,    9.86232 },
					{ 0.00036,   2,    5.79499 },
					{ -0.00059,   2,    4.26958 },
					{ 0.00040,   2,    3.89408 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ 0.29775,   0,   24.87930 },
					{ 0.00007,   0,   38.01670 },
					{ 0.16717,   1,   21.34430 },
					{ -0.94130,   1,   10.78720 },
					{ -0.22219,   2,    9.86232 },
					{ -0.01889,   2,    5.79499 },
					{ 0.00896,   2,    4.26958 },
					{ -0.00540,   2,    3.89408 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.11383,   0,   24.87930 },
					{ -0.00178,   0,   38.01670 },
					{ 0.06154,   1,   21.34430 },
					{ -0.36575,   1,   10.78720 },
					{ -0.26765,   2,    9.86232 },
					{ 0.49578,   2,    5.79499 },
					{ -0.08577,   2,    4.26958 },
					{ 0.80025,   2,    3.89408 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ -0.86866,   1,    9.60545 },
					{ -0.13928,   1,   16.53070 },
					{ 0.02692,   2,    6.14581 },
					{ -0.06150,   2,    5.56589 },
					{ 0.03497,   2,    3.40991 },
					{ -0.02227,   2,    3.00537 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.35028,   1,    9.60545 },
					{ -0.04167,   1,   16.53070 },
					{ -0.38270,   2,    6.14581 },
					{ 0.87627,   2,    5.56589 },
					{ 0.53558,   2,    3.40991 },
					{ 0.07692,   2,    3.00537 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.34299,   2,    3.60415 },
					{ 0.03216,   2,   10.93990 },
					{ 0.30117,   2,    5.67612 },
					{ 0.42553,   2,    2.43096 },
					{ 0.01531,   2,    1.38690 }
				}
			}
		}
	},
	{
		"Mn2+", 25, 2,
		6,
		{
			{
				1, 0, // 1S
				8,
				{
					{ -0.94657,   0,   24.95320 },
					{ -0.01952,   0,   38.12890 },
					{ -0.04418,   1,   21.40630 },
					{ 0.00067,   1,   10.81970 },
					{ -0.00208,   2,    9.89126 },
					{ 0.00241,   2,    7.02482 },
					{ -0.00139,   2,    5.70696 },
					{ 0.00019,   2,    3.69431 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ -0.29616,   0,   24.95320 },
					{ -0.00022,   0,   38.12890 },
					{ -0.16846,   1,   21.40630 },
					{ 0.93413,   1,   10.81970 },
					{ 0.22835,   2,    9.89126 },
					{ 0.00613,   2,    7.02482 },
					{ 0.01079,   2,    5.70696 },
					{ -0.00019,   2,    3.69431 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.10977,   0,   24.95320 },
					{ -0.00055,   0,   38.12890 },
					{ 0.06518,   1,   21.40630 },
					{ -0.36791,   1,   10.81970 },
					{ -0.23561,   2,    9.89126 },
					{ -0.07922,   2,    7.02482 },
					{ 0.61666,   2,    5.70696 },
					{ 0.64997,   2,    3.69431 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ -0.86962,   1,    9.61830 },
					{ -0.13746,   1,   16.57800 },
					{ 0.03006,   2,    6.16383 },
					{ -0.06466,   2,    5.64417 },
					{ 0.02339,   2,    3.47672 },
					{ -0.01188,   2,    2.83848 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.34328,   1,    9.61830 },
					{ -0.04065,   1,   16.57800 },
					{ -0.46266,   2,    6.16383 },
					{ 0.93328,   2,    5.64417 },
					{ 0.46089,   2,    3.47672 },
					{ 0.17862,   2,    2.83848 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.27716,   2,    3.65559 },
					{ 0.03372,   2,   10.72370 },
					{ 0.29360,   2,    5.53874 },
					{ 0.45627,   2,    2.37383 },
					{ 0.08058,   2,    1.48214 }
				}
			}
		}
	},
	{
		"Fe3+", 26, 3,
		6,
		{
			{
				1, 0, // 1S
				8,
				{
					{ -0.95612,   0,   25.56420 },
					{ -0.02669,   0,   39.12300 },
					{ -0.02145,   1,   21.75300 },
					{ -0.00624,   1,   11.12230 },
					{ 0.00687,   2,    9.59524 },
					{ -0.00800,   2,    7.06020 },
					{ -0.00059,   2,    3.93600 },
					{ 0.00483,   2,    5.81447 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ -0.30621,   0,   25.56420 },
					{ 0.00053,   0,   39.12300 },
					{ -0.17232,   1,   21.75300 },
					{ 1.00260,   1,   11.12230 },
					{ 0.19940,   2,    9.59524 },
					{ -0.04601,   2,    7.06020 },
					{ -0.00275,   2,    3.93600 },
					{ 0.03302,   2,    5.81447 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.11332,   0,   25.56420 },
					{ 0.00031,   0,   39.12300 },
					{ 0.07322,   1,   21.75300 },
					{ -0.41624,   1,   11.12230 },
					{ -0.27405,   2,    9.59524 },
					{ 0.10304,   2,    7.06020 },
					{ 0.63599,   2,    3.93600 },
					{ 0.51945,   2,    5.81447 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ -0.86817,   1,   10.07620 },
					{ -0.13889,   1,   17.19370 },
					{ 0.02670,   2,    6.14060 },
					{ -0.05797,   2,    5.79810 },
					{ 0.01824,   2,    3.51400 },
					{ -0.00906,   2,    2.82800 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.35664,   1,   10.07620 },
					{ -0.04049,   1,   17.19370 },
					{ -0.66827,   2,    6.14060 },
					{ 1.19420,   2,    5.79810 },
					{ 0.56136,   2,    3.51400 },
					{ 0.02384,   2,    2.82800 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.45214,   2,    3.65374 },
					{ 0.03091,   2,   11.50000 },
					{ 0.29862,   2,    6.09290 },
					{ 0.33661,   2,    2.35365 },
					{ 0.00050,   2,    1.40000 }
				}
			}
		}
	},
	{
		"Fe2+", 26, 2,
		6,
		{
			{
				1, 0, // 1S
				8,
				{
					{ -0.95555,   0,   25.71370 },
					{ -0.02255,   0,   39.05300 },
					{ -0.03136,   1,   20.81600 },
					{ 0.00214,   1,   11.13180 },
					{ -0.00258,   2,    9.17491 },
					{ 0.00259,   2,    6.42077 },
					{ -0.00183,   2,    4.70947 },
					{ 0.00067,   2,    3.76114 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ -0.31636,   0,   25.71370 },
					{ 0.00748,   0,   39.05300 },
					{ -0.18979,   1,   20.81600 },
					{ 1.04533,   1,   11.13180 },
					{ 0.17352,   2,    9.17491 },
					{ -0.02793,   2,    6.42077 },
					{ 0.02014,   2,    4.70947 },
					{ -0.00558,   2,    3.76114 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.11613,   0,   25.71370 },
					{ -0.00233,   0,   39.05300 },
					{ 0.08114,   1,   20.81600 },
					{ -0.43452,   1,   11.13180 },
					{ -0.30260,   2,    9.17491 },
					{ 0.51965,   2,    6.42077 },
					{ 0.27831,   2,    4.70947 },
					{ 0.50592,   2,    3.76114 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ -0.86260,   1,   10.04700 },
					{ -0.14532,   1,   17.01000 },
					{ 0.01250,   2,    6.76323 },
					{ -0.04233,   2,    5.86095 },
					{ 0.01532,   2,    3.64334 },
					{ -0.00682,   2,    2.82298 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.34696,   1,   10.04700 },
					{ -0.04298,   1,   17.01000 },
					{ -0.22344,   2,    6.76323 },
					{ 0.69419,   2,    5.86095 },
					{ 0.51938,   2,    3.64334 },
					{ 0.12428,   2,    2.82298 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.39478,   2,    3.82038 },
					{ 0.03041,   2,   11.38040 },
					{ 0.26069,   2,    6.12682 },
					{ 0.42103,   2,    2.26586 },
					{ 0.04002,   2,    1.41149 }
				}
			}
		}
	},
	{
		"Co3+", 27, 3,
		6,
		{
			{
				1, 0, // 1S
				8,
				{
					{ -0.97197,   0,   26.24860 },
					{ -0.02587,   0,   42.23860 },
					{ 0.00177,   1,   22.47140 },
					{ -0.01732,   1,   11.90340 },
					{ 0.01447,   2,   10.78270 },
					{ -0.01072,   2,    7.15000 },
					{ 0.00713,   2,    5.31983 },
					{ -0.00192,   2,    3.91429 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ -0.31253,   0,   26.24860 },
					{ 0.00015,   0,   42.23860 },
					{ -0.16942,   1,   22.47140 },
					{ 0.94464,   1,   11.90340 },
					{ 0.23487,   2,   10.78270 },
					{ 0.01199,   2,    7.15000 },
					{ 0.00410,   2,    5.31983 },
					{ -0.00052,   2,    3.91429 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.11864,   0,   26.24860 },
					{ -0.00076,   0,   42.23860 },
					{ 0.06659,   1,   22.47140 },
					{ -0.37776,   1,   11.90340 },
					{ -0.27445,   2,   10.78270 },
					{ 0.19911,   2,    7.15000 },
					{ 0.58195,   2,    5.31983 },
					{ 0.43774,   2,    3.91429 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.87655,   1,   11.14440 },
					{ 0.08123,   1,   19.63950 },
					{ 0.08768,   2,    8.08691 },
					{ -0.01509,   2,    6.07049 },
					{ 0.00344,   2,    3.70410 },
					{ -0.00072,   2,    2.97054 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.34847,   1,   11.14440 },
					{ -0.02399,   1,   19.63950 },
					{ -0.14664,   2,    8.08691 },
					{ 0.61672,   2,    6.07049 },
					{ 0.59523,   2,    3.70410 },
					{ 0.01708,   2,    2.97054 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.34086,   2,    4.79634 },
					{ 0.02683,   2,   12.32740 },
					{ 0.19581,   2,    6.92523 },
					{ 0.51593,   2,    2.85514 },
					{ 0.04686,   2,    1.66429 }
				}
			}
		}
	},
	{
		"Co2+", 27, 2,
		6,
		{
			{
				1, 0, // 1S
				8,
				{
					{ -0.96949,   0,   26.31870 },
					{ -0.02527,   0,   42.13220 },
					{ -0.00320,   1,   22.41430 },
					{ -0.01491,   1,   11.70850 },
					{ 0.01224,   2,   10.75420 },
					{ -0.00773,   2,    7.13183 },
					{ 0.00483,   2,    5.06381 },
					{ -0.00146,   2,    3.72268 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ -0.31369,   0,   26.31870 },
					{ 0.00137,   0,   42.13220 },
					{ -0.16824,   1,   22.41430 },
					{ 0.97792,   1,   11.70850 },
					{ 0.19024,   2,   10.75420 },
					{ 0.02407,   2,    7.13183 },
					{ -0.00304,   2,    5.06381 },
					{ 0.00152,   2,    3.72268 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.11873,   0,   26.31870 },
					{ -0.00161,   0,   42.13220 },
					{ 0.06378,   1,   22.41430 },
					{ -0.38327,   1,   11.70850 },
					{ -0.26754,   2,   10.75420 },
					{ 0.25673,   2,    7.13183 },
					{ 0.61568,   2,    5.06381 },
					{ 0.35528,   2,    3.72268 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.87045,   1,   11.14100 },
					{ 0.08485,   1,   19.42750 },
					{ 0.08327,   2,    8.32565 },
					{ -0.00735,   2,    5.75727 },
					{ 0.00256,   2,    3.57166 },
					{ -0.00029,   2,    2.24143 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.34593,   1,   11.14100 },
					{ -0.02343,   1,   19.42750 },
					{ -0.08083,   2,    8.32565 },
					{ 0.60577,   2,    5.75727 },
					{ 0.54526,   2,    3.57166 },
					{ 0.02063,   2,    2.24143 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.36301,   2,    4.95296 },
					{ 0.02707,   2,   12.29630 },
					{ 0.14777,   2,    7.03565 },
					{ 0.49771,   2,    2.74850 },
					{ 0.11388,   2,    1.69027 }
				}
			}
		}
	},
	{
		"Ni2+", 28, 2,
		6,
		{
			{
				1, 0, // 1S
				8,
				{
					{ -0.95915,   0,   27.44620 },
					{ -0.02873,   0,   41.47700 },
					{ -0.01623,   1,   22.39170 },
					{ -0.00402,   1,   12.49460 },
					{ 0.00311,   2,   10.32110 },
					{ -0.00283,   2,    6.90563 },
					{ 0.00237,   2,    5.00445 },
					{ -0.00094,   2,    4.04433 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ -0.31765,   0,   27.44620 },
					{ 0.00412,   0,   41.47700 },
					{ -0.20486,   1,   22.39170 },
					{ 1.00594,   1,   12.49460 },
					{ 0.24400,   2,   10.32110 },
					{ -0.04791,   2,    6.90563 },
					{ 0.03991,   2,    5.00445 },
					{ -0.01434,   2,    4.04433 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.11827,   0,   27.44620 },
					{ -0.00129,   0,   41.47700 },
					{ 0.08729,   1,   22.39170 },
					{ -0.41967,   1,   12.49460 },
					{ -0.31399,   2,   10.32110 },
					{ 0.53349,   2,    6.90563 },
					{ 0.30346,   2,    5.00445 },
					{ 0.45091,   2,    4.04433 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.82845,   1,   11.47560 },
					{ 0.11591,   1,   19.07920 },
					{ 0.05842,   2,   11.22600 },
					{ 0.05802,   2,    6.26272 },
					{ -0.03491,   2,    4.83135 },
					{ 0.00823,   2,    3.40620 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.32582,   1,   11.47560 },
					{ -0.03683,   1,   19.07920 },
					{ -0.05319,   2,   11.22600 },
					{ 0.37067,   2,    6.26272 },
					{ 0.39265,   2,    4.83135 },
					{ 0.37098,   2,    3.40620 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.35355,   2,    4.86729 },
					{ 0.02753,   2,   12.83450 },
					{ 0.20286,   2,    7.12212 },
					{ 0.48187,   2,    2.75130 },
					{ 0.08941,   2,    1.61913 }
				}
			}
		}
	},
	{
		"Cu2+", 29, 2,
		6,
		{
			{
				1, 0, // 1S
				8,
				{
					{ -0.95804,   0,   28.44790 },
					{ -0.02945,   0,   42.54010 },
					{ -0.01634,   1,   23.56670 },
					{ -0.00451,   1,   12.88110 },
					{ 0.00285,   2,   11.52970 },
					{ -0.00306,   2,    6.43914 },
					{ 0.00496,   2,    5.08981 },
					{ -0.00271,   2,    4.55572 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ 0.32104,   0,   28.44790 },
					{ -0.00547,   0,   42.54010 },
					{ 0.18777,   1,   23.56670 },
					{ -0.98233,   1,   12.88110 },
					{ -0.21631,   2,   11.52970 },
					{ -0.02331,   2,    6.43914 },
					{ 0.02096,   2,    5.08981 },
					{ -0.01121,   2,    4.55572 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.12095,   0,   28.44790 },
					{ -0.00228,   0,   42.54010 },
					{ 0.07760,   1,   23.56670 },
					{ -0.40305,   1,   12.88110 },
					{ -0.25376,   2,   11.52970 },
					{ 0.80148,   2,    6.43914 },
					{ -0.67321,   2,    5.08981 },
					{ 1.08895,   2,    4.55572 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.84736,   1,   12.07520 },
					{ 0.09934,   1,   20.14760 },
					{ 0.07609,   2,   10.15520 },
					{ 0.01339,   2,    5.96866 },
					{ -0.00521,   2,    3.67422 },
					{ 0.00226,   2,    2.56840 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.35070,   1,   12.07520 },
					{ -0.02594,   1,   20.14760 },
					{ -0.03735,   2,   10.15520 },
					{ 0.66686,   2,    5.96866 },
					{ 0.46276,   2,    3.67422 },
					{ 0.00279,   2,    2.56840 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.41646,   2,    4.48665 },
					{ 0.02630,   2,   13.26420 },
					{ 0.25763,   2,    7.36110 },
					{ 0.35074,   2,    2.67500 },
					{ 0.10582,   2,    1.84690 }
				}
			}
		}
	},
	{
		"Zn2+", 30, 2,
		6,
		{
			{
				1, 0, // 1S
				8,
				{
					{ -0.96049,   0,   29.29720 },
					{ -0.03203,   0,   43.94410 },
					{ -0.00890,   1,   24.39300 },
					{ -0.00735,   1,   13.03440 },
					{ 0.00499,   2,   11.65720 },
					{ -0.00290,   2,    7.07461 },
					{ 0.00445,   2,    4.60368 },
					{ -0.00304,   2,    4.15360 }
				}
			},
			{
				2, 0, // 2S
				8,
				{
					{ 0.32478,   0,   29.29720 },
					{ -0.00559,   0,   43.94410 },
					{ 0.18359,   1,   24.39300 },
					{ -1.03866,   1,   13.03440 },
					{ -0.15209,   2,   11.65720 },
					{ -0.01787,   2,    7.07461 },
					{ 0.01169,   2,    4.60368 },
					{ -0.00836,   2,    4.15360 }
				}
			},
			{
				3, 0, // 3S
				8,
				{
					{ 0.12210,   0,   29.29720 },
					{ -0.00194,   0,   43.94410 },
					{ 0.07832,   1,   24.39300 },
					{ -0.43354,   1,   13.03440 },
					{ -0.24477,   2,   11.65720 },
					{ 0.60751,   2,    7.07461 },
					{ 0.55998,   2,    4.60368 },
					{ 0.07194,   2,    4.15360 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.85325,   1,   12.35130 },
					{ 0.11401,   1,   20.28450 },
					{ 0.05264,   2,   10.10640 },
					{ 0.01129,   2,    6.93875 },
					{ -0.00084,   2,    4.15538 },
					{ 0.00035,   2,    1.88801 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.34501,   1,   12.35130 },
					{ -0.03491,   1,   20.28450 },
					{ -0.09628,   2,   10.10640 },
					{ 0.56458,   2,    6.93875 },
					{ 0.61746,   2,    4.15538 },
					{ 0.01288,   2,    1.88801 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.41026,   2,    4.92159 },
					{ 0.02418,   2,   13.92570 },
					{ 0.23248,   2,    7.92771 },
					{ 0.37314,   2,    2.89899 },
					{ 0.11920,   2,    1.91089 }
				}
			}
		}
	},
	{
		"O-", 8, -1,
		3,
		{
			{
				1, 0, // 1S
				5,
				{
					{ 0.93779,   0,    7.61467 },
					{ 0.03825,   0,   13.42550 },
					{ 0.00206,   1,    3.21269 },
					{ 0.03520,   1,    6.31491 },
					{ 0.00040,   1,    1.75076 }
				}
			},
			{
				2, 0, // 2S
				5,
				{
					{ -0.21102,   0,    7.61467 },
					{ -0.00681,   0,   13.42550 },
					{ 0.51254,   1,    3.21269 },
					{ -0.11661,   1,    6.31491 },
					{ 0.62979,   1,    1.75076 }
				}
			},
			{
				2, 1, // 2P
				5,
				{
					{ 0.52755,   1,    1.74399 },
					{ 0.30564,   1,    3.42975 },
					{ 0.28894,   1,    0.86393 },
					{ 0.00890,   1,    0.40831 },
					{ 0.01376,   1,    7.82044 }
				}
			}
		}
	},
	{
		"F-", 9, -1,
		3,
		{
			{
				1, 0, // 1S
				5,
				{
					{ 0.89308,   0,    8.93690 },
					{ 0.03498,   0,   14.84990 },
					{ 0.00446,   1,    3.27783 },
					{ 0.08884,   1,    8.10301 },
					{ -0.00055,   1,    1.85159 }
				}
			},
			{
				2, 0, // 2S
				5,
				{
					{ -0.20022,   0,    8.93690 },
					{ -0.00852,   0,   14.84990 },
					{ 0.62675,   1,    3.27783 },
					{ -0.09809,   1,    8.10301 },
					{ 0.48432,   1,    1.85159 }
				}
			},
			{
				2, 1, // 2P
				5,
				{
					{ 0.47039,   1,    2.07537 },
					{ 0.30842,   1,    3.93342 },
					{ 0.09885,   1,    1.46603 },
					{ 0.24705,   1,    0.95683 },
					{ 0.01686,   1,    8.32950 }
				}
			}
		}
	},
	{
		"S-", 16, -1,
		5,
		{
			{
				1, 0, // 1S
				7,
				{
					{ 0.92673,   0,   15.78440 },
					{ 0.03436,   0,   24.31730 },
					{ 0.00485,   1,    6.72829 },
					{ 0.04839,   1,   14.02420 },
					{ 0.00025,   2,    2.55720 },
					{ -0.00012,   2,    1.48480 },
					{ -0.00131,   2,    5.68198 }
				}
			},
			{
				2, 0, // 2S
				7,
				{
					{ -0.25928,   0,   15.78440 },
					{ -0.00781,   0,   24.31730 },
					{ 0.80034,   1,    6.72829 },
					{ -0.13560,   1,   14.02420 },
					{ 0.00504,   2,    2.55720 },
					{ -0.00015,   2,    1.48480 },
					{ 0.35078,   2,    5.68198 }
				}
			},
			{
				3, 0, // 3S
				7,
				{
					{ -0.07378,   0,   15.78440 },
					{ -0.00186,   0,   24.31730 },
					{ 0.24398,   1,    6.72829 },
					{ -0.04061,   1,   14.02420 },
					{ -0.69742,   2,    2.55720 },
					{ -0.43754,   2,    1.48480 },
					{ 0.17086,   2,    5.68198 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.65158,   1,    7.07744 },
					{ 0.03916,   1,   12.99860 },
					{ 0.01316,   2,    2.62059 },
					{ 0.00095,   2,    0.89975 },
					{ 0.37601,   2,    5.42495 },
					{ -0.00214,   2,    1.65797 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.15153,   1,    7.07744 },
					{ -0.00927,   1,   12.99860 },
					{ 0.29742,   2,    2.62059 },
					{ 0.33297,   2,    0.89975 },
					{ -0.10428,   2,    5.42495 },
					{ 0.52358,   2,    1.65797 }
				}
			}
		}
	},
	{
		"Cl-", 17, -1,
		5,
		{
			{
				1, 0, // 1S
				7,
				{
					{ 0.92389,   0,   17.24050 },
					{ 0.01357,   0,   28.92250 },
					{ 0.00274,   1,    6.81054 },
					{ 0.07693,   1,   15.17810 },
					{ 0.00002,   2,    2.87181 },
					{ -0.00002,   2,    1.67233 },
					{ -0.00055,   2,    5.86922 }
				}
			},
			{
				2, 0, // 2S
				7,
				{
					{ -0.25995,   0,   17.24050 },
					{ -0.00339,   0,   28.92250 },
					{ 0.90297,   1,    6.81054 },
					{ -0.14067,   1,   15.17810 },
					{ 0.00350,   2,    2.87181 },
					{ 0.00057,   2,    1.67233 },
					{ 0.23387,   2,    5.86922 }
				}
			},
			{
				3, 0, // 3S
				7,
				{
					{ -0.07613,   0,   17.24050 },
					{ -0.00122,   0,   28.92250 },
					{ 0.29244,   1,    6.81054 },
					{ -0.04564,   1,   15.17810 },
					{ -0.69781,   2,    2.87181 },
					{ -0.45281,   2,    1.67233 },
					{ 0.15074,   2,    5.86922 }
				}
			},
			{
				2, 1, // 2P
				6,
				{
					{ 0.65375,   1,    7.62972 },
					{ 0.03936,   1,   13.76380 },
					{ 0.01297,   2,    2.92646 },
					{ 0.00056,   2,    1.01565 },
					{ 0.36998,   2,    5.93996 },
					{ -0.00273,   2,    1.83188 }
				}
			},
			{
				3, 1, // 3P
				6,
				{
					{ -0.16223,   1,    7.62972 },
					{ -0.00998,   1,   13.76380 },
					{ 0.32959,   2,    2.92646 },
					{ 0.30269,   2,    1.01565 },
					{ -0.11336,   2,    5.93996 },
					{ 0.52436,   2,    1.83188 }
				}
			}
		}
	},
	{
		"Se-", 34, -1,
		8,
		{
			{
				1, 0, // 1S
				10,
				{
					{ 0.83803,   0,   31.69100 },
					{ 0.17586,   0,   40.78910 },
					{ -0.01822,   1,   27.22790 },
					{ 0.01010,   1,   15.10370 },
					{ -0.00566,   2,   13.64750 },
					{ 0.00222,   2,    8.43695 },
					{ -0.00104,   2,    5.76402 },
					{ 0.00033,   3,    3.33859 },
					{ -0.00023,   3,    2.21979 },
					{ 0.00008,   3,    1.44724 }
				}
			},
			{
				2, 0, // 2S
				10,
				{
					{ 0.34668,   0,   31.69100 },
					{ -0.00284,   0,   40.78910 },
					{ 0.17042,   1,   27.22790 },
					{ -1.03180,   1,   15.10370 },
					{ -0.16965,   2,   13.64750 },
					{ -0.01757,   2,    8.43695 },
					{ 0.00247,   2,    5.76402 },
					{ -0.00074,   3,    3.33859 },
					{ 0.00052,   3,    2.21979 },
					{ -0.00016,   3,    1.44724 }
				}
			},
			{
				3, 0, // 3S
				10,
				{
					{ -0.13299,   0,   31.69100 },
					{ -0.00124,   0,   40.78910 },
					{ -0.08016,   1,   27.22790 },
					{ 0.45390,   1,   15.10370 },
					{ 0.25982,   2,   13.64750 },
					{ -0.51134,   2,    8.43695 },
					{ -0.72233,   2,    5.76402 },
					{ -0.00899,   3,    3.33859 },
					{ 0.00148,   3,    2.21979 },
					{ -0.00050,   3,    1.44724 }
				}
			},
			{
				4, 0, // 4S
				10,
				{
					{ -0.03758,   0,   31.69100 },
					{ -0.00053,   0,   40.78910 },
					{ -0.02327,   1,   27.22790 },
					{ 0.13052,   1,   15.10370 },
					{ 0.08621,   2,   13.64750 },
					{ -0.17437,   2,    8.43695 },
					{ -0.25572,   2,    5.76402 },
					{ 0.45452,   3,    3.33859 },
					{ 0.50413,   3,    2.21979 },
					{ 0.19284,   3,    1.44724 }
				}
			},
			{
				2, 1, // 2P
				9,
				{
					{ 0.79107,   1,   14.89160 },
					{ 0.09984,   1,   23.42820 },
					{ 0.11305,   2,   14.10790 },
					{ 0.04195,   2,    8.71082 },
					{ -0.01398,   2,    5.84405 },
					{ 0.00506,   3,    4.58260 },
					{ -0.00156,   3,    2.66211 },
					{ 0.00064,   3,    1.56157 },
					{ -0.00020,   3,    0.89382 }
				}
			},
			{
				3, 1, // 3P
				9,
				{
					{ 0.32629,   1,   14.89160 },
					{ 0.03555,   1,   23.42820 },
					{ 0.11021,   2,   14.10790 },
					{ -0.33306,   2,    8.71082 },
					{ -0.74110,   2,    5.84405 },
					{ -0.08657,   3,    4.58260 },
					{ -0.00193,   3,    2.66211 },
					{ -0.00065,   3,    1.56157 },
					{ -0.00014,   3,    0.89382 }
				}
			},
			{
				4, 1, // 4P
				9,
				{
					{ 0.07426,   1,   14.89160 },
					{ 0.00703,   1,   23.42820 },
					{ 0.02345,   2,   14.10790 },
					{ -0.07668,   2,    8.71082 },
					{ -0.20000,   2,    5.84405 },
					{ 0.02368,   3,    4.58260 },
					{ 0.46992,   3,    2.66211 },
					{ 0.48894,   3,    1.56157 },
					{ 0.19639,   3,    0.89382 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.48271,   2,    4.61472 },
					{ 0.12133,   2,    2.86374 },
					{ 0.29718,   2,    7.14841 },
					{ 0.18569,   2,    9.71251 },
					{ 0.02895,   2,   15.80930 }
				}
			}
		}
	},
	{
		"Br-", 35, -1,
		8,
		{
			{
				1, 0, // 1S
				10,
				{
					{ 0.80855,   0,   32.83790 },
					{ 0.19674,   0,   41.13520 },
					{ -0.00607,   1,   26.42870 },
					{ 0.00564,   1,   15.94410 },
					{ -0.00262,   2,   13.73830 },
					{ 0.00106,   2,    8.75244 },
					{ -0.00048,   2,    5.99564 },
					{ 0.00014,   3,    3.63572 },
					{ -0.00009,   3,    2.37420 },
					{ 0.00003,   3,    1.51690 }
				}
			},
			{
				2, 0, // 2S
				10,
				{
					{ 0.37829,   0,   32.83790 },
					{ -0.02717,   0,   41.13520 },
					{ 0.20459,   1,   26.42870 },
					{ -1.05731,   1,   15.94410 },
					{ -0.19545,   2,   13.73830 },
					{ -0.00594,   2,    8.75244 },
					{ -0.00130,   2,    5.99564 },
					{ 0.00027,   3,    3.63572 },
					{ -0.00013,   3,    2.37420 },
					{ 0.00005,   3,    1.51690 }
				}
			},
			{
				3, 0, // 3S
				10,
				{
					{ -0.14606,   0,   32.83790 },
					{ 0.00771,   0,   41.13520 },
					{ -0.10114,   1,   26.42870 },
					{ 0.48088,   1,   15.94410 },
					{ 0.28103,   2,   13.73830 },
					{ -0.54032,   2,    8.75244 },
					{ -0.71411,   2,    5.99564 },
					{ -0.00495,   3,    3.63572 },
					{ -0.00132,   3,    2.37420 },
					{ 0.00023,   3,    1.51690 }
				}
			},
			{
				4, 0, // 4S
				10,
				{
					{ -0.04114,   0,   32.83790 },
					{ 0.00051,   0,   41.13520 },
					{ -0.03362,   1,   26.42870 },
					{ 0.15017,   1,   15.94410 },
					{ 0.09054,   2,   13.73830 },
					{ -0.17726,   2,    8.75244 },
					{ -0.29049,   2,    5.99564 },
					{ 0.45299,   3,    3.63572 },
					{ 0.55239,   3,    2.37420 },
					{ 0.15888,   3,    1.51690 }
				}
			},
			{
				2, 1, // 2P
				9,
				{
					{ 0.73379,   1,   15.80950 },
					{ 0.10307,   1,   23.96750 },
					{ 0.17236,   2,   14.97930 },
					{ 0.04791,   2,    8.60775 },
					{ -0.02504,   2,    5.87791 },
					{ 0.01002,   3,    5.16140 },
					{ -0.00191,   3,    2.81552 },
					{ 0.00091,   3,    1.68578 },
					{ -0.00030,   3,    1.04140 }
				}
			},
			{
				3, 1, // 3P
				9,
				{
					{ 0.30621,   1,   15.80950 },
					{ 0.03753,   1,   23.96750 },
					{ 0.12902,   2,   14.97930 },
					{ -0.41356,   2,    8.60775 },
					{ -0.67051,   2,    5.87791 },
					{ -0.06117,   3,    5.16140 },
					{ -0.01008,   3,    2.81552 },
					{ 0.00326,   3,    1.68578 },
					{ -0.00111,   3,    1.04140 }
				}
			},
			{
				4, 1, // 4P
				9,
				{
					{ 0.07770,   1,   15.80950 },
					{ 0.00719,   1,   23.96750 },
					{ 0.02682,   2,   14.97930 },
					{ -0.09153,   2,    8.60775 },
					{ -0.22932,   2,    5.87791 },
					{ 0.05351,   3,    5.16140 },
					{ 0.51702,   3,    2.81552 },
					{ 0.43505,   3,    1.68578 },
					{ 0.18893,   3,    1.04140 }
				}
			},
			{
				3, 2, // 3D
				5,
				{
					{ 0.46287,   2,    4.85590 },
					{ 0.11062,   2,    3.11959 },
					{ 0.31170,   2,    7.33254 },
					{ 0.19318,   2,   10.10240 },
					{ 0.02960,   2,   16.28710 }
				}
			}
		}
	}
};

void localWfnToEntry(
	LocalDef_Wfn &wfn,
	discamb::HC_WfnBankEntry entry)
{
	vector<size_t> closedShell(5); 
	size_t nElectrons;
	entry.atomic_number = wfn.atomicNumber;
	
	nElectrons = int(wfn.atomicNumber) + wfn.charge;

	closedShell[0] = 2;
	closedShell[0] = 10;
	closedShell[0] = 18;
	closedShell[0] = 36;
	closedShell[0] = 54;

}
	
// max subshell D
void stringToConfiguration(
    const string s,
    vector<vector<size_t> > &configuration)
{
    size_t i, shell, subshell, nElectrons;
    vector<string> words;

    configuration.clear();

    discamb::string_utilities::split(s, words, ')');

    // K(2 L(8 3S(2 3P(6 4S(0 3D(4 

    for (i = 0; i < words.size(); i++)
    {
        // shell e.g. K(2)
        if (!isdigit(words[i][0]))
        {
            if (words[i][0] == 'K')
                configuration.resize(1, vector<size_t>(1, 2));
            if (words[i][0] == 'L')
            {
                configuration.resize(2, vector<size_t>(2, 2));
                configuration[1][1] = 6;
            }
            if (words[i][0] == 'M')
            {
                configuration.resize(3, vector<size_t>(3, 2));
                configuration[2][1] = 6;
                configuration[2][2] = 10;
            }

        }
        else
        {
            shell = atoi((string() + words[i][0]).c_str());
            if (words[i][1] == 'S')
                subshell = 0;
            else
            {
                if (words[i][1] == 'P')
                    subshell = 1;
                else
                {
                    if (words[i][1] == 'D')
                        subshell = 2;
                    else
                    {
                        cout << "ERROR" << endl;
                        exit(1);
                    }
                }

            }

            nElectrons = atoi(words[i].substr(3).c_str());

            if (nElectrons > 0)
            {
                if (shell > configuration.size())
                    configuration.resize(shell);
                if (subshell + 1 > configuration[shell - 1].size())
                    configuration[shell - 1].resize(subshell + 1);
                configuration[shell - 1][subshell] = nElectrons;
            }

        }

    }
}

void setConfigurationData(
    map<string, vector<vector<size_t> > > &configurations)
{
    size_t i;
    vector<string> words;
    vector<vector<size_t> > configuration;
    //configuration.resize(68);

    //"Mn3+ K(2)L(8)3S(2)3P(6)4S(0)3D(4)"

    for (i = 0; i < 68; i++)
    {
        discamb::string_utilities::split(cr_configuration_data[i], words);
        stringToConfiguration(words[1], configuration);
        configurations[words[0]] = configuration;
    }

}

//struct Type
//{
//    string label;
//    string symbol;
//    int atomicNumber;
//    int nElectrons;
//    int charge;
//    int period;
//    char block;
//};

void valenceOrbitals(
    LocalDef_Wfn &wfn,
    const vector<vector<size_t> > &configuration,
    vector<pair<size_t, size_t> > &val_orb)
{

    size_t _no_valence[] = { 0, 2, 10, 18, 36, 54 };
    vector<size_t> no_valence(_no_valence, _no_valence + 6);
    size_t n_electrons;
    size_t outerShell = 1;
    size_t period = discamb::periodic_table::period(wfn.atomicNumber);
    char block = discamb::periodic_table::block(wfn.atomicNumber);

    n_electrons = wfn.atomicNumber - wfn.charge; 

    val_orb.clear();

    if (find(no_valence.begin(), no_valence.end(), n_electrons) != no_valence.end())
        return;

    if (wfn.charge > 0 && (n_electrons == 28 || n_electrons == 46))
        return;

    if (block == 'S' || block == 'P')
        val_orb.push_back(make_pair(period, 0));

    if (block == 'P')
        val_orb.push_back(make_pair(period, 1));


    // D block elements with 10 d electrons - no valence orbitals

    if (block == 'D')
        if (configuration.size()>2)
            if (configuration[2].size() > 2)
            {
                if (configuration[2][2] != 10)
                    val_orb.push_back(make_pair(3, 2));
                else
                    if (configuration.size()>3)
                        val_orb.push_back(make_pair(4, 0));
            }


}


} // anonymous namespace


namespace discamb{

ClementiRoettiData::ClementiRoettiData()
{
	size_t entryIdx;
	HC_WfnBankEntry entry;
    // LocalDef_Wfn wfns
    /*
    struct LocalDef_Wfn {
    std::string label;
    size_t atomicNumber;
    int charge;
    size_t nOrbs;
    LocalDef_Orbital orbs[20];
    };
    */
    map<string, vector<vector<size_t> > > configurations;
    vector<pair<size_t, size_t> > val_orb;
    pair<size_t, size_t> orbitalIndex;
    setConfigurationData(configurations);
    size_t stoIdx, nSTO;
    double angstrom = 1.0 / 0.52917721092;
    double normalizationFactor;

	for (entryIdx = 0; entryIdx < localDef_N_Entries; entryIdx++)
	{
        HC_WfnBankEntry entry;
        entry.atomic_number = wfns[entryIdx].atomicNumber;
        entry.charge = wfns[entryIdx].charge;
        valenceOrbitals(wfns[entryIdx], configurations[wfns[entryIdx].label], val_orb);
        
        entry.orbitals.resize(wfns[entryIdx].nOrbs);
        entry.orbital_occupancy.resize(wfns[entryIdx].nOrbs);
        entry.wfnAtomTypeName = wfns[entryIdx].label;


        for (size_t i = 0; i < wfns[entryIdx].nOrbs; i++)
        {
            orbitalIndex.first = wfns[entryIdx].orbs[i].principal_number;
            orbitalIndex.second = wfns[entryIdx].orbs[i].azimutal_number;
            if (find(val_orb.begin(), val_orb.end(), orbitalIndex) != val_orb.end())
                entry.valence_orbitals_indices.push_back(i);
            else
                entry.core_orbitals_indices.push_back(i);

            entry.orbital_occupancy[i] = configurations[wfns[entryIdx].label][orbitalIndex.first - 1][orbitalIndex.second];

            nSTO = wfns[entryIdx].orbs[i].nTerms;

            entry.orbitals[i].coefficients.resize(nSTO);
            entry.orbitals[i].exponents.resize(nSTO);
            entry.orbitals[i].power_r.resize(nSTO);

            for (stoIdx = 0; stoIdx < nSTO; stoIdx++)
            {
                
                entry.orbitals[i].exponents[stoIdx] = wfns[entryIdx].orbs[i].term[stoIdx].exponent * angstrom;
                entry.orbitals[i].power_r[stoIdx] = wfns[entryIdx].orbs[i].term[stoIdx].powerR;
                normalizationFactor = sto_atomic_wfn::stoNormalizationFactor(entry.orbitals[i].power_r[stoIdx],
                                                                             entry.orbitals[i].exponents[stoIdx]);
                
                entry.orbitals[i].coefficients[stoIdx] = normalizationFactor * 
                                                         wfns[entryIdx].orbs[i].term[stoIdx].coefficient;
            }

            entry.orbitals[i].azimuthal_number = wfns[entryIdx].orbs[i].azimutal_number;
            entry.orbitals[i].principal_number = wfns[entryIdx].orbs[i].principal_number;
        }
        
        mEntries[entry.wfnAtomTypeName] = entry;
	}
}

ClementiRoettiData::~ClementiRoettiData()
{

}

void ClementiRoettiData::getEntries(
    std::vector<discamb::HC_WfnBankEntry> &entries)
{
    std::map<std::string, discamb::HC_WfnBankEntry>::const_iterator it = mEntries.begin();
    entries.clear();
    for (; it != mEntries.end(); it++)
        entries.push_back(it->second);
}

const HC_WfnBankEntry &ClementiRoettiData::getEntry(
    const std::string &atomType, 
    bool &hasType) 
const
{
    hasType = (mEntries.find(atomType) != mEntries.end());

    if (hasType)
        return mEntries.find(atomType)->second;
    return mEmptyEntry;
}

const HC_WfnBankEntry &ClementiRoettiData::getEntry(
    const std::string &atomType)
const
{
    std::map<std::string, discamb::HC_WfnBankEntry>::const_iterator it;
    it = mEntries.find(atomType);
    if (it == mEntries.end())
        on_error::throwException("Invalid atomType label when attempting to access Clementi-Roetti wavefunction data",
            __FILE__, __LINE__);
    return mEntries.find(atomType)->second;
}

} // namespace discamb
